@@ -31,24 +31,22 @@
 #define UNICODE
 #include <vlc/vlc.h>
 #include <windows.h>
+#include <shellapi.h>
 
-#if !defined(UNDER_CE)
-# ifndef _WIN32_IE
-#   define  _WIN32_IE 0x501
-# endif
-# include <fcntl.h>
-# include <io.h>
-# include <shlobj.h>
-# include <wininet.h>
-# define PSAPI_VERSION 1
-# include <psapi.h>
-# define HeapEnableTerminationOnCorruption (HEAP_INFORMATION_CLASS)1
+#ifndef _WIN32_IE
+#  define  _WIN32_IE 0x501
+#endif
+#include <fcntl.h>
+#include <io.h>
+#include <shlobj.h>
+#include <wininet.h>
+#define PSAPI_VERSION 1
+#include <psapi.h>
+#define HeapEnableTerminationOnCorruption (HEAP_INFORMATION_CLASS)1
 static void check_crashdump(void);
 LONG WINAPI vlc_exception_filter(struct _EXCEPTION_POINTERS *lpExceptionInfo);
 static const wchar_t *crashdump_path;
-#endif
 
-#ifndef UNDER_CE
 static char *FromWide (const wchar_t *wide)
 {
     size_t len;
@@ -59,69 +57,23 @@ static char *FromWide (const wchar_t *wide)
         WideCharToMultiByte (CP_UTF8, 0, wide, -1, out, len, NULL, NULL);
     return out;
 }
-#else
-static int parse_cmdline (char *line, char ***argvp)
-{
-    char **argv = malloc (sizeof (char *));
-    int argc = 0;
-
-    while (*line != '\0')
-    {
-        char quote = 0;
-
-        /* Skips white spaces */
-        while (strchr ("\t ", *line))
-            line++;
-        if (!*line)
-            break;
-
-        /* Starts a new parameter */
-        argv = realloc (argv, (argc + 2) * sizeof (char *));
-        if (*line == '"')
-        {
-            quote = '"';
-            line++;
-        }
-        argv[argc++] = line;
-
-    more:
-            while (*line && !strchr ("\t ", *line))
-            line++;
-
-    if (line > argv[argc - 1] && line[-1] == quote)
-        /* End of quoted parameter */
-        line[-1] = 0;
-    else
-        if (*line && quote)
-    {
-        /* Space within a quote */
-        line++;
-        goto more;
-    }
-    else
-        /* End of unquoted parameter */
-        if (*line)
-            *line++ = 0;
-    }
-    argv[argc] = NULL;
-    *argvp = argv;
-    return argc;
-}
-#endif
 
 int WINAPI WinMain( HINSTANCE hInstance, HINSTANCE hPrevInstance,
-#ifndef UNDER_CE
                     LPSTR lpCmdLine,
-#else
-                    LPWSTR lpCmdLine,
-#endif
                     int nCmdShow )
 {
     int argc;
 
-#ifndef UNDER_CE
+    /* VLC does not change the thread locale, so gettext/libintil will use the
+     * user default locale as reference. */
+    /* gettext versions 0.18-0.18.1 will use the Windows Vista locale name
+     * if the GETTEXT_MUI environment variable is set. If not set or if running
+     * on Windows 2000/XP/2003 an hard-coded language ID list is used. This
+     * putenv() call may become redundant with later versions of gettext. */
+    putenv("GETTEXT_MUI=1");
 #ifdef TOP_BUILDDIR
     putenv("VLC_PLUGIN_PATH=Z:"TOP_BUILDDIR"/modules");
+    putenv("VLC_DATA_PATH=Z:"TOP_SRCDIR"/share");
 #endif
 
     HeapSetInformation(NULL, HeapEnableTerminationOnCorruption, NULL, 0);
@@ -140,7 +92,8 @@ int WINAPI WinMain( HINSTANCE hInstance, HINSTANCE hPrevInstance,
             mySetProcessDEPPolicy(PROCESS_DEP_ENABLE);
 
         /* Do NOT load any library from cwd. */
-        mySetDllDirectoryA = (BOOL WINAPI (*)(const char*)) GetProcAddress(h_Kernel32, "SetDllDirectoryA");
+        mySetDllDirectoryA = (BOOL WINAPI (*)(const char*))
+                            GetProcAddress(h_Kernel32, "SetDllDirectoryA");
         if(mySetDllDirectoryA)
             mySetDllDirectoryA("");
 
@@ -155,12 +108,10 @@ int WINAPI WinMain( HINSTANCE hInstance, HINSTANCE hPrevInstance,
     char *argv[argc + 3];
     BOOL crash_handling = TRUE;
     int j = 0;
+    char *lang = NULL;
 
     argv[j++] = FromWide( L"--media-library" );
     argv[j++] = FromWide( L"--no-ignore-config" );
-#ifdef TOP_SRCDIR
-    argv[j++] = FromWide (L"--data-path=Z:"TOP_SRCDIR"/share");
-#endif
     for (int i = 1; i < argc; i++)
     {
         if(!wcscmp(wargv[i], L"--no-crashdump"))
@@ -168,6 +119,12 @@ int WINAPI WinMain( HINSTANCE hInstance, HINSTANCE hPrevInstance,
             crash_handling = FALSE;
             continue; /* don't give argument to libvlc */
         }
+        if (!wcsncmp(wargv[i], L"--language", 10) )
+        {
+            if (i < argc - 1 && wcsncmp( wargv[i + 1], L"--", 2 ))
+                lang = FromWide (wargv[++i]);
+            continue;
+        }
 
         argv[j++] = FromWide (wargv[i]);
     }
@@ -182,36 +139,58 @@ int WINAPI WinMain( HINSTANCE hInstance, HINSTANCE hPrevInstance,
         if( S_OK != SHGetFolderPathW( NULL, CSIDL_APPDATA | CSIDL_FLAG_CREATE,
                     NULL, SHGFP_TYPE_CURRENT, path ) )
             fprintf( stderr, "Can't open the vlc conf PATH\n" );
-        swprintf( path+wcslen( path ), L"%s", L"\\vlc\\crashdump" );
+        _snwprintf( path+wcslen( path ), MAX_PATH,  L"%s", L"\\vlc\\crashdump" );
         crashdump_path = &path[0];
 
         check_crashdump();
         SetUnhandledExceptionFilter(vlc_exception_filter);
     }
 
-    _setmode( _fileno( stdin ), _O_BINARY ); /* Needed for pipes */
+    _setmode( STDIN_FILENO, _O_BINARY ); /* Needed for pipes */
 
-#else /* UNDER_CE */
-    char **argv, psz_cmdline[wcslen(lpCmdLine) * 4];
+    /* */
+    if (!lang)
+    {
+        HKEY h_key;
+        if( RegOpenKeyEx( HKEY_CURRENT_USER, TEXT("Software\\VideoLAN\\VLC\\"), 0, KEY_READ, &h_key )
+                == ERROR_SUCCESS )
+        {
+            TCHAR szData[256];
+            DWORD len = 256;
+            if( RegQueryValueEx( h_key, TEXT("Lang"), NULL, NULL, (LPBYTE) &szData, &len ) == ERROR_SUCCESS )
+                lang = FromWide( szData );
+        }
+    }
 
-    WideCharToMultiByte( CP_UTF8, 0, lpCmdLine, -1,
-                         psz_cmdline, sizeof (psz_cmdline), NULL, NULL );
-
-    argc = parse_cmdline (psz_cmdline, &argv);
-#endif
+    if (lang && strncmp( lang, "auto", 4 ) )
+    {
+        char tmp[11];
+        snprintf(tmp, 11, "LANG=%s", lang);
+        putenv(tmp);
+    }
+    free(lang);
 
     /* Initialize libvlc */
     libvlc_instance_t *vlc;
     vlc = libvlc_new (argc, (const char **)argv);
     if (vlc != NULL)
     {
+        libvlc_set_app_id (vlc, "org.VideoLAN.VLC", PACKAGE_VERSION,
+                           PACKAGE_NAME);
         libvlc_set_user_agent (vlc, "VLC media player", "VLC/"PACKAGE_VERSION);
+        libvlc_add_intf (vlc, "hotkeys,none");
         libvlc_add_intf (vlc, "globalhotkeys,none");
         libvlc_add_intf (vlc, NULL);
         libvlc_playlist_play (vlc, -1, 0, NULL);
         libvlc_wait (vlc);
         libvlc_release (vlc);
     }
+    else
+        MessageBox (NULL, TEXT("VLC media player could not start.\n"
+                    "Either the command line options were invalid or no plugins were found.\n"),
+                    TEXT("VLC media player"),
+                    MB_OK|MB_ICONERROR);
+
 
     for (int i = 0; i < argc; i++)
         free (argv[i]);
@@ -220,7 +199,6 @@ int WINAPI WinMain( HINSTANCE hInstance, HINSTANCE hPrevInstance,
     return 0;
 }
 
-#if !defined( UNDER_CE )
 /* Crashdumps handling */
 static void check_crashdump(void)
 {
@@ -229,38 +207,45 @@ static void check_crashdump(void)
         return;
     fclose( fd );
 
-    int answer = MessageBox( NULL, L"VLC media player just crashed." \
-    " Do you want to send a bug report to the developers team?",
+    int answer = MessageBox( NULL, L"Ooops: VLC media player just crashed.\n" \
+    "Would you like to send a bug report to the developers team?",
     L"VLC crash reporting", MB_YESNO);
 
     if(answer == IDYES)
     {
-        HINTERNET Hint = InternetOpen(L"VLC Crash Reporter", INTERNET_OPEN_TYPE_PRECONFIG, NULL,NULL,0);
+        HINTERNET Hint = InternetOpen(L"VLC Crash Reporter",
+                INTERNET_OPEN_TYPE_PRECONFIG, NULL,NULL,0);
         if(Hint)
         {
-            HINTERNET ftp = InternetConnect(Hint, L"crash.videolan.org", INTERNET_DEFAULT_FTP_PORT,
-                                            NULL, NULL, INTERNET_SERVICE_FTP, INTERNET_FLAG_PASSIVE, 0);
+            HINTERNET ftp = InternetConnect(Hint, L"crash.videolan.org",
+                        INTERNET_DEFAULT_FTP_PORT, NULL, NULL,
+                        INTERNET_SERVICE_FTP, INTERNET_FLAG_PASSIVE, 0);
             if(ftp)
             {
                 SYSTEMTIME now;
                 GetSystemTime(&now);
                 wchar_t remote_file[MAX_PATH];
-                swprintf( remote_file, L"/crashes-win32/%04d%02d%02d%02d%02d%02d",
-                          now.wYear, now.wMonth, now.wDay, now.wHour, now.wMinute, now.wSecond );
+                _snwprintf(remote_file, MAX_PATH,
+                        L"/crashes-win32/%04d%02d%02d%02d%02d%02d",
+                        now.wYear, now.wMonth, now.wDay, now.wHour,
+                        now.wMinute, now.wSecond );
 
-                if( FtpPutFile( ftp, crashdump_path, remote_file, FTP_TRANSFER_TYPE_BINARY, 0) )
-                    MessageBox( NULL, L"Report sent correctly. Thanks a lot for the help.",
-                                L"Report sent", MB_OK);
+                if( FtpPutFile( ftp, crashdump_path, remote_file,
+                            FTP_TRANSFER_TYPE_BINARY, 0) )
+                    MessageBox( NULL, L"Report sent correctly. Thanks a lot " \
+                                "for the help.", L"Report sent", MB_OK);
                 else
-                    MessageBox( NULL, L"There was an error while transferring to the FTP server. "\
-                                "Thanks a lot for the help anyway.",
+                    MessageBox( NULL, L"There was an error while "\
+                                "transferring the data to the FTP server.\n"\
+                                "Thanks a lot for the help.",
                                 L"Report sending failed", MB_OK);
                 InternetCloseHandle(ftp);
             }
             else
             {
-                MessageBox( NULL, L"There was an error while connecting to the FTP server. "\
-                                "Thanks a lot for the help anyway.",
+                MessageBox( NULL, L"There was an error while connecting to " \
+                                "the FTP server. "\
+                                "Thanks a lot for the help.",
                                 L"Report sending failed", MB_OK);
                 fprintf(stderr,"Can't connect to FTP server 0x%08lu\n",
                         (unsigned long)GetLastError());
@@ -269,7 +254,7 @@ static void check_crashdump(void)
         }
         else
         {
-              MessageBox( NULL, L"There was an error while connecting to Internet. "\
+              MessageBox( NULL, L"There was an error while connecting to the Internet.\n"\
                                 "Thanks a lot for the help anyway.",
                                 L"Report sending failed", MB_OK);
         }
@@ -285,7 +270,8 @@ LONG WINAPI vlc_exception_filter(struct _EXCEPTION_POINTERS *lpExceptionInfo)
 {
     if(IsDebuggerPresent())
     {
-        //If a debugger is present, pass the exception to the debugger with EXCEPTION_CONTINUE_SEARCH
+        //If a debugger is present, pass the exception to the debugger
+        //with EXCEPTION_CONTINUE_SEARCH
         return EXCEPTION_CONTINUE_SEARCH;
     }
     else
@@ -305,22 +291,22 @@ LONG WINAPI vlc_exception_filter(struct _EXCEPTION_POINTERS *lpExceptionInfo)
         osvi.dwOSVersionInfoSize = sizeof( OSVERSIONINFO );
         GetVersionEx( &osvi );
 
-        fwprintf( fd, L"[version]\nOS=%d.%d.%d.%d.%s\nVLC=" VERSION_MESSAGE, osvi.dwMajorVersion,
-                                                               osvi.dwMinorVersion,
-                                                               osvi.dwBuildNumber,
-                                                               osvi.dwPlatformId,
-                                                               osvi.szCSDVersion);
+        fwprintf( fd, L"[version]\nOS=%d.%d.%d.%d.%s\nVLC=" VERSION_MESSAGE,
+                osvi.dwMajorVersion, osvi.dwMinorVersion, osvi.dwBuildNumber,
+                osvi.dwPlatformId, osvi.szCSDVersion);
 
-        const CONTEXT *const pContext = (const CONTEXT *)lpExceptionInfo->ContextRecord;
-        const EXCEPTION_RECORD *const pException = (const EXCEPTION_RECORD *)lpExceptionInfo->ExceptionRecord;
-        /*No nested exceptions for now*/
-        fwprintf( fd, L"\n\n[exceptions]\n%08x at %px",pException->ExceptionCode,
-                                                pException->ExceptionAddress );
+        const CONTEXT *const pContext = (const CONTEXT *)
+            lpExceptionInfo->ContextRecord;
+        const EXCEPTION_RECORD *const pException = (const EXCEPTION_RECORD *)
+            lpExceptionInfo->ExceptionRecord;
+        /* No nested exceptions for now */
+        fwprintf( fd, L"\n\n[exceptions]\n%08x at %px", 
+                pException->ExceptionCode, pException->ExceptionAddress );
 
         for( unsigned int i = 0; i < pException->NumberParameters; i++ )
             fwprintf( fd, L" | %p", pException->ExceptionInformation[i] );
 
-#ifdef WIN64
+#ifdef _WIN64
         fwprintf( fd, L"\n\n[context]\nRDI:%px\nRSI:%px\n" \
                     "RBX:%px\nRDX:%px\nRCX:%px\nRAX:%px\n" \
                     "RBP:%px\nRIP:%px\nRSP:%px\nR8:%px\n" \
@@ -343,7 +329,7 @@ LONG WINAPI vlc_exception_filter(struct _EXCEPTION_POINTERS *lpExceptionInfo)
 
         fwprintf( fd, L"\n[stacktrace]\n#EIP|base|module\n" );
 
-#ifdef WIN64
+#ifdef _WIN64
         LPCVOID caller = (LPCVOID)pContext->Rip;
         LPVOID *pBase  = (LPVOID*)pContext->Rbp;
 #else
@@ -386,4 +372,3 @@ LONG WINAPI vlc_exception_filter(struct _EXCEPTION_POINTERS *lpExceptionInfo)
         exit( 1 );
     }
 }
-#endif
