@@ -2,7 +2,7 @@
  * vlc_config_cat.h : Definition of configuration categories
  *****************************************************************************
  * Copyright (C) 2003 VLC authors and VideoLAN
- * $Id: 9626943a04ec305e675660c8fb4ebdfe79942fa7 $
+ * $Id: 7cb796b4794b98f1267747d590cdc3922371f742 $
  *
  * Authors: Clément Stenac <zorglub@videolan.org>
  *          Anil Daoud <anil@videolan.org>
@@ -24,6 +24,7 @@
 
 #ifndef VLC_HELP_H
 #define VLC_HELP_H 1
+# include <vlc_plugin.h>
 
 /*
  *  First, we need help strings for the General Settings and for the
@@ -80,11 +81,10 @@
 #define VFILTER_HELP N_( \
     "Video filters are used to process the video stream." )
 
-#define SUBPIC_TITLE N_( "Subtitles/OSD")
+#define SUBPIC_TITLE N_( "Subtitles / OSD")
 #define SUBPIC_HELP N_( "Settings related to On-Screen-Display,"\
         " subtitles and \"overlay subpictures\"")
 /*
-#define TEXT_TITLE N_("Text rendering")
 #define TEXT_HELP N_( \
     "Use the settings of the \"freetype\" module to choose the font you " \
     "want VLC to use for text rendering (to display subtitles for example).")
@@ -114,10 +114,9 @@
 #define ADEC_TITLE  N_( "Audio codecs" )
 #define ADEC_HELP N_( "Settings for the audio-only decoders and encoders." )
 
-#define SDEC_TITLE N_( "Subtitles codecs")
-#define SDEC_HELP N_( "Settings for subtitles, teletext and CC decoders and encoders." )
+#define SDEC_TITLE N_( "Subtitle codecs")
+#define SDEC_HELP N_( "Settings for subtitle, teletext and CC decoders and encoders." )
 
-#define ADVANCED_TITLE N_("General Input" )
 #define ADVANCED_HELP N_( "General input settings. Use with care..." )
 
 /* Sout */
@@ -162,11 +161,6 @@
                 "more information. You can configure default options for " \
                 "each sout stream module here.")
 
-#define SOUT_SAP_TITLE N_( "SAP" )
-#define SOUT_SAP_HELP N_( \
-                 "SAP is a way to publically announce streams that are being "\
-                 "sent using multicast UDP or RTP." )
-
 #define SOUT_VOD_TITLE N_( "VOD" )
 #define SOUT_VOD_HELP N_( "VLC's implementation of Video On Demand" )
 
@@ -186,10 +180,6 @@
 #define AADVANCED_TITLE N_( "Advanced" )
 #define AADVANCED_HELP N_( "Advanced settings. Use with care...")
 
-#define CPU_TITLE N_( "CPU features" )
-#define CPU_HELP N_( "You can choose to disable some CPU accelerations " \
-        "here. Use with extreme care!" )
-
 #define MISC_TITLE N_( "Advanced settings" )
 
 
@@ -199,33 +189,23 @@
 #define NETWORK_HELP N_( "These modules provide network functions to all " \
                 "other parts of VLC." )
 
-#define CHROMA_TITLE N_("Chroma modules settings")
-#define CHROMA_HELP N_("These settings affect chroma transformation modules.")
-
-#define PACKETIZER_TITLE  N_("Packetizer modules settings" )
 #define PACKETIZER_HELP "These are general settings for the "\
         "packetizers used in VLC's stream output subsystem."
 
-#define ENCODER_TITLE N_("Encoders settings")
 #define ENCODER_HELP N_( \
-    "These are general settings for video/audio/subtitles encoding modules.")
+    "These are general settings for video/audio/subtitle encoding modules.")
 
-
-#define DIALOGS_TITLE N_("Dialog providers settings")
 #define DIALOGS_HELP  N_( \
     "Dialog providers can be configured here.")
 
-#define SUBTITLE_DEMUX_TITLE N_("Subtitle demuxer settings")
 #define SUBTITLE_DEMUX_HELP N_( \
     "In this section you can force the behavior of the subtitle demuxer, " \
-    "for example by setting the subtitles type or file name.")
+    "for example by setting the subtitle type or file name.")
 
 /*
  *  A little help for modules with unknown capabilities
  */
 
-#define UNKNOWN_TITLE N_("No help available" )
-#define UNKNOWN_HELP N_("There is no help available for these modules.")
 #endif
 
 /* This function is deprecated and is kept only for compatibility */
@@ -266,7 +246,6 @@ static const struct config_category_t categories_array[] =
     { SUBCAT_SOUT_MUX, SOUT_MUX_TITLE, SOUT_MUX_HELP },
     { SUBCAT_SOUT_ACO, SOUT_ACO_TITLE, SOUT_ACO_HELP },
     { SUBCAT_SOUT_PACKETIZER, SOUT_PACKET_TITLE, SOUT_PACKET_HELP },
-    { SUBCAT_SOUT_SAP, SOUT_SAP_TITLE, SOUT_SAP_HELP },
     { SUBCAT_SOUT_VOD, SOUT_VOD_TITLE, SOUT_VOD_HELP },
 
     { CAT_PLAYLIST, PLAYLIST_TITLE , PLAYLIST_HELP },
@@ -274,7 +253,6 @@ static const struct config_category_t categories_array[] =
     { SUBCAT_PLAYLIST_SD, SD_TITLE, SD_HELP },
 
     { CAT_ADVANCED, AADVANCED_TITLE, AADVANCED_HELP },
-    { SUBCAT_ADVANCED_CPU, CPU_TITLE, CPU_HELP },
     { SUBCAT_ADVANCED_MISC, MISC_TITLE, AADVANCED_HELP },
 
     { -1, NULL, NULL }
