@@ -1,26 +1,34 @@
 /*****************************************************************************
  * dvdnav.c: DVD module using the dvdnav library.
  *****************************************************************************
- * Copyright (C) 2004-2009 the VideoLAN team
- * $Id: bf9bfece36619688d90eebafaa53e8d0b6495ced $
+ * Copyright (C) 2004-2009 VLC authors and VideoLAN
+ * $Id: 13aaa3d5bda6f374bbc344c964b585b398db1549 $
  *
  * Authors: Laurent Aimar <fenrir@via.ecp.fr>
  *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * This program is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU Lesser General Public License as published by
+ * the Free Software Foundation; either version 2.1 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+ * GNU Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License along
- * with this program; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin Street, Fifth Floor, Boston MA 02110-1301, USA.
+ * You should have received a copy of the GNU Lesser General Public License
+ * along with this program; if not, write to the Free Software Foundation,
+ * Inc., 51 Franklin Street, Fifth Floor, Boston MA 02110-1301, USA.
  *****************************************************************************/
 
+/*****************************************************************************
+ * NOTA BENE: this module requires the linking against a library which is
+ * known to require licensing under the GNU General Public License version 2
+ * (or later). Therefore, the result of compiling this module will normally
+ * be subject to the terms of that later license.
+ *****************************************************************************/
+
+
 /*****************************************************************************
  * Preamble
  *****************************************************************************/
@@ -30,6 +38,14 @@
 #endif
 
 #include <assert.h>
+#ifdef HAVE_UNISTD_H
+#   include <unistd.h>
+#endif
+#include <sys/types.h>
+#include <sys/stat.h>
+#include <fcntl.h>
+#include <errno.h>
+
 #include <vlc_common.h>
 #include <vlc_plugin.h>
 #include <vlc_input.h>
@@ -39,22 +55,7 @@
 #include <vlc_fs.h>
 #include <vlc_url.h>
 #include <vlc_vout.h>
-
 #include <vlc_dialog.h>
-
-#ifdef HAVE_UNISTD_H
-#   include <unistd.h>
-#endif
-#include <sys/types.h>
-#ifdef HAVE_SYS_STAT_H
-#   include <sys/stat.h>
-#endif
-#ifdef HAVE_FCNTL_H
-#   include <fcntl.h>
-#endif
-#include <errno.h>
-
-#include <vlc_keys.h>
 #include <vlc_iso_lang.h>
 
 /* FIXME we should find a better way than including that */
@@ -170,8 +171,6 @@ static int ControlInternal( demux_t *, int, ... );
 
 static void StillTimer( void * );
 
-static int EventKey( vlc_object_t *, char const *,
-                     vlc_value_t, vlc_value_t, void * );
 static int EventMouse( vlc_object_t *, char const *,
                        vlc_value_t, vlc_value_t, void * );
 static int EventIntf( vlc_object_t *, char const *,
@@ -205,7 +204,7 @@ static int Open( vlc_object_t *p_this )
     else
         psz_file = strdup( p_demux->psz_file );
 
-#if defined( WIN32 ) || defined( __OS2__ )
+#if defined( _WIN32 ) || defined( __OS2__ )
     if( psz_file != NULL )
     {
         /* Remove trailing backslash, otherwise dvdnav_open will fail */
@@ -364,8 +363,6 @@ static int Open( vlc_object_t *p_this )
     var_Create( p_sys->p_input, "menu-palette", VLC_VAR_ADDRESS );
     var_Create( p_sys->p_input, "highlight", VLC_VAR_BOOL );
 
-    /* catch all key event */
-    var_AddCallback( p_demux->p_libvlc, "key-action", EventKey, p_demux );
     /* catch vout creation event */
     var_AddCallback( p_sys->p_input, "intf-event", EventIntf, p_demux );
 
@@ -394,9 +391,6 @@ static void Close( vlc_object_t *p_this )
         var_DelCallback( p_sys->p_vout, "mouse-clicked", EventMouse, p_demux );
     }
 
-    /* Stop key event handler (FIXME: should really be per-vout too) */
-    var_DelCallback( p_demux->p_libvlc, "key-action", EventKey, p_demux );
-
     /* Stop still image handler */
     if( p_sys->still.b_created )
         vlc_timer_destroy( p_sys->still.timer );
@@ -576,10 +570,53 @@ static int Control( demux_t *p_demux, int i_query, va_list args )
             return VLC_EGENERIC;
         }
 
+        case DEMUX_NAV_ACTIVATE:
+        {
+            pci_t *pci = dvdnav_get_current_nav_pci( p_sys->dvdnav );
+
+            ButtonUpdate( p_demux, true );
+            dvdnav_button_activate( p_sys->dvdnav, pci );
+            break;
+        }
+
+        case DEMUX_NAV_UP:
+        {
+            pci_t *pci = dvdnav_get_current_nav_pci( p_sys->dvdnav );
+
+            dvdnav_upper_button_select( p_sys->dvdnav, pci );
+            break;
+        }
+
+        case DEMUX_NAV_DOWN:
+        {
+            pci_t *pci = dvdnav_get_current_nav_pci( p_sys->dvdnav );
+
+            dvdnav_lower_button_select( p_sys->dvdnav, pci );
+            break;
+        }
+
+        case DEMUX_NAV_LEFT:
+        {
+            pci_t *pci = dvdnav_get_current_nav_pci( p_sys->dvdnav );
+
+            dvdnav_left_button_select( p_sys->dvdnav, pci );
+            break;
+        }
+
+        case DEMUX_NAV_RIGHT:
+        {
+            pci_t *pci = dvdnav_get_current_nav_pci( p_sys->dvdnav );
+
+            dvdnav_right_button_select( p_sys->dvdnav, pci );
+            break;
+        }
+
         /* TODO implement others */
         default:
             return VLC_EGENERIC;
     }
+
+    return VLC_SUCCESS;
 }
 
 static int ControlInternal( demux_t *p_demux, int i_query, ... )
@@ -819,12 +856,17 @@ static int Demux( demux_t *p_demux )
         if( dvdnav_current_title_info( p_sys->dvdnav, &i_title,
                                        &i_part ) == DVDNAV_STATUS_OK )
         {
-            if( i_title >= 0 && i_title < p_sys->i_title &&
-                i_part >= 1 && i_part <= p_sys->title[i_title]->i_seekpoint &&
-                p_demux->info.i_seekpoint != i_part - 1 )
+            if( i_title >= 0 && i_title < p_sys->i_title )
             {
-                p_demux->info.i_update |= INPUT_UPDATE_SEEKPOINT;
-                p_demux->info.i_seekpoint = i_part - 1;
+                p_demux->info.i_update |= INPUT_UPDATE_TITLE;
+                p_demux->info.i_title = i_title;
+
+                if( i_part >= 1 && i_part <= p_sys->title[i_title]->i_seekpoint &&
+                        p_demux->info.i_seekpoint != i_part - 1 )
+                {
+                    p_demux->info.i_update |= INPUT_UPDATE_SEEKPOINT;
+                    p_demux->info.i_seekpoint = i_part - 1;
+                }
             }
         }
         break;
@@ -941,7 +983,6 @@ static char *DemuxGetLanguageCode( demux_t *p_demux, const char *psz_var )
         if( *psz_lang == '\0' )
             continue;
         if( !strcasecmp( pl->psz_eng_name, psz_lang ) ||
-            !strcasecmp( pl->psz_native_name, psz_lang ) ||
             !strcasecmp( pl->psz_iso639_1, psz_lang ) ||
             !strcasecmp( pl->psz_iso639_2T, psz_lang ) ||
             !strcasecmp( pl->psz_iso639_2B, psz_lang ) )
@@ -1156,7 +1197,7 @@ static int DemuxBlock( demux_t *p_demux, const uint8_t *p, int len )
         }
 
         /* Create a block */
-        block_t *p_pkt = block_New( p_demux, i_size );
+        block_t *p_pkt = block_Alloc( i_size );
         memcpy( p_pkt->p_buffer, p, i_size);
 
         /* Parse it and send it */
@@ -1379,39 +1420,6 @@ static int EventMouse( vlc_object_t *p_vout, char const *psz_var,
     return VLC_SUCCESS;
 }
 
-static int EventKey( vlc_object_t *p_this, char const *psz_var,
-                     vlc_value_t oldval, vlc_value_t newval, void *p_data )
-{
-    demux_t *p_demux = p_data;
-    demux_sys_t *p_sys = p_demux->p_sys;
-
-    /* FIXME: thread-safe ? */
-    pci_t *pci = dvdnav_get_current_nav_pci( p_sys->dvdnav );
-
-    switch( newval.i_int )
-    {
-    case ACTIONID_NAV_LEFT:
-        dvdnav_left_button_select( p_sys->dvdnav, pci );
-        break;
-    case ACTIONID_NAV_RIGHT:
-        dvdnav_right_button_select( p_sys->dvdnav, pci );
-        break;
-    case ACTIONID_NAV_UP:
-        dvdnav_upper_button_select( p_sys->dvdnav, pci );
-        break;
-    case ACTIONID_NAV_DOWN:
-        dvdnav_lower_button_select( p_sys->dvdnav, pci );
-        break;
-    case ACTIONID_NAV_ACTIVATE:
-        ButtonUpdate( p_demux, true );
-        dvdnav_button_activate( p_sys->dvdnav, pci );
-        break;
-    }
-
-    (void)p_this;    (void)psz_var;    (void)oldval;
-    return VLC_SUCCESS;
-}
-
 static int EventIntf( vlc_object_t *p_input, char const *psz_var,
                       vlc_value_t oldval, vlc_value_t val, void *p_data )
 {
@@ -1456,28 +1464,22 @@ static int ProbeDVD( const char *psz_name )
 #endif
 
     int ret = VLC_EGENERIC;
-
-#ifdef HAVE_SYS_STAT_H
     struct stat stat_info;
 
     if( fstat( fd, &stat_info ) == -1 )
          goto bailout;
-
     if( !S_ISREG( stat_info.st_mode ) )
     {
         if( S_ISDIR( stat_info.st_mode ) || S_ISBLK( stat_info.st_mode ) )
             ret = VLC_SUCCESS; /* Let dvdnav_open() do the probing */
         goto bailout;
     }
-#endif
-    /* Match extension as the anchor exhibits too many false positives */
-    const char *ext = strrchr( psz_name, '.' );
-    if( ext == NULL )
-        goto bailout;
-    ext++;
-    if( strcasecmp( ext, "iso" ) && strcasecmp( ext, "img" ) &&
-        strcasecmp( ext, "mdf" ) && strcasecmp( ext, "dvd" ) &&
-        strcasecmp( ext, "bin" ) && strcasecmp( ext, "nrg" ) )
+
+    /* ISO 9660 volume descriptor */
+    char iso_dsc[6];
+    if( lseek( fd, 0x8000 + 1, SEEK_SET ) == -1
+     || read( fd, iso_dsc, sizeof (iso_dsc) ) < sizeof (iso_dsc)
+     || memcmp( iso_dsc, "CD001\x01", 6 ) )
         goto bailout;
 
     /* Try to find the anchor (2 bytes at LBA 256) */
@@ -1487,7 +1489,6 @@ static int ProbeDVD( const char *psz_name )
      && read( fd, &anchor, 2 ) == 2
      && GetWLE( &anchor ) == 2 )
         ret = VLC_SUCCESS; /* Found a potential anchor */
-
 bailout:
     close( fd );
     return ret;
