@@ -5,19 +5,19 @@
  *
  * Authors: Christophe Massiot <massiot@via.ecp.fr>
  *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * This program is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU Lesser General Public License as published by
+ * the Free Software Foundation; either version 2.1 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+ * GNU Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
+ * You should have received a copy of the GNU Lesser General Public License
+ * along with this program; if not, write to the Free Software Foundation,
+ * Inc., 51 Franklin Street, Fifth Floor, Boston MA 02110-1301, USA.
  *****************************************************************************/
 
 #ifdef HAVE_CONFIG_H
@@ -518,8 +518,8 @@ static int InitAudio( demux_t *p_demux )
         es_format_Init( &fmt, AUDIO_ES, VLC_FOURCC('a','r','a','w') );
         fmt.i_id = p_audio->i_id;
         fmt.audio.i_channels          = 2;
-        fmt.audio.i_physical_channels = 6;
-        fmt.audio.i_original_channels = 6;
+        fmt.audio.i_original_channels =
+        fmt.audio.i_physical_channels = AOUT_CHANS_STEREO;
         fmt.audio.i_rate              = p_sys->i_sample_rate;
         fmt.audio.i_bitspersample     = 16;
         fmt.audio.i_blockalign = fmt.audio.i_channels *
@@ -539,7 +539,7 @@ static int InitAudio( demux_t *p_demux )
 static int HandleVideo( demux_t *p_demux, const uint8_t *p_buffer )
 {
     demux_sys_t *p_sys = p_demux->p_sys;
-    block_t *p_current_picture = block_New( p_demux, p_sys->i_vblock_size );
+    block_t *p_current_picture = block_Alloc( p_sys->i_vblock_size );
     if( unlikely( !p_current_picture ) )
         return VLC_ENOMEM;
     uint8_t *p_y = p_current_picture->p_buffer;
@@ -609,7 +609,7 @@ static int HandleAudio( demux_t *p_demux, const uint8_t *p_buffer )
         hdsdi_audio_t *p_audio = &p_sys->p_audios[i];
         if ( p_audio->i_channel != -1 && p_audio->p_es != NULL )
         {
-            block_t *p_block = block_New( p_demux, p_sys->i_ablock_size );
+            block_t *p_block = block_Alloc( p_sys->i_ablock_size );
             if( unlikely( !p_block ) )
                 return VLC_ENOMEM;
             SparseCopy( (int16_t *)p_block->p_buffer, (const int16_t *)p_buffer,
@@ -669,7 +669,7 @@ static int InitCapture( demux_t *p_demux )
     }
 
     /* Wait for standard to settle down */
-    while ( !p_demux->b_die )
+    while ( vlc_object_alive(p_demux) )
     {
         struct pollfd pfd[1];
 
@@ -707,7 +707,7 @@ static int InitCapture( demux_t *p_demux )
             }
         }
     }
-    if ( p_demux->b_die )
+    if ( !vlc_object_alive(p_demux) )
     {
         close( p_sys->i_vfd );
         return VLC_EGENERIC;
