@@ -1,8 +1,8 @@
 /*****************************************************************************
  * dts.c: parse DTS audio sync info and packetize the stream
  *****************************************************************************
- * Copyright (C) 2003-2009 the VideoLAN team
- * $Id: 50240ea3cb8ed36c3250e8db7e8ab646f39fe8b4 $
+ * Copyright (C) 2003-2009 VLC authors and VideoLAN
+ * $Id: 09ebc0d75727cb402aa26df3cefc97a58785d9bb $
  *
  * Authors: Jon Lech Johansen <jon-vl@nanocrew.net>
  *          Gildas Bazin <gbazin@netcourrier.com>
@@ -34,13 +34,13 @@
 #include <vlc_common.h>
 #include <vlc_plugin.h>
 #include <vlc_codec.h>
-#include <vlc_aout.h>
 #include <vlc_block_helper.h>
 #include <vlc_bits.h>
 #include <vlc_modules.h>
 #include <vlc_cpu.h>
 
 #include "../packetizer/packetizer_helper.h"
+#include "dts_header.h"
 
 /*****************************************************************************
  * Module descriptor
@@ -91,21 +91,18 @@ struct decoder_sys_t
     unsigned int i_channels_conf;
 };
 
-#define DTS_HEADER_SIZE 14
-
 /****************************************************************************
  * Local prototypes
  ****************************************************************************/
 static int OpenCommon( vlc_object_t *, bool b_packetizer );
 static block_t *DecodeBlock( decoder_t *, block_t ** );
 
-static inline int SyncCode( const uint8_t * );
 static int  SyncInfo( const uint8_t *, bool *, unsigned int *, unsigned int *,
                       unsigned int *, unsigned int *, unsigned int * );
 
-static uint8_t       *GetOutBuffer ( decoder_t *, block_t ** );
-static aout_buffer_t *GetAoutBuffer( decoder_t * );
-static block_t       *GetSoutBuffer( decoder_t * );
+static uint8_t *GetOutBuffer ( decoder_t *, block_t ** );
+static block_t *GetAoutBuffer( decoder_t * );
+static block_t *GetSoutBuffer( decoder_t * );
 
 /*****************************************************************************
  * OpenDecoder: probe the decoder
@@ -113,7 +110,7 @@ static block_t       *GetSoutBuffer( decoder_t * );
 static int OpenDecoder( vlc_object_t *p_this )
 {
     /* HACK: Don't use this codec if we don't have an dts audio filter */
-    if( !HAVE_FPU || !module_exists( "dtstofloat32" ) )
+    if( !module_exists( "dtstofloat32" ) )
         return VLC_EGENERIC;
 
     return OpenCommon( p_this, false );
@@ -389,7 +386,7 @@ static uint8_t *GetOutBuffer( decoder_t *p_dec, block_t **pp_out_buffer )
     }
     else
     {
-        aout_buffer_t *p_aout_buffer = GetAoutBuffer( p_dec );
+        block_t *p_aout_buffer = GetAoutBuffer( p_dec );
         p_buf = p_aout_buffer ? p_aout_buffer->p_buffer : NULL;
         *pp_out_buffer = p_aout_buffer;
     }
@@ -400,15 +397,16 @@ static uint8_t *GetOutBuffer( decoder_t *p_dec, block_t **pp_out_buffer )
 /*****************************************************************************
  * GetAoutBuffer:
  *****************************************************************************/
-static aout_buffer_t *GetAoutBuffer( decoder_t *p_dec )
+static block_t *GetAoutBuffer( decoder_t *p_dec )
 {
     decoder_sys_t *p_sys = p_dec->p_sys;
-    aout_buffer_t *p_buf;
+    block_t *p_buf;
 
     /* Hack for DTS S/PDIF filter which needs to send 3 frames at a time
      * (plus a few header bytes) */
     p_buf = decoder_NewAudioBuffer( p_dec, p_sys->i_frame_length * 4 );
     if( p_buf == NULL ) return NULL;
+
     p_buf->i_nb_samples = p_sys->i_frame_length;
     p_buf->i_buffer = p_sys->i_frame_size;
 
@@ -427,7 +425,7 @@ static block_t *GetSoutBuffer( decoder_t *p_dec )
     decoder_sys_t *p_sys = p_dec->p_sys;
     block_t *p_block;
 
-    p_block = block_New( p_dec, p_sys->i_frame_size );
+    p_block = block_Alloc( p_sys->i_frame_size );
     if( p_block == NULL ) return NULL;
 
     p_block->i_pts = p_block->i_dts = date_Get( &p_sys->end_date );
@@ -457,124 +455,6 @@ static const unsigned int ppi_dts_bitrate[] =
     2048000, 3072000, 3840000, 1/*open*/, 2/*variable*/, 3/*lossless*/
 };
 
-static int SyncInfo16be( const uint8_t *p_buf,
-                         unsigned int *pi_audio_mode,
-                         unsigned int *pi_sample_rate,
-                         unsigned int *pi_bit_rate,
-                         unsigned int *pi_frame_length )
-{
-    unsigned int i_frame_size;
-    unsigned int i_lfe;
-
-    *pi_frame_length = (p_buf[4] & 0x01) << 6 | (p_buf[5] >> 2);
-    i_frame_size = (p_buf[5] & 0x03) << 12 | (p_buf[6] << 4) |
-                   (p_buf[7] >> 4);
-
-    *pi_audio_mode = (p_buf[7] & 0x0f) << 2 | (p_buf[8] >> 6);
-    *pi_sample_rate = (p_buf[8] >> 2) & 0x0f;
-    *pi_bit_rate = (p_buf[8] & 0x03) << 3 | ((p_buf[9] >> 5) & 0x07);
-
-    i_lfe = (p_buf[10] >> 1) & 0x03;
-    if( i_lfe ) *pi_audio_mode |= 0x10000;
-
-    return i_frame_size + 1;
-}
-
-static void BufLeToBe( uint8_t *p_out, const uint8_t *p_in, int i_in )
-{
-    int i;
-
-    for( i = 0; i < i_in/2; i++  )
-    {
-        p_out[i*2] = p_in[i*2+1];
-        p_out[i*2+1] = p_in[i*2];
-    }
-}
-
-static int Buf14To16( uint8_t *p_out, const uint8_t *p_in, int i_in, int i_le )
-{
-    unsigned char tmp, cur = 0;
-    int bits_in, bits_out = 0;
-    int i, i_out = 0;
-
-    for( i = 0; i < i_in; i++  )
-    {
-        if( i%2 )
-        {
-            tmp = p_in[i-i_le];
-            bits_in = 8;
-        }
-        else
-        {
-            tmp = p_in[i+i_le] & 0x3F;
-            bits_in = 8 - 2;
-        }
-
-        if( bits_out < 8 )
-        {
-            int need = __MIN( 8 - bits_out, bits_in );
-            cur <<= need;
-            cur |= ( tmp >> (bits_in - need) );
-            tmp <<= (8 - bits_in + need);
-            tmp >>= (8 - bits_in + need);
-            bits_in -= need;
-            bits_out += need;
-        }
-
-        if( bits_out == 8 )
-        {
-            p_out[i_out] = cur;
-            cur = 0;
-            bits_out = 0;
-            i_out++;
-        }
-
-        bits_out += bits_in;
-        cur <<= bits_in;
-        cur |= tmp;
-    }
-
-    return i_out;
-}
-
-static inline int SyncCode( const uint8_t *p_buf )
-{
-    /* 14 bits, little endian version of the bitstream */
-    if( p_buf[0] == 0xff && p_buf[1] == 0x1f &&
-        p_buf[2] == 0x00 && p_buf[3] == 0xe8 &&
-        (p_buf[4] & 0xf0) == 0xf0 && p_buf[5] == 0x07 )
-    {
-        return VLC_SUCCESS;
-    }
-    /* 14 bits, big endian version of the bitstream */
-    else if( p_buf[0] == 0x1f && p_buf[1] == 0xff &&
-             p_buf[2] == 0xe8 && p_buf[3] == 0x00 &&
-             p_buf[4] == 0x07 && (p_buf[5] & 0xf0) == 0xf0 )
-    {
-        return VLC_SUCCESS;
-    }
-    /* 16 bits, big endian version of the bitstream */
-    else if( p_buf[0] == 0x7f && p_buf[1] == 0xfe &&
-             p_buf[2] == 0x80 && p_buf[3] == 0x01 )
-    {
-        return VLC_SUCCESS;
-    }
-    /* 16 bits, little endian version of the bitstream */
-    else if( p_buf[0] == 0xfe && p_buf[1] == 0x7f &&
-             p_buf[2] == 0x01 && p_buf[3] == 0x80 )
-    {
-        return VLC_SUCCESS;
-    }
-    /* DTS-HD */
-    else if( p_buf[0] == 0x64 && p_buf[1] ==  0x58 &&
-             p_buf[2] == 0x20 && p_buf[3] ==  0x25 )
-    {
-        return VLC_SUCCESS;
-    }
-
-    return VLC_EGENERIC;
-}
-
 static int SyncInfo( const uint8_t *p_buf,
                      bool *pb_dts_hd,
                      unsigned int *pi_channels,
@@ -584,84 +464,12 @@ static int SyncInfo( const uint8_t *p_buf,
                      unsigned int *pi_frame_length )
 {
     unsigned int i_audio_mode;
-    unsigned int i_frame_size;
 
-    /* 14 bits, little endian version of the bitstream */
-    if( p_buf[0] == 0xff && p_buf[1] == 0x1f &&
-        p_buf[2] == 0x00 && p_buf[3] == 0xe8 &&
-        (p_buf[4] & 0xf0) == 0xf0 && p_buf[5] == 0x07 )
-    {
-        uint8_t conv_buf[DTS_HEADER_SIZE];
-        Buf14To16( conv_buf, p_buf, DTS_HEADER_SIZE, 1 );
-        i_frame_size = SyncInfo16be( conv_buf, &i_audio_mode, pi_sample_rate,
-                                     pi_bit_rate, pi_frame_length );
-        i_frame_size = i_frame_size * 8 / 14 * 2;
-    }
-    /* 14 bits, big endian version of the bitstream */
-    else if( p_buf[0] == 0x1f && p_buf[1] == 0xff &&
-             p_buf[2] == 0xe8 && p_buf[3] == 0x00 &&
-             p_buf[4] == 0x07 && (p_buf[5] & 0xf0) == 0xf0 )
-    {
-        uint8_t conv_buf[DTS_HEADER_SIZE];
-        Buf14To16( conv_buf, p_buf, DTS_HEADER_SIZE, 0 );
-        i_frame_size = SyncInfo16be( conv_buf, &i_audio_mode, pi_sample_rate,
-                                     pi_bit_rate, pi_frame_length );
-        i_frame_size = i_frame_size * 8 / 14 * 2;
-    }
-    /* 16 bits, big endian version of the bitstream */
-    else if( p_buf[0] == 0x7f && p_buf[1] == 0xfe &&
-             p_buf[2] == 0x80 && p_buf[3] == 0x01 )
-    {
-        i_frame_size = SyncInfo16be( p_buf, &i_audio_mode, pi_sample_rate,
-                                     pi_bit_rate, pi_frame_length );
-    }
-    /* 16 bits, little endian version of the bitstream */
-    else if( p_buf[0] == 0xfe && p_buf[1] == 0x7f &&
-             p_buf[2] == 0x01 && p_buf[3] == 0x80 )
-    {
-        uint8_t conv_buf[DTS_HEADER_SIZE];
-        BufLeToBe( conv_buf, p_buf, DTS_HEADER_SIZE );
-        i_frame_size = SyncInfo16be( p_buf, &i_audio_mode, pi_sample_rate,
-                                     pi_bit_rate, pi_frame_length );
-    }
-    /* DTS-HD */
-    else
-    {
-        assert( p_buf[0] == 0x64 && p_buf[1] ==  0x58 &&
-                p_buf[2] == 0x20 && p_buf[3] ==  0x25 );
+    unsigned int i_frame_size = GetSyncInfo( p_buf, pb_dts_hd,
+            pi_sample_rate, pi_bit_rate, pi_frame_length, &i_audio_mode);
 
-        int i_dts_hd_size;
-        bs_t s;
-        bs_init( &s, &p_buf[4], DTS_HEADER_SIZE - 4 );
-
-        bs_skip( &s, 8 + 2 );
-
-        if( bs_read1( &s ) )
-        {
-            bs_skip( &s, 12 );
-            i_dts_hd_size = bs_read( &s, 20 ) + 1;
-        }
-        else
-        {
-            bs_skip( &s, 8 );
-            i_dts_hd_size = bs_read( &s, 16 ) + 1;
-        }
-        //uint16_t s0 = bs_read( &s, 16 );
-        //uint16_t s1 = bs_read( &s, 16 );
-        //fprintf( stderr, "DTS HD=%d : %x %x\n", i_dts_hd_size, s0, s1 );
-
-        *pb_dts_hd = true;
-        /* As we ignore the stream, do not modify those variables:
-        *pi_channels = ;
-        *pi_channels_conf = ;
-        *pi_sample_rate = ;
-        *pi_bit_rate = ;
-        *pi_frame_length = ;
-        */
-        return i_dts_hd_size;
-    }
-
-    *pb_dts_hd = false;
+    if( *pb_dts_hd == true )
+        return i_frame_size;
 
     switch( i_audio_mode & 0xFFFF )
     {
