@@ -1,11 +1,11 @@
 /*****************************************************************************
- * dbus-player.h : dbus control module (mpris v1.0) - /Player object
+ * dbus_player.c : dbus control module (mpris v1.0) - /Player object
  *****************************************************************************
  * Copyright © 2006-2011 Rafaël Carré
  * Copyright © 2007-2011 Mirsal Ennaime
  * Copyright © 2009-2011 The VideoLAN team
  * Copyright © 2013      Alex Merry
- * $Id: bd3b93cb8cef796d81a2ce1a4c2801eee314f8b6 $
+ * $Id: 3d37e6a21bf67bd6ed88425de2c6104edb5f6d81 $
  *
  * Authors:    Mirsal Ennaime <mirsal at mirsal fr>
  *             Rafaël Carré <funman at videolanorg>
@@ -33,26 +33,19 @@
 #include <vlc_common.h>
 #include <vlc_playlist.h>
 #include <vlc_interface.h>
-#include <vlc_aout_intf.h>
 
 #include <math.h>
 
 #include "dbus_player.h"
 #include "dbus_common.h"
 
-static void MarshalLoopStatus ( intf_thread_t *, DBusMessageIter * );
-
-DBUS_METHOD( Position )
-{ /* returns position in microseconds */
-    REPLY_INIT;
-    OUT_ARGUMENTS;
-    DBusMessageIter v;
+static int
+MarshalPosition( intf_thread_t *p_intf, DBusMessageIter *container )
+{
+    /* returns position in microseconds */
     dbus_int64_t i_pos;
-
-    if( !dbus_message_iter_open_container( &args, DBUS_TYPE_VARIANT, "x", &v ) )
-        return DBUS_HANDLER_RESULT_NEED_MEMORY;
-
-    input_thread_t *p_input = playlist_CurrentInput( PL );
+    input_thread_t *p_input;
+    p_input = playlist_CurrentInput( p_intf->p_sys->p_playlist );
 
     if( !p_input )
         i_pos = 0;
@@ -63,13 +56,10 @@ DBUS_METHOD( Position )
         vlc_object_release( p_input );
     }
 
-    if( !dbus_message_iter_append_basic( &v, DBUS_TYPE_INT64, &i_pos ) )
-        return DBUS_HANDLER_RESULT_NEED_MEMORY;
+    if( !dbus_message_iter_append_basic( container, DBUS_TYPE_INT64, &i_pos ) )
+        return VLC_ENOMEM;
 
-    if( !dbus_message_iter_close_container( &args, &v ) )
-        return DBUS_HANDLER_RESULT_NEED_MEMORY;
-
-    REPLY_SEND;
+    return VLC_SUCCESS;
 }
 
 DBUS_METHOD( SetPosition )
@@ -166,33 +156,19 @@ DBUS_METHOD( Seek )
     REPLY_SEND;
 }
 
-static void
+static int
 MarshalVolume( intf_thread_t *p_intf, DBusMessageIter *container )
 {
-    audio_volume_t i_vol = aout_VolumeGet( p_intf->p_sys->p_playlist );
+    float f_vol = playlist_VolumeGet( p_intf->p_sys->p_playlist );
+    if( f_vol < 0.f )
+        f_vol = 1.f; /* ? */
 
-    /* A volume of 1.0 represents a sensible maximum, ie: 0dB */
-    double d_vol = (double) i_vol / AOUT_VOLUME_DEFAULT;
+    double d_vol = f_vol;
 
-    dbus_message_iter_append_basic( container, DBUS_TYPE_DOUBLE, &d_vol );
-}
+    if( !dbus_message_iter_append_basic( container, DBUS_TYPE_DOUBLE, &d_vol ) )
+        return VLC_ENOMEM;
 
-DBUS_METHOD( VolumeGet )
-{
-    REPLY_INIT;
-    OUT_ARGUMENTS;
-
-    DBusMessageIter v;
-
-    if( !dbus_message_iter_open_container( &args, DBUS_TYPE_VARIANT, "d", &v ) )
-        return DBUS_HANDLER_RESULT_NEED_MEMORY;
-
-    MarshalVolume( p_this, &v );
-
-    if( !dbus_message_iter_close_container( &args, &v ) )
-        return DBUS_HANDLER_RESULT_NEED_MEMORY;
-
-    REPLY_SEND;
+    return VLC_SUCCESS;
 }
 
 DBUS_METHOD( VolumeSet )
@@ -203,12 +179,7 @@ DBUS_METHOD( VolumeSet )
     if( VLC_SUCCESS != DemarshalSetPropertyValue( p_from, &d_dbus_vol ) )
         return DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
 
-    d_dbus_vol *= AOUT_VOLUME_DEFAULT;
-    if( d_dbus_vol < 0. )
-        d_dbus_vol = 0.;
-    if( d_dbus_vol > AOUT_VOLUME_MAX )
-        d_dbus_vol = AOUT_VOLUME_MAX;
-    aout_VolumeSet( PL, lround(d_dbus_vol) );
+    playlist_VolumeSet( PL, fmaxf( d_dbus_vol, 0.f ) );
 
     REPLY_SEND;
 }
@@ -305,7 +276,35 @@ DBUS_METHOD( OpenUri )
     REPLY_SEND;
 }
 
-static void
+static int
+MarshalCanGoNext( intf_thread_t *p_intf, DBusMessageIter *container )
+{
+    VLC_UNUSED( p_intf );
+
+    dbus_bool_t b_can_go_next = TRUE;
+
+    if( !dbus_message_iter_append_basic( container, DBUS_TYPE_BOOLEAN,
+                                         &b_can_go_next ) )
+        return VLC_ENOMEM;
+
+    return VLC_SUCCESS;
+}
+
+static int
+MarshalCanGoPrevious( intf_thread_t *p_intf, DBusMessageIter *container )
+{
+    VLC_UNUSED( p_intf );
+
+    dbus_bool_t b_can_go_previous = TRUE;
+
+    if( !dbus_message_iter_append_basic( container, DBUS_TYPE_BOOLEAN,
+                                         &b_can_go_previous ) )
+        return VLC_ENOMEM;
+
+    return VLC_SUCCESS;
+}
+
+static int
 MarshalCanPlay( intf_thread_t *p_intf, DBusMessageIter *container )
 {
     playlist_t *p_playlist = p_intf->p_sys->p_playlist;
@@ -314,28 +313,14 @@ MarshalCanPlay( intf_thread_t *p_intf, DBusMessageIter *container )
     dbus_bool_t b_can_play = playlist_CurrentSize( p_playlist ) > 0;
     PL_UNLOCK;
 
-    dbus_message_iter_append_basic( container, DBUS_TYPE_BOOLEAN, &b_can_play );
+    if( !dbus_message_iter_append_basic( container, DBUS_TYPE_BOOLEAN,
+                                         &b_can_play ) )
+        return VLC_ENOMEM;
+
+    return VLC_SUCCESS;
 }
 
-DBUS_METHOD( CanPlay )
-{
-    REPLY_INIT;
-    OUT_ARGUMENTS;
-
-    DBusMessageIter v;
-
-    if( !dbus_message_iter_open_container( &args, DBUS_TYPE_VARIANT, "b", &v ) )
-        return DBUS_HANDLER_RESULT_NEED_MEMORY;
-
-    MarshalCanPlay( p_this, &v );
-
-    if( !dbus_message_iter_close_container( &args, &v ) )
-        return DBUS_HANDLER_RESULT_NEED_MEMORY;
-
-    REPLY_SEND;
-}
-
-static void
+static int
 MarshalCanPause( intf_thread_t *p_intf, DBusMessageIter *container )
 {
     dbus_bool_t b_can_pause = FALSE;
@@ -348,53 +333,27 @@ MarshalCanPause( intf_thread_t *p_intf, DBusMessageIter *container )
         vlc_object_release( p_input );
     }
 
-    dbus_message_iter_append_basic( container, DBUS_TYPE_BOOLEAN,
-                                    &b_can_pause );
+    if( !dbus_message_iter_append_basic( container, DBUS_TYPE_BOOLEAN,
+                                         &b_can_pause ) )
+        return VLC_ENOMEM;
+
+    return VLC_SUCCESS;
 }
 
-DBUS_METHOD( CanPause )
+static int
+MarshalCanControl( intf_thread_t *p_intf, DBusMessageIter *container )
 {
-    REPLY_INIT;
-    OUT_ARGUMENTS;
-
-    DBusMessageIter v;
-
-    if( !dbus_message_iter_open_container( &args, DBUS_TYPE_VARIANT, "b", &v ) )
-        return DBUS_HANDLER_RESULT_NEED_MEMORY;
-
-    MarshalCanPause( p_this, &v );
-
-    if( !dbus_message_iter_close_container( &args, &v ) )
-        return DBUS_HANDLER_RESULT_NEED_MEMORY;
-
-    REPLY_SEND;
-}
-
-DBUS_METHOD( CanControl )
-{
-    VLC_UNUSED( p_this );
-
-    REPLY_INIT;
-    OUT_ARGUMENTS;
-
-    DBusMessageIter v;
+    VLC_UNUSED( p_intf );
     dbus_bool_t b_can_control = TRUE;
 
-    if( !dbus_message_iter_open_container( &args, DBUS_TYPE_VARIANT,
-                                           "b", &v ) )
-        return DBUS_HANDLER_RESULT_NEED_MEMORY;
-
-    if( !dbus_message_iter_append_basic( &v, DBUS_TYPE_BOOLEAN,
+    if( !dbus_message_iter_append_basic( container, DBUS_TYPE_BOOLEAN,
                                          &b_can_control ) )
-        return DBUS_HANDLER_RESULT_NEED_MEMORY;
+        return VLC_ENOMEM;
 
-    if( !dbus_message_iter_close_container( &args, &v ) )
-        return DBUS_HANDLER_RESULT_NEED_MEMORY;
-
-    REPLY_SEND;
+    return VLC_SUCCESS;
 }
 
-static void
+static int
 MarshalCanSeek( intf_thread_t *p_intf, DBusMessageIter *container )
 {
     dbus_bool_t b_can_seek = FALSE;
@@ -407,50 +366,23 @@ MarshalCanSeek( intf_thread_t *p_intf, DBusMessageIter *container )
         vlc_object_release( p_input );
     }
 
-    dbus_message_iter_append_basic( container, DBUS_TYPE_BOOLEAN, &b_can_seek );
+    if( !dbus_message_iter_append_basic( container, DBUS_TYPE_BOOLEAN,
+                                         &b_can_seek ) )
+        return VLC_ENOMEM;
+
+    return VLC_SUCCESS;
 }
 
-DBUS_METHOD( CanSeek )
-{
-    REPLY_INIT;
-    OUT_ARGUMENTS;
-
-    DBusMessageIter v;
-
-    if( !dbus_message_iter_open_container( &args, DBUS_TYPE_VARIANT, "b", &v ) )
-        return DBUS_HANDLER_RESULT_NEED_MEMORY;
-
-    MarshalCanSeek( p_this, &v );
-
-    if( !dbus_message_iter_close_container( &args, &v ) )
-        return DBUS_HANDLER_RESULT_NEED_MEMORY;
-
-    REPLY_SEND;
-}
-
-static void
+static int
 MarshalShuffle( intf_thread_t *p_intf, DBusMessageIter *container )
 {
     dbus_bool_t b_shuffle = var_GetBool( p_intf->p_sys->p_playlist, "random" );
-    dbus_message_iter_append_basic( container, DBUS_TYPE_BOOLEAN, &b_shuffle );
-}
 
-DBUS_METHOD( ShuffleGet )
-{
-    REPLY_INIT;
-    OUT_ARGUMENTS;
+    if( !dbus_message_iter_append_basic( container, DBUS_TYPE_BOOLEAN,
+                                         &b_shuffle ))
+        return VLC_ENOMEM;
 
-    DBusMessageIter v;
-
-    if( !dbus_message_iter_open_container( &args, DBUS_TYPE_VARIANT, "b", &v ) )
-        return DBUS_HANDLER_RESULT_NEED_MEMORY;
-
-    MarshalShuffle( p_this, &v );
-
-    if( !dbus_message_iter_close_container( &args, &v ) )
-        return DBUS_HANDLER_RESULT_NEED_MEMORY;
-
-    REPLY_SEND;
+    return VLC_SUCCESS;
 }
 
 DBUS_METHOD( ShuffleSet )
@@ -466,7 +398,7 @@ DBUS_METHOD( ShuffleSet )
     REPLY_SEND;
 }
 
-static void
+static int
 MarshalPlaybackStatus( intf_thread_t *p_intf, DBusMessageIter *container )
 {
     input_thread_t *p_input;
@@ -492,33 +424,19 @@ MarshalPlaybackStatus( intf_thread_t *p_intf, DBusMessageIter *container )
     else
         psz_playback_status = PLAYBACK_STATUS_STOPPED;
 
-    dbus_message_iter_append_basic( container, DBUS_TYPE_STRING,
-                                    &psz_playback_status );
+    if( !dbus_message_iter_append_basic( container, DBUS_TYPE_STRING,
+                                         &psz_playback_status ) )
+        return VLC_ENOMEM;
+
+    return VLC_SUCCESS;
 }
 
-DBUS_METHOD( PlaybackStatus )
-{
-    REPLY_INIT;
-    OUT_ARGUMENTS;
-
-    DBusMessageIter v;
-
-    if( !dbus_message_iter_open_container( &args, DBUS_TYPE_VARIANT, "s", &v ) )
-        return DBUS_HANDLER_RESULT_NEED_MEMORY;
-
-    MarshalPlaybackStatus( p_this, &v );
-
-    if( !dbus_message_iter_close_container( &args, &v ) )
-        return DBUS_HANDLER_RESULT_NEED_MEMORY;
-
-    REPLY_SEND;
-}
-
-static void
+static int
 MarshalRate( intf_thread_t *p_intf, DBusMessageIter *container )
 {
     double d_rate;
     input_thread_t *p_input;
+
     if( ( p_input = playlist_CurrentInput( p_intf->p_sys->p_playlist ) ) )
     {
         d_rate = var_GetFloat( p_input, "rate" );
@@ -527,25 +445,11 @@ MarshalRate( intf_thread_t *p_intf, DBusMessageIter *container )
     else
         d_rate = 1.0;
 
-    dbus_message_iter_append_basic( container, DBUS_TYPE_DOUBLE, &d_rate );
-}
+    if( !dbus_message_iter_append_basic( container, DBUS_TYPE_DOUBLE,
+                                         &d_rate ) )
+        return VLC_ENOMEM;
 
-DBUS_METHOD( RateGet )
-{
-    REPLY_INIT;
-    OUT_ARGUMENTS;
-
-    DBusMessageIter v;
-
-    if( !dbus_message_iter_open_container( &args, DBUS_TYPE_VARIANT, "d", &v ) )
-        return DBUS_HANDLER_RESULT_NEED_MEMORY;
-
-    MarshalRate( p_this, &v );
-
-    if( !dbus_message_iter_close_container( &args, &v ) )
-        return DBUS_HANDLER_RESULT_NEED_MEMORY;
-
-    REPLY_SEND;
+    return VLC_SUCCESS;
 }
 
 DBUS_METHOD( RateSet )
@@ -569,51 +473,31 @@ DBUS_METHOD( RateSet )
     REPLY_SEND;
 }
 
-DBUS_METHOD( MinimumRate )
+static int
+MarshalMinimumRate( intf_thread_t *p_intf, DBusMessageIter *container )
 {
-    VLC_UNUSED( p_this );
-
-    REPLY_INIT;
-    OUT_ARGUMENTS;
-
-    DBusMessageIter v;
+    VLC_UNUSED( p_intf );
     double d_min_rate = (double) INPUT_RATE_MIN / INPUT_RATE_DEFAULT;
 
-    if( !dbus_message_iter_open_container( &args, DBUS_TYPE_VARIANT, "d", &v ) )
-        return DBUS_HANDLER_RESULT_NEED_MEMORY;
+    if( !dbus_message_iter_append_basic( container, DBUS_TYPE_DOUBLE, &d_min_rate ) )
+        return VLC_ENOMEM;
 
-    if( !dbus_message_iter_append_basic( &v, DBUS_TYPE_DOUBLE, &d_min_rate ) )
-        return DBUS_HANDLER_RESULT_NEED_MEMORY;
-
-    if( !dbus_message_iter_close_container( &args, &v ) )
-        return DBUS_HANDLER_RESULT_NEED_MEMORY;
-
-    REPLY_SEND;
+    return VLC_SUCCESS;
 }
 
-DBUS_METHOD( MaximumRate )
+static int
+MarshalMaximumRate( intf_thread_t *p_intf, DBusMessageIter *container )
 {
-    VLC_UNUSED( p_this );
-
-    REPLY_INIT;
-    OUT_ARGUMENTS;
-
-    DBusMessageIter v;
+    VLC_UNUSED( p_intf );
     double d_max_rate = (double) INPUT_RATE_MAX / INPUT_RATE_DEFAULT;
 
-    if( !dbus_message_iter_open_container( &args, DBUS_TYPE_VARIANT, "d", &v ) )
-        return DBUS_HANDLER_RESULT_NEED_MEMORY;
+    if( !dbus_message_iter_append_basic( container, DBUS_TYPE_DOUBLE, &d_max_rate ) )
+        return VLC_ENOMEM;
 
-    if( !dbus_message_iter_append_basic( &v, DBUS_TYPE_DOUBLE, &d_max_rate ) )
-        return DBUS_HANDLER_RESULT_NEED_MEMORY;
-
-    if( !dbus_message_iter_close_container( &args, &v ) )
-        return DBUS_HANDLER_RESULT_NEED_MEMORY;
-
-    REPLY_SEND;
+    return VLC_SUCCESS;
 }
 
-static void
+static int
 MarshalLoopStatus( intf_thread_t *p_intf, DBusMessageIter *container )
 {
     const char *psz_loop_status;
@@ -627,26 +511,11 @@ MarshalLoopStatus( intf_thread_t *p_intf, DBusMessageIter *container )
     else
         psz_loop_status = LOOP_STATUS_NONE;
 
-    dbus_message_iter_append_basic( container, DBUS_TYPE_STRING,
-                                    &psz_loop_status );
-}
+    if( !dbus_message_iter_append_basic( container, DBUS_TYPE_STRING,
+                                         &psz_loop_status ) )
+        return VLC_ENOMEM;
 
-DBUS_METHOD( LoopStatusGet )
-{
-    REPLY_INIT;
-    OUT_ARGUMENTS;
-
-    DBusMessageIter v;
-
-    if( !dbus_message_iter_open_container( &args, DBUS_TYPE_VARIANT, "s", &v ) )
-        return DBUS_HANDLER_RESULT_NEED_MEMORY;
-
-    MarshalLoopStatus( p_this, &v );
-
-    if( !dbus_message_iter_close_container( &args, &v ) )
-        return DBUS_HANDLER_RESULT_NEED_MEMORY;
-
-    REPLY_SEND;
+    return VLC_SUCCESS;
 }
 
 DBUS_METHOD( LoopStatusSet )
@@ -714,24 +583,6 @@ MarshalMetadata( intf_thread_t *p_intf, DBusMessageIter *container )
     return VLC_SUCCESS;
 }
 
-DBUS_METHOD( Metadata )
-{
-    REPLY_INIT;
-    OUT_ARGUMENTS;
-
-    DBusMessageIter v;
-
-    if( !dbus_message_iter_open_container( &args, DBUS_TYPE_VARIANT,
-                                           "a{sv}", &v ) )
-        return DBUS_HANDLER_RESULT_NEED_MEMORY;
-
-    if( MarshalMetadata( p_this, &v ) != VLC_SUCCESS ||
-        !dbus_message_iter_close_container( &args, &v ) ) {
-        return DBUS_HANDLER_RESULT_NEED_MEMORY;
-    }
-
-    REPLY_SEND;
-}
 
 /******************************************************************************
  * Seeked: non-linear playback signal
@@ -759,11 +610,22 @@ DBUS_SIGNAL( SeekedSignal )
 }
 
 #define PROPERTY_MAPPING_BEGIN if( 0 ) {}
-#define PROPERTY_FUNC( interface, property, function ) \
-    else if( !strcmp( psz_interface_name, interface ) && \
-             !strcmp( psz_property_name,  property ) ) \
-        return function( p_conn, p_from, p_this );
-#define PROPERTY_MAPPING_END return DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
+#define PROPERTY_GET_FUNC( prop, signature ) \
+    else if( !strcmp( psz_property_name,  #prop ) ) { \
+        if( !dbus_message_iter_open_container( &args, DBUS_TYPE_VARIANT, signature, &v ) ) \
+            return DBUS_HANDLER_RESULT_NEED_MEMORY; \
+        if( VLC_SUCCESS != Marshal##prop( p_this, &v ) ) { \
+            dbus_message_iter_abandon_container( &args, &v ); \
+            return DBUS_HANDLER_RESULT_NEED_MEMORY; \
+        } \
+        if( !dbus_message_iter_close_container( &args, &v ) ) \
+            return DBUS_HANDLER_RESULT_NEED_MEMORY; \
+    }
+#define PROPERTY_SET_FUNC( prop ) \
+    else if( !strcmp( psz_property_name,  #prop ) ) { \
+        return prop##Set( p_conn, p_from, p_this ); \
+    }
+#define PROPERTY_MAPPING_END else { return DBUS_HANDLER_RESULT_NOT_YET_HANDLED; }
 
 DBUS_METHOD( GetProperty )
 {
@@ -789,21 +651,33 @@ DBUS_METHOD( GetProperty )
     msg_Dbg( (vlc_object_t*) p_this, "Getting property %s",
                                      psz_property_name );
 
+    if( strcmp( psz_interface_name, DBUS_MPRIS_PLAYER_INTERFACE ) ) {
+        return DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
+    }
+
+    REPLY_INIT;
+    OUT_ARGUMENTS;
+    DBusMessageIter v;
+
     PROPERTY_MAPPING_BEGIN
-    PROPERTY_FUNC( DBUS_MPRIS_PLAYER_INTERFACE, "Metadata", Metadata )
-    PROPERTY_FUNC( DBUS_MPRIS_PLAYER_INTERFACE, "Position", Position )
-    PROPERTY_FUNC( DBUS_MPRIS_PLAYER_INTERFACE, "PlaybackStatus", PlaybackStatus )
-    PROPERTY_FUNC( DBUS_MPRIS_PLAYER_INTERFACE, "LoopStatus", LoopStatusGet )
-    PROPERTY_FUNC( DBUS_MPRIS_PLAYER_INTERFACE, "Shuffle", ShuffleGet )
-    PROPERTY_FUNC( DBUS_MPRIS_PLAYER_INTERFACE, "Volume", VolumeGet )
-    PROPERTY_FUNC( DBUS_MPRIS_PLAYER_INTERFACE, "Rate", RateGet )
-    PROPERTY_FUNC( DBUS_MPRIS_PLAYER_INTERFACE, "MinimumRate", MinimumRate )
-    PROPERTY_FUNC( DBUS_MPRIS_PLAYER_INTERFACE, "MaximumRate", MaximumRate )
-    PROPERTY_FUNC( DBUS_MPRIS_PLAYER_INTERFACE, "CanControl", CanControl )
-    PROPERTY_FUNC( DBUS_MPRIS_PLAYER_INTERFACE, "CanPlay", CanPlay )
-    PROPERTY_FUNC( DBUS_MPRIS_PLAYER_INTERFACE, "CanPause", CanPause )
-    PROPERTY_FUNC( DBUS_MPRIS_PLAYER_INTERFACE, "CanSeek", CanSeek )
+    PROPERTY_GET_FUNC( Metadata,       "a{sv}" )
+    PROPERTY_GET_FUNC( Position,       "x" )
+    PROPERTY_GET_FUNC( PlaybackStatus, "s" )
+    PROPERTY_GET_FUNC( LoopStatus,     "s" )
+    PROPERTY_GET_FUNC( Shuffle,        "b" )
+    PROPERTY_GET_FUNC( Volume,         "d" )
+    PROPERTY_GET_FUNC( Rate,           "d" )
+    PROPERTY_GET_FUNC( MinimumRate,    "d" )
+    PROPERTY_GET_FUNC( MaximumRate,    "d" )
+    PROPERTY_GET_FUNC( CanControl,     "b" )
+    PROPERTY_GET_FUNC( CanPlay,        "b" )
+    PROPERTY_GET_FUNC( CanGoNext,      "b" )
+    PROPERTY_GET_FUNC( CanGoPrevious,  "b" )
+    PROPERTY_GET_FUNC( CanPause,       "b" )
+    PROPERTY_GET_FUNC( CanSeek,        "b" )
     PROPERTY_MAPPING_END
+
+    REPLY_SEND;
 }
 
 DBUS_METHOD( SetProperty )
@@ -828,17 +702,77 @@ DBUS_METHOD( SetProperty )
     }
 
     PROPERTY_MAPPING_BEGIN
-    PROPERTY_FUNC( DBUS_MPRIS_PLAYER_INTERFACE, "LoopStatus", LoopStatusSet )
-    PROPERTY_FUNC( DBUS_MPRIS_PLAYER_INTERFACE, "Shuffle",    ShuffleSet )
-    PROPERTY_FUNC( DBUS_MPRIS_PLAYER_INTERFACE, "Volume",     VolumeSet )
-    PROPERTY_FUNC( DBUS_MPRIS_PLAYER_INTERFACE, "Rate",       RateSet )
+    PROPERTY_SET_FUNC( LoopStatus )
+    PROPERTY_SET_FUNC( Shuffle )
+    PROPERTY_SET_FUNC( Volume )
+    PROPERTY_SET_FUNC( Rate )
     PROPERTY_MAPPING_END
 }
 
 #undef PROPERTY_MAPPING_BEGIN
 #undef PROPERTY_GET_FUNC
+#undef PROPERTY_SET_FUNC
 #undef PROPERTY_MAPPING_END
 
+#define ADD_PROPERTY( prop, signature ) \
+    if( VLC_SUCCESS != AddProperty( (intf_thread_t*) p_this, \
+                &dict, #prop, signature, Marshal##prop ) ) { \
+        dbus_message_iter_abandon_container( &args, &dict ); \
+        return VLC_ENOMEM; \
+    }
+
+DBUS_METHOD( GetAllProperties )
+{
+    REPLY_INIT;
+    OUT_ARGUMENTS;
+
+    DBusError error;
+    DBusMessageIter dict;
+
+    char *const psz_interface_name = NULL;
+
+    dbus_error_init( &error );
+    dbus_message_get_args( p_from, &error,
+            DBUS_TYPE_STRING, &psz_interface_name,
+            DBUS_TYPE_INVALID );
+
+    if( dbus_error_is_set( &error ) )
+    {
+        msg_Err( (vlc_object_t*) p_this, "D-Bus message reading : %s",
+                                         error.message );
+        dbus_error_free( &error );
+        return DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
+    }
+
+    msg_Dbg( (vlc_object_t*) p_this, "Getting All properties" );
+
+    if( !dbus_message_iter_open_container( &args, DBUS_TYPE_ARRAY, "{sv}", &dict ) )
+        return DBUS_HANDLER_RESULT_NEED_MEMORY;
+
+    ADD_PROPERTY ( Metadata,       "a{sv}" );
+    ADD_PROPERTY ( Position,       "x"     );
+    ADD_PROPERTY ( PlaybackStatus, "s"     );
+    ADD_PROPERTY ( LoopStatus,     "s"     );
+    ADD_PROPERTY ( Shuffle,        "b"     );
+    ADD_PROPERTY ( Volume,         "d"     );
+    ADD_PROPERTY ( Rate,           "d"     );
+    ADD_PROPERTY ( MinimumRate,    "d"     );
+    ADD_PROPERTY ( MaximumRate,    "d"     );
+    ADD_PROPERTY ( CanControl,     "b"     );
+    ADD_PROPERTY ( CanPlay,        "b"     );
+    ADD_PROPERTY ( CanGoNext,      "b"     );
+    ADD_PROPERTY ( CanGoPrevious,  "b"     );
+    ADD_PROPERTY ( CanPause,       "b"     );
+    ADD_PROPERTY ( CanSeek,        "b"     );
+
+    if( !dbus_message_iter_close_container( &args, &dict ))
+        return DBUS_HANDLER_RESULT_NEED_MEMORY;
+
+    REPLY_SEND;
+}
+
+#undef ADD_PROPERTY
+
 #define METHOD_FUNC( interface, method, function ) \
     else if( dbus_message_is_method_call( p_from, interface, method ) )\
         return function( p_conn, p_from, p_this )
@@ -849,7 +783,7 @@ handle_player ( DBusConnection *p_conn, DBusMessage *p_from, void *p_this )
     if(0);
     METHOD_FUNC( DBUS_INTERFACE_PROPERTIES,   "Get",        GetProperty );
     METHOD_FUNC( DBUS_INTERFACE_PROPERTIES,   "Set",        SetProperty );
-/*  METHOD_FUNC( DBUS_INTERFACE_PROPERTIES,   "GetAll",     GetAllProperties );*/
+    METHOD_FUNC( DBUS_INTERFACE_PROPERTIES,   "GetAll",     GetAllProperties );
 
     /* here D-Bus method names are associated to an handler */
 
@@ -880,6 +814,16 @@ int SeekedEmit( intf_thread_t * p_intf )
     return VLC_SUCCESS;
 }
 
+#define PROPERTY_MAPPING_BEGIN if( 0 ) {}
+#define PROPERTY_ENTRY( prop, signature ) \
+    else if( !strcmp( ppsz_properties[i], #prop ) ) \
+    { \
+        if( VLC_SUCCESS != AddProperty( (intf_thread_t*) p_intf, \
+                    &changed_properties, #prop, signature, Marshal##prop ) ) \
+            return DBUS_HANDLER_RESULT_NEED_MEMORY; \
+    }
+#define PROPERTY_MAPPING_END else { return DBUS_HANDLER_RESULT_NOT_YET_HANDLED; }
+
 /**
  * PropertiesChangedSignal() synthetizes and sends the
  * org.freedesktop.DBus.Properties.PropertiesChanged signal
@@ -889,7 +833,7 @@ PropertiesChangedSignal( intf_thread_t    *p_intf,
                          vlc_dictionary_t *p_changed_properties )
 {
     DBusConnection  *p_conn = p_intf->p_sys->p_conn;
-    DBusMessageIter changed_properties, invalidated_properties, entry, variant;
+    DBusMessageIter changed_properties, invalidated_properties;
     const char *psz_interface_name = DBUS_MPRIS_PLAYER_INTERFACE;
     char **ppsz_properties = NULL;
     int i_properties = 0;
@@ -900,106 +844,56 @@ PropertiesChangedSignal( intf_thread_t    *p_intf,
 
     OUT_ARGUMENTS;
     ADD_STRING( &psz_interface_name );
-    dbus_message_iter_open_container( &args, DBUS_TYPE_ARRAY, "{sv}",
-                                      &changed_properties );
+
+    if( !dbus_message_iter_open_container( &args, DBUS_TYPE_ARRAY, "{sv}",
+                                           &changed_properties ) )
+        return DBUS_HANDLER_RESULT_NEED_MEMORY;
 
     i_properties = vlc_dictionary_keys_count( p_changed_properties );
     ppsz_properties = vlc_dictionary_all_keys( p_changed_properties );
 
+    if( unlikely(!ppsz_properties) )
+    {
+        dbus_message_iter_abandon_container( &args, &changed_properties );
+        return DBUS_HANDLER_RESULT_NEED_MEMORY;
+    }
+
     for( int i = 0; i < i_properties; i++ )
     {
-        dbus_message_iter_open_container( &changed_properties,
-                                          DBUS_TYPE_DICT_ENTRY, NULL,
-                                          &entry );
+        PROPERTY_MAPPING_BEGIN
+        PROPERTY_ENTRY( Metadata,       "a{sv}" )
+        PROPERTY_ENTRY( PlaybackStatus, "s"     )
+        PROPERTY_ENTRY( LoopStatus,     "s"     )
+        PROPERTY_ENTRY( Rate,           "d"     )
+        PROPERTY_ENTRY( Shuffle,        "b"     )
+        PROPERTY_ENTRY( Volume,         "d"     )
+        PROPERTY_ENTRY( CanSeek,        "b"     )
+        PROPERTY_ENTRY( CanPlay,        "b"     )
+        PROPERTY_ENTRY( CanPause,       "b"     )
+        PROPERTY_MAPPING_END
 
-        dbus_message_iter_append_basic( &entry, DBUS_TYPE_STRING,
-                                        &ppsz_properties[i] );
-
-        if( !strcmp( ppsz_properties[i], "Metadata" ) )
-        {
-            dbus_message_iter_open_container( &entry,
-                                              DBUS_TYPE_VARIANT, "a{sv}",
-                                              &variant );
-            MarshalMetadata( p_intf, &variant );
-            dbus_message_iter_close_container( &entry, &variant );
-        }
-        else if( !strcmp( ppsz_properties[i], "PlaybackStatus" ) )
-        {
-            dbus_message_iter_open_container( &entry,
-                                              DBUS_TYPE_VARIANT, "s",
-                                              &variant );
-            MarshalPlaybackStatus( p_intf, &variant );
-            dbus_message_iter_close_container( &entry, &variant );
-        }
-        else if( !strcmp( ppsz_properties[i], "LoopStatus" ) )
-        {
-            dbus_message_iter_open_container( &entry,
-                                              DBUS_TYPE_VARIANT, "s",
-                                              &variant );
-            MarshalLoopStatus( p_intf, &variant );
-            dbus_message_iter_close_container( &entry, &variant );
-        }
-        else if( !strcmp( ppsz_properties[i], "Rate" ) )
-        {
-            dbus_message_iter_open_container( &entry,
-                                              DBUS_TYPE_VARIANT, "d",
-                                              &variant );
-            MarshalRate( p_intf, &variant );
-            dbus_message_iter_close_container( &entry, &variant );
-        }
-        else if( !strcmp( ppsz_properties[i], "Shuffle" ) )
-        {
-            dbus_message_iter_open_container( &entry,
-                                              DBUS_TYPE_VARIANT, "b",
-                                              &variant );
-            MarshalShuffle( p_intf, &variant );
-            dbus_message_iter_close_container( &entry, &variant );
-        }
-        else if( !strcmp( ppsz_properties[i], "Volume" ) )
-        {
-            dbus_message_iter_open_container( &entry,
-                                              DBUS_TYPE_VARIANT, "d",
-                                              &variant );
-            MarshalVolume( p_intf, &variant );
-            dbus_message_iter_close_container( &entry, &variant );
-        }
-        else if( !strcmp( ppsz_properties[i], "CanSeek" ) )
-        {
-            dbus_message_iter_open_container( &entry,
-                                              DBUS_TYPE_VARIANT, "b",
-                                              &variant );
-            MarshalCanSeek( p_intf, &variant );
-            dbus_message_iter_close_container( &entry, &variant );
-        }
-        else if( !strcmp( ppsz_properties[i], "CanPlay" ) )
-        {
-            dbus_message_iter_open_container( &entry,
-                                              DBUS_TYPE_VARIANT, "b",
-                                              &variant );
-            MarshalCanPlay( p_intf, &variant );
-            dbus_message_iter_close_container( &entry, &variant );
-        }
-        else if( !strcmp( ppsz_properties[i], "CanPause" ) )
-        {
-            dbus_message_iter_open_container( &entry,
-                                              DBUS_TYPE_VARIANT, "b",
-                                              &variant );
-            MarshalCanPause( p_intf, &variant );
-            dbus_message_iter_close_container( &entry, &variant );
-        }
-        dbus_message_iter_close_container( &changed_properties, &entry );
         free( ppsz_properties[i] );
     }
 
-    dbus_message_iter_close_container( &args, &changed_properties );
-    dbus_message_iter_open_container( &args, DBUS_TYPE_ARRAY, "s",
-                                      &invalidated_properties );
-    dbus_message_iter_close_container( &args, &invalidated_properties );
+    if( !dbus_message_iter_close_container( &args, &changed_properties ) )
+        return DBUS_HANDLER_RESULT_NEED_MEMORY;
+
+    if( !dbus_message_iter_open_container( &args, DBUS_TYPE_ARRAY, "s",
+                                           &invalidated_properties ) )
+        return DBUS_HANDLER_RESULT_NEED_MEMORY;
+
+    if( !dbus_message_iter_close_container( &args, &invalidated_properties ) )
+        return DBUS_HANDLER_RESULT_NEED_MEMORY;
+
     free( ppsz_properties );
 
     SIGNAL_SEND;
 }
 
+#undef PROPERTY_MAPPING_BEGIN
+#undef PROPERTY_ADD
+#undef PROPERTY_MAPPING_END
+
 /*****************************************************************************
  * PropertiesChangedEmit: Emits the Seeked signal
  *****************************************************************************/
