@@ -1,23 +1,23 @@
 /*****************************************************************************
  * asf.c : ASF demux module
  *****************************************************************************
- * Copyright © 2002-2004, 2006-2008, 2010 the VideoLAN team
+ * Copyright © 2002-2004, 2006-2008, 2010 VLC authors and VideoLAN
  *
  * Authors: Laurent Aimar <fenrir@via.ecp.fr>
  *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * This program is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU Lesser General Public License as published by
+ * the Free Software Foundation; either version 2.1 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+ * GNU Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston MA 02110-1301, USA.
+ * You should have received a copy of the GNU Lesser General Public License
+ * along with this program; if not, write to the Free Software Foundation,
+ * Inc., 51 Franklin Street, Fifth Floor, Boston MA 02110-1301, USA.
  *****************************************************************************/
 
 /*****************************************************************************
@@ -261,7 +261,13 @@ static int SeekIndex( demux_t *p_demux, mtime_t i_date, float f_pos )
 
     uint64_t i_offset = (uint64_t)p_index->index_entry[i_entry].i_packet_number *
                         p_sys->p_fp->i_min_data_packet_size;
-    return stream_Seek( p_demux->s, p_sys->i_data_begin + i_offset );
+
+    if ( stream_Seek( p_demux->s, p_sys->i_data_begin + i_offset ) == VLC_SUCCESS )
+    {
+        es_out_Control( p_demux->out, ES_OUT_SET_NEXT_DISPLAY_TIME, VLC_TS_0 + i_date );
+        return VLC_SUCCESS;
+    }
+    else return VLC_EGENERIC;
 }
 
 static void SeekPrepare( demux_t *p_demux )
@@ -548,15 +554,11 @@ static int DemuxPayload(demux_t *p_demux, struct asf_packet_t *pkt, int i_payloa
     {
         int i_sub_payload_data_length = i_payload_data_length;
         if( i_replicated_data_length == 1 )
-        {
             i_sub_payload_data_length = pkt->p_peek[pkt->i_skip++];
-            i_payload_data_length--;
-        }
 
         stream_Read(p_demux->s, NULL, pkt->i_skip);
 
-        if ( i_sub_payload_data_length &&
-             DemuxSubPayload(p_demux, tk, i_sub_payload_data_length, i_pts,
+        if (DemuxSubPayload(p_demux, tk, i_sub_payload_data_length, i_pts,
                             i_media_object_offset) < 0)
             return -1;
 
@@ -750,6 +752,17 @@ static int DemuxInit( demux_t *p_demux )
         goto error;
     }
 
+    if ( ASF_FindObject( p_sys->p_root->p_hdr,
+                         &asf_object_content_encryption_guid, 0 ) != NULL
+         || ASF_FindObject( p_sys->p_root->p_hdr,
+                            &asf_object_extended_content_encryption_guid, 0 ) != NULL
+         || ASF_FindObject( p_sys->p_root->p_hdr,
+                         &asf_object_advanced_content_encryption_guid, 0 ) != NULL )
+    {
+        msg_Warn( p_demux, "ASF plugin discarded (DRM encumbered content)" );
+        goto error;
+    }
+
     p_sys->i_track = ASF_CountObject( p_sys->p_root->p_hdr,
                                       &asf_object_stream_properties_guid );
     if( p_sys->i_track <= 0 )
@@ -833,8 +846,8 @@ static int DemuxInit( demux_t *p_demux )
             i_format = GetWLE( &p_data[0] );
             wf_tag_to_fourcc( i_format, &fmt.i_codec, NULL );
             fmt.audio.i_channels        = GetWLE(  &p_data[2] );
-            fmt.audio.i_rate      = GetDWLE( &p_data[4] );
-            fmt.i_bitrate         = GetDWLE( &p_data[8] ) * 8;
+            fmt.audio.i_rate            = GetDWLE( &p_data[4] );
+            fmt.i_bitrate               = GetDWLE( &p_data[8] ) * 8;
             fmt.audio.i_blockalign      = GetWLE(  &p_data[12] );
             fmt.audio.i_bitspersample   = GetWLE(  &p_data[14] );
 
@@ -948,8 +961,8 @@ static int DemuxInit( demux_t *p_demux )
                 else
                     wf_tag_to_fourcc( i_format, &fmt.i_codec, NULL );
                 fmt.audio.i_channels        = GetWLE(  &p_data[2] );
-                fmt.audio.i_rate      = GetDWLE( &p_data[4] );
-                fmt.i_bitrate         = GetDWLE( &p_data[8] ) * 8;
+                fmt.audio.i_rate            = GetDWLE( &p_data[4] );
+                fmt.i_bitrate               = GetDWLE( &p_data[8] ) * 8;
                 fmt.audio.i_blockalign      = GetWLE(  &p_data[12] );
                 fmt.audio.i_bitspersample   = GetWLE(  &p_data[14] );
                 fmt.b_packetized = true;
@@ -1119,7 +1132,6 @@ error:
 static void DemuxEnd( demux_t *p_demux )
 {
     demux_sys_t *p_sys = p_demux->p_sys;
-    int         i;
 
     if( p_sys->p_root )
     {
@@ -1132,7 +1144,7 @@ static void DemuxEnd( demux_t *p_demux )
         p_sys->meta = NULL;
     }
 
-    for( i = 0; i < 128; i++ )
+    for( int i = 0; i < 128; i++ )
     {
         asf_track_t *tk = p_sys->track[i];
 
