@@ -1,23 +1,23 @@
 /*****************************************************************************
  * libavi.c : LibAVI
  *****************************************************************************
- * Copyright (C) 2001 the VideoLAN team
- * $Id: af519ded73b703dc6a9fb83b71a0c1b71c4f23d9 $
+ * Copyright (C) 2001 VLC authors and VideoLAN
+ * $Id: 43db2e5b4598793dccbe213aaf4cdf9a68110046 $
  * Authors: Laurent Aimar <fenrir@via.ecp.fr>
  *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * This program is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU Lesser General Public License as published by
+ * the Free Software Foundation; either version 2.1 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+ * GNU Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston MA 02110-1301, USA.
+ * You should have received a copy of the GNU Lesser General Public License
+ * along with this program; if not, write to the Free Software Foundation,
+ * Inc., 51 Franklin Street, Fifth Floor, Boston MA 02110-1301, USA.
  *****************************************************************************/
 
 
@@ -26,13 +26,13 @@
 #endif
 
 #include <vlc_common.h>
-#include <vlc_demux.h>
+#include <vlc_demux.h>                                   /* stream_*, *_ES */
 #include <vlc_codecs.h>                            /* VLC_BITMAPINFOHEADER */
 
 #include "libavi.h"
 
 #ifndef NDEBUG
-#define AVI_DEBUG 1
+# define AVI_DEBUG 1
 #endif
 
 #define __EVEN( x ) (((x) + 1) & ~1)
@@ -67,8 +67,7 @@ static int AVI_ChunkReadCommon( stream_t *s, avi_chunk_t *p_chk )
 
 #ifdef AVI_DEBUG
     msg_Dbg( (vlc_object_t*)s,
-             "found Chunk fourcc:%8.8x (%4.4s) size:%"PRId64" pos:%"PRId64,
-             p_chk->common.i_chunk_fourcc,
+             "found chunk, fourcc: %4.4s size:%"PRId64" pos:%"PRId64,
              (char*)&p_chk->common.i_chunk_fourcc,
              p_chk->common.i_chunk_size,
              p_chk->common.i_chunk_pos );
@@ -113,6 +112,7 @@ static int AVI_ChunkRead_list( stream_t *s, avi_chunk_t *p_container )
     avi_chunk_t *p_chk;
     const uint8_t *p_peek;
     bool b_seekable;
+    int i_ret = VLC_SUCCESS;
 
     if( p_container->common.i_chunk_size > 0 && p_container->common.i_chunk_size < 4 )
     {
@@ -174,7 +174,7 @@ static int AVI_ChunkRead_list( stream_t *s, avi_chunk_t *p_container )
         }
         p_container->common.p_last = p_chk;
 
-        if( AVI_ChunkRead( s, p_chk, p_container ) )
+        if( i_ret = AVI_ChunkRead( s, p_chk, p_container ) )
         {
             break;
         }
@@ -197,6 +197,7 @@ static int AVI_ChunkRead_list( stream_t *s, avi_chunk_t *p_container )
     }
     msg_Dbg( (vlc_object_t*)s, "</list \'%4.4s\'>", (char*)&p_container->list.i_type );
 
+    if ( i_ret == AVI_ZERO_FOURCC ) return i_ret;
     return VLC_SUCCESS;
 }
 
@@ -417,8 +418,16 @@ static int AVI_ChunkRead_strf( stream_t *s, avi_chunk_t *p_chk )
                      p_chk->strf.vids.p_bih->biBitCount );
 #endif
             break;
+        case AVIFOURCC_iavs:
+        case AVIFOURCC_ivas:
+            p_chk->strf.common.i_cat = UNKNOWN_ES;
+            break;
+        case( AVIFOURCC_txts ):
+            p_chk->strf.common.i_cat = SPU_ES;
+            break;
         default:
-            msg_Warn( (vlc_object_t*)s, "unknown stream type" );
+            msg_Warn( (vlc_object_t*)s, "unknown stream type: %4.4s",
+                    (char*)&p_strh->strh.i_type );
             p_chk->strf.common.i_cat = UNKNOWN_ES;
             break;
     }
@@ -439,6 +448,12 @@ static void AVI_ChunkFree_strf( avi_chunk_t *p_chk )
 
 static int AVI_ChunkRead_strd( stream_t *s, avi_chunk_t *p_chk )
 {
+    if ( p_chk->common.i_chunk_size == 0 )
+    {
+        msg_Dbg( (vlc_object_t*)s, "Zero sized pre-JUNK section met" );
+        return AVI_STRD_ZERO_CHUNK;
+    }
+
     AVI_READCHUNK_ENTER;
     p_chk->strd.p_data = xmalloc( p_chk->common.i_chunk_size );
     memcpy( p_chk->strd.p_data, p_buff + 8, p_chk->common.i_chunk_size );
@@ -576,7 +591,54 @@ static void AVI_ChunkFree_indx( avi_chunk_t *p_chk )
     FREENULL( p_indx->idx.super );
 }
 
+static int AVI_ChunkRead_vprp( stream_t *s, avi_chunk_t *p_chk )
+{
+    avi_chunk_vprp_t *p_vprp = (avi_chunk_vprp_t*)p_chk;
 
+    AVI_READCHUNK_ENTER;
+
+    AVI_READ4BYTES( p_vprp->i_video_format_token );
+    AVI_READ4BYTES( p_vprp->i_video_standard );
+    AVI_READ4BYTES( p_vprp->i_vertical_refresh );
+    AVI_READ4BYTES( p_vprp->i_h_total_in_t );
+    AVI_READ4BYTES( p_vprp->i_v_total_in_lines );
+    AVI_READ4BYTES( p_vprp->i_frame_aspect_ratio );
+    AVI_READ4BYTES( p_vprp->i_frame_width_in_pixels );
+    AVI_READ4BYTES( p_vprp->i_frame_height_in_pixels );
+    AVI_READ4BYTES( p_vprp->i_nb_fields_per_frame );
+    for( unsigned i = 0; i < __MIN( p_vprp->i_nb_fields_per_frame, 2 ); i++ )
+    {
+        AVI_READ4BYTES( p_vprp->field_info[i].i_compressed_bm_height );
+        AVI_READ4BYTES( p_vprp->field_info[i].i_compressed_bm_width );
+        AVI_READ4BYTES( p_vprp->field_info[i].i_valid_bm_height );
+        AVI_READ4BYTES( p_vprp->field_info[i].i_valid_bm_width );
+        AVI_READ4BYTES( p_vprp->field_info[i].i_valid_bm_x_offset );
+        AVI_READ4BYTES( p_vprp->field_info[i].i_valid_bm_y_offset );
+        AVI_READ4BYTES( p_vprp->field_info[i].i_video_x_offset_in_t );
+        AVI_READ4BYTES( p_vprp->field_info[i].i_video_y_valid_start_line );
+    }
+
+#ifdef AVI_DEBUG
+    msg_Dbg( (vlc_object_t*)s, "vprp: format:%d standard:%d",
+             p_vprp->i_video_format_token, p_vprp->i_video_standard );
+#endif
+    AVI_READCHUNK_EXIT( VLC_SUCCESS );
+}
+
+static int AVI_ChunkRead_dmlh( stream_t *s, avi_chunk_t *p_chk )
+{
+    avi_chunk_dmlh_t *p_dmlh = (avi_chunk_dmlh_t*)p_chk;
+
+    AVI_READCHUNK_ENTER;
+
+    AVI_READ4BYTES( p_dmlh->dwTotalFrames );
+
+#ifdef AVI_DEBUG
+    msg_Dbg( (vlc_object_t*)s, "dmlh: dwTotalFrames %d",
+             p_dmlh->dwTotalFrames );
+#endif
+    AVI_READCHUNK_EXIT( VLC_SUCCESS );
+}
 
 static const struct
 {
@@ -584,32 +646,45 @@ static const struct
     const char *psz_type;
 } AVI_strz_type[] =
 {
-    { AVIFOURCC_IARL, "archive location" },
-    { AVIFOURCC_IART, "artist" },
-    { AVIFOURCC_ICMS, "commisioned" },
-    { AVIFOURCC_ICMT, "comments" },
-    { AVIFOURCC_ICOP, "copyright" },
-    { AVIFOURCC_ICRD, "creation date" },
-    { AVIFOURCC_ICRP, "cropped" },
-    { AVIFOURCC_IDIM, "dimensions" },
-    { AVIFOURCC_IDPI, "dots per inch" },
-    { AVIFOURCC_IENG, "engineer" },
-    { AVIFOURCC_IGNR, "genre" },
-    { AVIFOURCC_IKEY, "keywords" },
-    { AVIFOURCC_ILGT, "lightness" },
-    { AVIFOURCC_IMED, "medium" },
-    { AVIFOURCC_INAM, "name" },
-    { AVIFOURCC_IPLT, "palette setting" },
-    { AVIFOURCC_IPRD, "product" },
-    { AVIFOURCC_ISBJ, "subject" },
-    { AVIFOURCC_ISFT, "software" },
-    { AVIFOURCC_ISHP, "sharpness" },
-    { AVIFOURCC_ISRC, "source" },
-    { AVIFOURCC_ISRF, "source form" },
-    { AVIFOURCC_ITCH, "technician" },
-    { AVIFOURCC_ISMP, "time code" },
-    { AVIFOURCC_IDIT, "digitalization time" },
-    { AVIFOURCC_strn, "stream name" },
+    { AVIFOURCC_IARL, "Archive location" },
+    { AVIFOURCC_IART, "Artist" },
+    { AVIFOURCC_ICMS, "Commisioned" },
+    { AVIFOURCC_ICMT, "Comments" },
+    { AVIFOURCC_ICOP, "Copyright" },
+    { AVIFOURCC_ICRD, "Creation date" },
+    { AVIFOURCC_ICRP, "Cropped" },
+    { AVIFOURCC_IDIM, "Dimensions" },
+    { AVIFOURCC_IDPI, "Dots per inch" },
+    { AVIFOURCC_IENG, "Engineer" },
+    { AVIFOURCC_IGNR, "Genre" },
+    { AVIFOURCC_ISGN, "Secondary Genre" },
+    { AVIFOURCC_IKEY, "Keywords" },
+    { AVIFOURCC_ILGT, "Lightness" },
+    { AVIFOURCC_IMED, "Medium" },
+    { AVIFOURCC_INAM, "Name" },
+    { AVIFOURCC_IPLT, "Palette setting" },
+    { AVIFOURCC_IPRD, "Product" },
+    { AVIFOURCC_ISBJ, "Subject" },
+    { AVIFOURCC_ISFT, "Software" },
+    { AVIFOURCC_ISHP, "Sharpness" },
+    { AVIFOURCC_ISRC, "Source" },
+    { AVIFOURCC_ISRF, "Source form" },
+    { AVIFOURCC_ITCH, "Technician" },
+    { AVIFOURCC_ISMP, "Time code" },
+    { AVIFOURCC_IDIT, "Digitalization time" },
+    { AVIFOURCC_IWRI, "Writer" },
+    { AVIFOURCC_IPRO, "Producer" },
+    { AVIFOURCC_ICNM, "Cinematographer" },
+    { AVIFOURCC_IPDS, "Production designer" },
+    { AVIFOURCC_IEDT, "Editor" },
+    { AVIFOURCC_ICDS, "Costume designer" },
+    { AVIFOURCC_IMUS, "Music" },
+    { AVIFOURCC_ISTD, "Production studio" },
+    { AVIFOURCC_IDST, "Distributor" },
+    { AVIFOURCC_ICNT, "Country" },
+    { AVIFOURCC_ISTR, "Starring" },
+    { AVIFOURCC_IFRM, "Total number of parts" },
+    { AVIFOURCC_strn, "Stream name" },
     { 0,              "???" }
 };
 static int AVI_ChunkRead_strz( stream_t *s, avi_chunk_t *p_chk )
@@ -674,9 +749,10 @@ static const struct
     { AVIFOURCC_strd, AVI_ChunkRead_strd, AVI_ChunkFree_strd },
     { AVIFOURCC_idx1, AVI_ChunkRead_idx1, AVI_ChunkFree_idx1 },
     { AVIFOURCC_indx, AVI_ChunkRead_indx, AVI_ChunkFree_indx },
+    { AVIFOURCC_vprp, AVI_ChunkRead_vprp, AVI_ChunkFree_nothing },
     { AVIFOURCC_JUNK, AVI_ChunkRead_nothing, AVI_ChunkFree_nothing },
+    { AVIFOURCC_dmlh, AVI_ChunkRead_dmlh, AVI_ChunkFree_nothing },
 
-    { AVIFOURCC_IARL, AVI_ChunkRead_strz, AVI_ChunkFree_strz },
     { AVIFOURCC_IARL, AVI_ChunkRead_strz, AVI_ChunkFree_strz },
     { AVIFOURCC_IART, AVI_ChunkRead_strz, AVI_ChunkFree_strz },
     { AVIFOURCC_ICMS, AVI_ChunkRead_strz, AVI_ChunkFree_strz },
@@ -688,6 +764,7 @@ static const struct
     { AVIFOURCC_IDPI, AVI_ChunkRead_strz, AVI_ChunkFree_strz },
     { AVIFOURCC_IENG, AVI_ChunkRead_strz, AVI_ChunkFree_strz },
     { AVIFOURCC_IGNR, AVI_ChunkRead_strz, AVI_ChunkFree_strz },
+    { AVIFOURCC_ISGN, AVI_ChunkRead_strz, AVI_ChunkFree_strz },
     { AVIFOURCC_IKEY, AVI_ChunkRead_strz, AVI_ChunkFree_strz },
     { AVIFOURCC_ILGT, AVI_ChunkRead_strz, AVI_ChunkFree_strz },
     { AVIFOURCC_IMED, AVI_ChunkRead_strz, AVI_ChunkFree_strz },
@@ -702,6 +779,24 @@ static const struct
     { AVIFOURCC_ITCH, AVI_ChunkRead_strz, AVI_ChunkFree_strz },
     { AVIFOURCC_ISMP, AVI_ChunkRead_strz, AVI_ChunkFree_strz },
     { AVIFOURCC_IDIT, AVI_ChunkRead_strz, AVI_ChunkFree_strz },
+    { AVIFOURCC_ILNG, AVI_ChunkRead_strz, AVI_ChunkFree_strz },
+    { AVIFOURCC_IRTD, AVI_ChunkRead_strz, AVI_ChunkFree_strz },
+    { AVIFOURCC_IWEB, AVI_ChunkRead_strz, AVI_ChunkFree_strz },
+    { AVIFOURCC_IPRT, AVI_ChunkRead_strz, AVI_ChunkFree_strz },
+    { AVIFOURCC_IWRI, AVI_ChunkRead_strz, AVI_ChunkFree_strz },
+    { AVIFOURCC_IPRO, AVI_ChunkRead_strz, AVI_ChunkFree_strz },
+    { AVIFOURCC_ICNM, AVI_ChunkRead_strz, AVI_ChunkFree_strz },
+    { AVIFOURCC_IPDS, AVI_ChunkRead_strz, AVI_ChunkFree_strz },
+    { AVIFOURCC_IEDT, AVI_ChunkRead_strz, AVI_ChunkFree_strz },
+    { AVIFOURCC_ICDS, AVI_ChunkRead_strz, AVI_ChunkFree_strz },
+    { AVIFOURCC_IMUS, AVI_ChunkRead_strz, AVI_ChunkFree_strz },
+    { AVIFOURCC_ISTD, AVI_ChunkRead_strz, AVI_ChunkFree_strz },
+    { AVIFOURCC_IDST, AVI_ChunkRead_strz, AVI_ChunkFree_strz },
+    { AVIFOURCC_ICNT, AVI_ChunkRead_strz, AVI_ChunkFree_strz },
+    { AVIFOURCC_ISTR, AVI_ChunkRead_strz, AVI_ChunkFree_strz },
+    { AVIFOURCC_IFRM, AVI_ChunkRead_strz, AVI_ChunkFree_strz },
+
+
     { AVIFOURCC_strn, AVI_ChunkRead_strz, AVI_ChunkFree_strz },
     { 0,           NULL,               NULL }
 };
@@ -719,12 +814,13 @@ static int AVI_ChunkFunctionFind( vlc_fourcc_t i_fourcc )
     }
 }
 
-int  _AVI_ChunkRead( stream_t *s, avi_chunk_t *p_chk, avi_chunk_t *p_father )
+int  AVI_ChunkRead( stream_t *s, avi_chunk_t *p_chk, avi_chunk_t *p_father )
 {
     int i_index;
 
     if( !p_chk )
     {
+        msg_Warn( (vlc_object_t*)s, "cannot read null chunk" );
         return VLC_EGENERIC;
     }
 
@@ -733,17 +829,24 @@ int  _AVI_ChunkRead( stream_t *s, avi_chunk_t *p_chk, avi_chunk_t *p_father )
         msg_Warn( (vlc_object_t*)s, "cannot read one chunk" );
         return VLC_EGENERIC;
     }
+
     if( p_chk->common.i_chunk_fourcc == VLC_FOURCC( 0, 0, 0, 0 ) )
     {
         msg_Warn( (vlc_object_t*)s, "found null fourcc chunk (corrupted file?)" );
-        return VLC_EGENERIC;
+        return AVI_ZERO_FOURCC;
     }
     p_chk->common.p_father = p_father;
 
     i_index = AVI_ChunkFunctionFind( p_chk->common.i_chunk_fourcc );
     if( AVI_Chunk_Function[i_index].AVI_ChunkRead_function )
     {
-        return AVI_Chunk_Function[i_index].AVI_ChunkRead_function( s, p_chk );
+        int i_return = AVI_Chunk_Function[i_index].AVI_ChunkRead_function( s, p_chk );
+        if ( i_return == AVI_STRD_ZERO_CHUNK || i_return == AVI_ZERO_FOURCC )
+        {
+            if ( !p_father ) return VLC_EGENERIC;
+            return AVI_NextChunk( s, p_father );
+        }
+        return i_return;
     }
     else if( ( ((char*)&p_chk->common.i_chunk_fourcc)[0] == 'i' &&
                ((char*)&p_chk->common.i_chunk_fourcc)[1] == 'x' ) ||
@@ -753,11 +856,13 @@ int  _AVI_ChunkRead( stream_t *s, avi_chunk_t *p_chk, avi_chunk_t *p_father )
         p_chk->common.i_chunk_fourcc = AVIFOURCC_indx;
         return AVI_ChunkRead_indx( s, p_chk );
     }
-    msg_Warn( (vlc_object_t*)s, "unknown chunk (not loaded)" );
+
+    msg_Warn( (vlc_object_t*)s, "unknown chunk: %4.4s (not loaded)",
+            (char*)&p_chk->common.i_chunk_fourcc );
     return AVI_NextChunk( s, p_chk );
 }
 
-void _AVI_ChunkFree( stream_t *s,
+void AVI_ChunkFree( stream_t *s,
                      avi_chunk_t *p_chk )
 {
     int i_index;
@@ -789,7 +894,8 @@ void _AVI_ChunkFree( stream_t *s,
     }
     else
     {
-        msg_Warn( (vlc_object_t*)s, "unknown chunk (not unloaded)" );
+        msg_Warn( (vlc_object_t*)s, "unknown chunk: %4.4s (not unloaded)",
+                (char*)&p_chk->common.i_chunk_fourcc );
     }
     p_chk->common.p_first = NULL;
     p_chk->common.p_last  = NULL;
@@ -800,23 +906,22 @@ void _AVI_ChunkFree( stream_t *s,
 static void AVI_ChunkDumpDebug_level( vlc_object_t *p_obj,
                                       avi_chunk_t  *p_chk, unsigned i_level )
 {
-    unsigned i;
     avi_chunk_t *p_child;
 
     char str[512];
-    if( i_level >= (sizeof(str) - 1)/5 )
+    if( i_level >= (sizeof(str) - 1)/4 )
         return;
 
     memset( str, ' ', sizeof( str ) );
-    for( i = 1; i < i_level; i++ )
+    for( unsigned i = 1; i < i_level; i++ )
     {
-        str[i * 5] = '|';
+        str[i * 4] = '|';
     }
     if( p_chk->common.i_chunk_fourcc == AVIFOURCC_RIFF ||
         p_chk->common.i_chunk_fourcc == AVIFOURCC_ON2  ||
         p_chk->common.i_chunk_fourcc == AVIFOURCC_LIST )
     {
-        snprintf( &str[i_level * 5], sizeof(str) - 5*i_level,
+        snprintf( &str[i_level * 4], sizeof(str) - 4*i_level,
                  "%c %4.4s-%4.4s size:%"PRIu64" pos:%"PRIu64,
                  i_level ? '+' : '*',
                  (char*)&p_chk->common.i_chunk_fourcc,
@@ -826,7 +931,7 @@ static void AVI_ChunkDumpDebug_level( vlc_object_t *p_obj,
     }
     else
     {
-        snprintf( &str[i_level * 5], sizeof(str) - 5*i_level,
+        snprintf( &str[i_level * 4], sizeof(str) - 4*i_level,
                  "+ %4.4s size:%"PRIu64" pos:%"PRIu64,
                  (char*)&p_chk->common.i_chunk_fourcc,
                  p_chk->common.i_chunk_size,
@@ -954,4 +1059,3 @@ void *_AVI_ChunkFind( avi_chunk_t *p_chk,
     return NULL;
 }
 
-
