@@ -1,25 +1,25 @@
 /*****************************************************************************
- * mkv.cpp : matroska demuxer
+ * mkv.hpp : matroska demuxer
  *****************************************************************************
- * Copyright (C) 2003-2005, 2008 the VideoLAN team
- * $Id: f30e37081e9748d34f389e77065c670d3aab4dc1 $
+ * Copyright (C) 2003-2005, 2008 VLC authors and VideoLAN
+ * $Id: efc88b3c2fef23ac529fc190ef26fc0b404441e2 $
  *
  * Authors: Laurent Aimar <fenrir@via.ecp.fr>
  *          Steve Lhomme <steve.lhomme@free.fr>
  *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * This program is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU Lesser General Public License as published by
+ * the Free Software Foundation; either version 2.1 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+ * GNU Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston MA 02110-1301, USA.
+ * You should have received a copy of the GNU Lesser General Public License
+ * along with this program; if not, write to the Free Software Foundation,
+ * Inc., 51 Franklin Street, Fifth Floor, Boston MA 02110-1301, USA.
  *****************************************************************************/
 
 #ifndef _MKV_H_
@@ -96,8 +96,6 @@
 
 #include "ebml/StdIOCallback.h"
 
-#include <vlc_keys.h>
-
 extern "C" {
    #include "../mp4/libmp4.h"
 }
@@ -113,6 +111,13 @@ extern "C" {
 #define MATROSKA_COMPRESSION_LZOX   2
 #define MATROSKA_COMPRESSION_HEADER 3
 
+enum
+{
+    MATROSKA_ENCODING_SCOPE_ALL_FRAMES = 1,
+    MATROSKA_ENCODING_SCOPE_PRIVATE = 2,
+    MATROSKA_ENCODING_SCOPE_NEXT = 4 /* unsupported */
+};
+
 #define MKVD_TIMECODESCALE 1000000
 
 #define MKV_IS_ID( el, C ) ( el != NULL && typeid( *el ) == typeid( C ) )
@@ -174,6 +179,13 @@ struct matroska_stream_c
 /*****************************************************************************
  * definitions of structures and functions used by this plugins
  *****************************************************************************/
+class PrivateTrackData
+{
+public:
+    virtual ~PrivateTrackData() {}
+    virtual int32_t Init() { return 0; }
+};
+
 struct mkv_track_t
 {
 //    ~mkv_track_t();
@@ -202,6 +214,9 @@ struct mkv_track_t
     /* audio */
     unsigned int i_original_rate;
 
+    /* Private track paramters */
+    PrivateTrackData *p_sys;
+
     bool            b_inited;
     /* data to be send first */
     int             i_data_init;
@@ -219,6 +234,7 @@ struct mkv_track_t
 
     /* encryption/compression */
     int                    i_compression_type;
+    uint32_t               i_encoding_scope;
     KaxContentCompSettings *p_compression_data;
 
 };
