@@ -1,8 +1,8 @@
 /*****************************************************************************
  * MainMenu.h: MacOS X interface module
  *****************************************************************************
- * Copyright (C) 2011 Felix Paul Kühne
- * $Id: f4818c85ba23d25bfe72379d5aabbbfb10b0ddf6 $
+ * Copyright (C) 2011-2013 Felix Paul Kühne
+ * $Id: e006b6b2acc3fbf3b64c3a304bd5baeab6c9bdef $
  *
  * Authors: Felix Paul Kühne <fkuehne -at- videolan -dot- org>
  *
@@ -34,12 +34,14 @@
     BOOL b_nib_audioeffects_loaded;
     BOOL b_nib_tracksynchro_loaded;
     BOOL b_nib_bookmarks_loaded;
+    BOOL b_nib_convertandsave_loaded;
 
     id o_about;                 /* VLAboutBox     */
     id o_videoeffects;          /* VLCVideoEffects */
     id o_audioeffects;          /* VLCAudioEffects */
     id o_trackSynchronization;  /* VLCTrackSynchronization */
     id o_bookmarks;             /* VLCBookmarks */
+    id o_convertandsave;        /* VLCConvertAndSave */
 
     id o_extMgr;                /* Extensions Manager */
 
@@ -66,6 +68,7 @@
     IBOutlet NSMenuItem * o_mi_open_capture;
     IBOutlet NSMenuItem * o_mi_open_recent;
     IBOutlet NSMenuItem * o_mi_open_wizard;
+    IBOutlet NSMenuItem * o_mi_convertandsave;
 
     IBOutlet NSMenu * o_mu_edit;
     IBOutlet NSMenuItem * o_mi_cut;
@@ -74,6 +77,15 @@
     IBOutlet NSMenuItem * o_mi_clear;
     IBOutlet NSMenuItem * o_mi_select_all;
 
+    IBOutlet NSMenu * o_mu_view;
+    IBOutlet NSMenuItem * o_mi_toggleJumpButtons;
+    IBOutlet NSMenuItem * o_mi_togglePlaymodeButtons;
+    IBOutlet NSMenuItem * o_mi_toggleEffectsButton;
+    IBOutlet NSMenuItem * o_mi_toggleSidebar;
+    IBOutlet NSMenu * o_mu_playlistTableColumns;
+    IBOutlet NSMenuItem * o_mi_playlistTableColumns;
+    NSMenu * o_mu_playlistTableColumnsContextMenu;
+
     IBOutlet NSMenu * o_mu_controls;
     IBOutlet NSMenuItem * o_mi_play;
     IBOutlet NSMenuItem * o_mi_stop;
@@ -93,6 +105,7 @@
     IBOutlet NSMenuItem * o_mi_random;
     IBOutlet NSMenuItem * o_mi_repeat;
     IBOutlet NSMenuItem * o_mi_loop;
+    IBOutlet NSMenuItem * o_mi_AtoBloop;
     IBOutlet NSMenuItem * o_mi_quitAfterPB;
     IBOutlet NSMenuItem * o_mi_fwd;
     IBOutlet NSMenuItem * o_mi_bwd;
@@ -132,15 +145,30 @@
     IBOutlet NSMenu * o_mu_aspect_ratio;
     IBOutlet NSMenuItem * o_mi_crop;
     IBOutlet NSMenu * o_mu_crop;
-    IBOutlet NSMenuItem * o_mi_subtitle;
-    IBOutlet NSMenu * o_mu_subtitle;
-    IBOutlet NSMenuItem * o_mi_addSub;
     IBOutlet NSMenuItem * o_mi_deinterlace;
     IBOutlet NSMenu * o_mu_deinterlace;
     IBOutlet NSMenuItem * o_mi_deinterlace_mode;
     IBOutlet NSMenu * o_mu_deinterlace_mode;
     IBOutlet NSMenuItem * o_mi_ffmpeg_pp;
     IBOutlet NSMenu * o_mu_ffmpeg_pp;
+
+    IBOutlet NSMenu * o_mu_subtitles;
+    IBOutlet NSMenuItem * o_mi_subtitle_track;
+    IBOutlet NSMenu * o_mu_subtitle_tracks;
+    IBOutlet NSMenuItem * o_mi_openSubtitleFile;
+    IBOutlet NSMenu * o_mu_subtitle_size;
+    IBOutlet NSMenuItem *o_mi_subtitle_size;
+    IBOutlet NSMenu * o_mu_subtitle_textcolor;
+    IBOutlet NSMenuItem *o_mi_subtitle_textcolor;
+    IBOutlet NSMenu * o_mu_subtitle_bgcolor;
+    IBOutlet NSMenuItem * o_mi_subtitle_bgcolor;
+    IBOutlet NSMenuItem * o_mi_subtitle_bgopacity;
+    IBOutlet NSView * o_mi_subtitle_bgopacity_view;
+    IBOutlet id o_mi_subtitle_bgopacity_lbl;
+    IBOutlet id o_mi_subtitle_bgopacity_lbl_gray;
+    IBOutlet id o_mi_subtitle_bgopacity_sld;
+    IBOutlet NSMenu * o_mu_subtitle_outlinethickness;
+    IBOutlet NSMenuItem * o_mi_subtitle_outlinethickness;
     IBOutlet NSMenuItem * o_mi_teletext;
     IBOutlet NSMenuItem * o_mi_teletext_transparent;
     IBOutlet NSMenuItem * o_mi_teletext_index;
@@ -190,6 +218,10 @@
     IBOutlet NSMenuItem * o_vmi_mute;
     IBOutlet NSMenuItem * o_vmi_fullscreen;
     IBOutlet NSMenuItem * o_vmi_snapshot;
+
+    // information for playlist table columns menu
+    NSDictionary * o_ptc_translation_dict;
+    NSArray * o_ptc_menuorder;
 }
 + (VLCMainMenu *)sharedInstance;
 
@@ -201,6 +233,7 @@
 - (void)setSubmenusEnabled:(BOOL)b_enabled;
 - (void)setRateControlsEnabled:(BOOL)b_enabled;
 - (void)setupExtensionsMenu;
+- (void)updateSidebarMenuItem;
 
 - (IBAction)intfOpenFile:(id)sender;
 - (IBAction)intfOpenFileGeneric:(id)sender;
@@ -208,10 +241,21 @@
 - (IBAction)intfOpenNet:(id)sender;
 - (IBAction)intfOpenCapture:(id)sender;
 
+- (IBAction)toggleEffectsButton:(id)sender;
+- (IBAction)toggleJumpButtons:(id)sender;
+- (IBAction)togglePlaymodeButtons:(id)sender;
+- (IBAction)toggleSidebar:(id)sender;
+- (IBAction)togglePlaylistColumnTable:(id)sender;
+- (void)setPlaylistColumnTableState:(NSInteger)i_state forColumn:(NSString *)o_column;
+- (NSMenu *)setupPlaylistTableColumnsMenu;
+
 - (IBAction)toggleRecord:(id)sender;
 - (void)updateRecordState:(BOOL)b_value;
 - (IBAction)setPlaybackRate:(id)sender;
 - (void)updatePlaybackRate;
+- (IBAction)toggleAtoBloop:(id)sender;
+
+- (IBAction)toggleAudioDevice:(id)sender;
 
 - (IBAction)toggleFullscreen:(id)sender;
 - (IBAction)resizeVideoWindow:(id)sender;
@@ -219,7 +263,14 @@
 - (IBAction)createVideoSnapshot:(id)sender;
 - (IBAction)toggleFullscreenDevice:(id)sender;
 
+- (IBAction)addSubtitleFile:(id)sender;
+- (IBAction)switchSubtitleOption:(id)sender;
+- (IBAction)switchSubtitleBackgroundOpacity:(id)sender;
+- (IBAction)telxTransparent:(id)sender;
+- (IBAction)telxNavLink:(id)sender;
+
 - (IBAction)showWizard:(id)sender;
+- (IBAction)showConvertAndSave:(id)sender;
 - (IBAction)showVideoEffects:(id)sender;
 - (IBAction)showAudioEffects:(id)sender;
 - (IBAction)showTrackSynchronization:(id)sender;
