@@ -1,17 +1,18 @@
 /*****************************************************************************
  * fspanel.m: MacOS X full screen panel
  *****************************************************************************
- * Copyright (C) 2006-2011 VLC authors and VideoLAN
- * $Id: 05fa310ba702c523571a59f06d6534d98b511fc2 $
+ * Copyright (C) 2006-2013 VLC authors and VideoLAN
+ * $Id: e387ab3780666eab619e4f86b0024d974e62a434 $
  *
  * Authors: Jérôme Decoodt <djc at videolan dot org>
  *          Felix Paul Kühne <fkuehne at videolan dot org>
+ *          David Fuhrmann <david dot fuhrmann at googlemail dot com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
- * 
+ *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
@@ -31,7 +32,6 @@
 #import "misc.h"
 #import "fspanel.h"
 #import "CompatibilityFixes.h"
-#import <vlc_aout_intf.h>
 
 @interface VLCFSPanel ()
 - (void)hideMouse;
@@ -42,21 +42,21 @@
  *****************************************************************************/
 @implementation VLCFSPanel
 /* We override this initializer so we can set the NSBorderlessWindowMask styleMask, and set a few other important settings */
-- (id)initWithContentRect:(NSRect)contentRect 
-                styleMask:(NSUInteger)aStyle 
-                  backing:(NSBackingStoreType)bufferingType 
+- (id)initWithContentRect:(NSRect)contentRect
+                styleMask:(NSUInteger)aStyle
+                  backing:(NSBackingStoreType)bufferingType
                     defer:(BOOL)flag
 {
     id win = [super initWithContentRect:contentRect styleMask:NSTexturedBackgroundWindowMask backing:bufferingType defer:flag];
     [win setOpaque:NO];
     [win setHasShadow: NO];
     [win setBackgroundColor:[NSColor clearColor]];
-    if (OSX_LION)
+    if (!OSX_SNOW_LEOPARD)
         [win setCollectionBehavior: NSWindowCollectionBehaviorFullScreenAuxiliary];
 
     /* let the window sit on top of everything else and start out completely transparent */
     [win setLevel:NSModalPanelWindowLevel];
-    i_device = 0;
+    i_device = config_GetInt(VLCIntf, "macosx-vdev");
     hideAgainTimer = fadeTimer = nil;
     [self setNonActive:nil];
     return win;
@@ -72,7 +72,8 @@
     if (!isInside)
         [self mouseExited:NULL];
 
-    [self center];
+    if (!OSX_SNOW_LEOPARD)
+        [self setAnimationBehavior:NSWindowAnimationBehaviorNone];
 
     /* get a notification if VLC isn't the active app anymore */
     [[NSNotificationCenter defaultCenter]
@@ -81,7 +82,8 @@
            name: NSApplicationDidResignActiveNotification
          object: NSApp];
 
-    /* get a notification if VLC is the active app again */
+    /* Get a notification if VLC is the active app again.
+     Needed as becomeKeyWindow does not get called when window is activated by clicking */
     [[NSNotificationCenter defaultCenter]
     addObserver: self
        selector: @selector(setActive:)
@@ -104,11 +106,14 @@
 {
     [[NSNotificationCenter defaultCenter] removeObserver: self];
 
-    if( hideAgainTimer )
-    {
+    if (hideAgainTimer) {
         [hideAgainTimer invalidate];
         [hideAgainTimer release];
     }
+
+    if (o_vout_window)
+        [o_vout_window release];
+
     [self setFadeTimer:nil];
     [super dealloc];
 }
@@ -125,10 +130,8 @@
     screen = [NSScreen screenWithDisplayID: (CGDirectDisplayID)i_device];
 
     if (!screen)
-    {
         /* invalid preferences or none specified, using main screen */
         screen = [NSScreen mainScreen];
-    }
 
     theScreensFrame = [screen frame];
     theWindowsFrame = [self frame];
@@ -153,9 +156,9 @@
     [[self contentView] setStreamTitle: o_title];
 }
 
-- (void)setStreamPos:(float) f_pos andTime:(NSString *)o_time
+- (void)updatePositionAndTime
 {
-    [[self contentView] setStreamPos:f_pos andTime: o_time];
+    [[self contentView] updatePositionAndTime];
 }
 
 - (void)setSeekable:(BOOL) b_seekable
@@ -171,18 +174,21 @@
 - (void)setNonActive:(id)noData
 {
     b_nonActive = YES;
-    [self orderOut: self];
 
     /* here's fadeOut, just without visibly fading */
     b_displayed = NO;
     [self setAlphaValue:0.0];
     [self setFadeTimer:nil];
+
     b_fadeQueued = NO;
+
+    [self orderOut: self];
 }
 
 - (void)setActive:(id)noData
 {
     b_nonActive = NO;
+
     [[VLCMain sharedInstance] showFullscreenController];
 }
 
@@ -190,21 +196,19 @@
 - (void)focus:(NSTimer *)timer
 {
     /* we need to push ourselves to front if the vout window was closed since our last display */
-    if( b_voutWasUpdated )
-    {
+    if (b_voutWasUpdated) {
         [self orderFront: self];
         b_voutWasUpdated = NO;
     }
 
-    if( [self alphaValue] < 1.0 )
+    if ([self alphaValue] < 1.0) {
         [self setAlphaValue:[self alphaValue]+0.1];
-    if( [self alphaValue] >= 1.0 )
-    {
+    }
+    if ([self alphaValue] >= 1.0) {
         b_displayed = YES;
         [self setAlphaValue: 1.0];
         [self setFadeTimer:nil];
-        if( b_fadeQueued )
-        {
+        if (b_fadeQueued) {
             b_fadeQueued=NO;
             [self setFadeTimer:[NSTimer scheduledTimerWithTimeInterval:0.1 target:self selector:@selector(unfocus:) userInfo:NULL repeats:YES]];
         }
@@ -214,29 +218,27 @@
 /* This routine is called repeatedly to hide the window */
 - (void)unfocus:(NSTimer *)timer
 {
-    if( b_keptVisible )
-    {
+    if (b_keptVisible) {
         b_keptVisible = NO;
         b_fadeQueued = NO;
         [self setFadeTimer: NULL];
         [self fadeIn];
         return;
     }
-    if( [self alphaValue] > 0.0 )
+    if ([self alphaValue] > 0.0) {
         [self setAlphaValue:[self alphaValue]-0.05];
-    if( [self alphaValue] <= 0.05 )
-    {
+    }
+    if ([self alphaValue] <= 0.05) {
         b_displayed = NO;
         [self setAlphaValue:0.0];
         [self setFadeTimer:nil];
-        if( b_fadeQueued )
-        {
+        if (b_fadeQueued) {
             b_fadeQueued=NO;
             [self setFadeTimer:
-                [NSTimer scheduledTimerWithTimeInterval:0.1 
-                                                 target:self 
-                                               selector:@selector(focus:) 
-                                               userInfo:NULL 
+                [NSTimer scheduledTimerWithTimeInterval:0.1
+                                                 target:self
+                                               selector:@selector(focus:)
+                                               userInfo:NULL
                                                 repeats:YES]];
         }
     }
@@ -245,13 +247,8 @@
 - (void)mouseExited:(NSEvent *)theEvent
 {
     /* give up our focus, so the vout may show us again without letting the user clicking it */
-    vout_thread_t *p_vout = getVout();
-    if (p_vout)
-    {
-        if (var_GetBool( p_vout, "fullscreen" ))
-            [[[[VLCMainWindow sharedInstance] videoView] window] makeKeyWindow];
-        vlc_object_release( p_vout );
-    }
+    if (o_vout_window && var_GetBool(pl_Get(VLCIntf), "fullscreen"))
+        [o_vout_window makeKeyWindow];
 }
 
 - (void)hideMouse
@@ -263,22 +260,20 @@
 {
     /* in case that the user don't want us to appear, make sure we hide the mouse */
 
-    if( !config_GetInt( VLCIntf, "macosx-fspanel" ) )
-    {
-        float time = (float)var_CreateGetInteger( VLCIntf, "mouse-hide-timeout" ) / 1000.;
+    if (!config_GetInt(VLCIntf, "macosx-fspanel")) {
+        float time = (float)var_CreateGetInteger(VLCIntf, "mouse-hide-timeout") / 1000.;
         [self setFadeTimer:[NSTimer scheduledTimerWithTimeInterval:time target:self selector:@selector(hideMouse) userInfo:nil repeats:NO]];
         return;
     }
 
-    if( b_nonActive )
+    if (b_nonActive)
         return;
 
     [self orderFront: nil];
 
-    if( [self alphaValue] < 1.0 || b_displayed != YES )
-    {
+    if ([self alphaValue] < 1.0 || b_displayed != YES) {
         if (![self fadeTimer])
-            [self setFadeTimer:[NSTimer scheduledTimerWithTimeInterval:0.05 target:self selector:@selector(focus:) userInfo:[NSNumber numberWithShort:1] repeats:YES]];
+            [self setFadeTimer:[NSTimer scheduledTimerWithTimeInterval:0.05 target:self selector:@selector(focus:) userInfo:[NSNumber numberWithInt:1] repeats:YES]];
         else if ([[[self fadeTimer] userInfo] shortValue]==0)
             b_fadeQueued=YES;
     }
@@ -287,13 +282,12 @@
 
 - (void)fadeOut
 {
-    if( NSPointInRect([NSEvent mouseLocation],[self frame]))
+    if (NSPointInRect([NSEvent mouseLocation],[self frame]))
         return;
 
-    if( ( [self alphaValue] > 0.0 ) )
-    {
+    if (([self alphaValue] > 0.0)) {
         if (![self fadeTimer])
-            [self setFadeTimer:[NSTimer scheduledTimerWithTimeInterval:0.05 target:self selector:@selector(unfocus:) userInfo:[NSNumber numberWithShort:0] repeats:YES]];
+            [self setFadeTimer:[NSTimer scheduledTimerWithTimeInterval:0.05 target:self selector:@selector(unfocus:) userInfo:[NSNumber numberWithInt:0] repeats:YES]];
         else if ([[[self fadeTimer] userInfo] shortValue]==1)
             b_fadeQueued=YES;
     }
@@ -306,19 +300,17 @@
     b_keptVisible = YES;
 
     /* get us a valid timer */
-    if(! b_alreadyCounting )
-    {
-        i_timeToKeepVisibleInSec = var_CreateGetInteger( VLCIntf, "mouse-hide-timeout" ) / 500;
-        if( hideAgainTimer )
-        {
+    if (!b_alreadyCounting) {
+        i_timeToKeepVisibleInSec = var_CreateGetInteger(VLCIntf, "mouse-hide-timeout") / 500;
+        if (hideAgainTimer) {
             [hideAgainTimer invalidate];
             [hideAgainTimer autorelease];
         }
         /* released in -autoHide and -dealloc */
         hideAgainTimer = [[NSTimer scheduledTimerWithTimeInterval: 0.5
-                                                          target: self 
+                                                          target: self
                                                         selector: @selector(keepVisible:)
-                                                        userInfo: nil 
+                                                        userInfo: nil
                                                          repeats: YES] retain];
         b_alreadyCounting = YES;
     }
@@ -327,12 +319,11 @@
 - (void)keepVisible:(NSTimer *)timer
 {
     /* if the user triggered an action, start over again */
-    if( b_keptVisible )
+    if (b_keptVisible)
         b_keptVisible = NO;
 
     /* count down until we hide ourselfes again and do so if necessary */
-    if( --i_timeToKeepVisibleInSec < 1 )
-    {
+    if (--i_timeToKeepVisibleInSec < 1) {
         [self hideMouse];
         [self fadeOut];
         [hideAgainTimer invalidate]; /* released in -autoHide and -dealloc */
@@ -367,19 +358,18 @@
     [self setFrameOrigin:point];
 }
 
-- (BOOL)isDisplayed
-{
-    return b_displayed;
-}
-
-- (void)setVoutWasUpdated: (int)i_newdevice;
+- (void)setVoutWasUpdated: (VLCWindow *)o_window
 {
     b_voutWasUpdated = YES;
-    if( i_newdevice != i_device )
-    {
+    if (o_vout_window)
+        [o_vout_window release];
+    o_vout_window = [o_window retain];
+    int i_newdevice = (int)[[o_vout_window screen] displayID];
+    if ((i_newdevice != i_device && i_device != 0) || i_newdevice != [[self screen] displayID]) {
         i_device = i_newdevice;
         [self center];
-    }
+    } else
+        i_device = i_newdevice;
 }
 @end
 
@@ -388,7 +378,7 @@
  *****************************************************************************/
 @implementation VLCFSPanelView
 
-#define addButton( o_button, imageOff, imageOn, _x, _y, action )                                \
+#define addButton(o_button, imageOff, imageOn, _x, _y, action, AXDesc, ToolTip)               \
     s_rc.origin.x = _x;                                                                         \
     s_rc.origin.y = _y;                                                                         \
     o_button = [[NSButton alloc] initWithFrame: s_rc];                                 \
@@ -401,9 +391,12 @@
     [o_button sizeToFit];                                                                       \
     [o_button setTarget: self];                                                                 \
     [o_button setAction: @selector(action:)];                                                   \
+    [[o_button cell] accessibilitySetOverrideValue:AXDesc forAttribute:NSAccessibilityDescriptionAttribute]; \
+    [[o_button cell] accessibilitySetOverrideValue:ToolTip forAttribute:NSAccessibilityTitleAttribute]; \
+    [o_button setToolTip: ToolTip]; \
     [self addSubview:o_button];
 
-#define addTextfield( class, o_text, align, font, color )                                    \
+#define addTextfield(class, o_text, align, font, color)                                    \
     o_text = [[class alloc] initWithFrame: s_rc];                            \
     [o_text setDrawsBackground: NO];                                                        \
     [o_text setBordered: NO];                                                               \
@@ -420,24 +413,27 @@
     id view = [super initWithFrame:frameRect];
     fillColor = [[NSColor clearColor] retain];
     NSRect s_rc = [self frame];
-    addButton( o_prev, @"fs_skip_previous" , @"fs_skip_previous_highlight", 174, 15, prev );
-    addButton( o_bwd, @"fs_rewind"        , @"fs_rewind_highlight"       , 211, 14, backward );
-    addButton( o_play, @"fs_play"          , @"fs_play_highlight"         , 265, 10, play );
-    addButton( o_fwd, @"fs_forward"       , @"fs_forward_highlight"      , 313, 14, forward );
-    addButton( o_next, @"fs_skip_next"     , @"fs_skip_next_highlight"    , 365, 15, next );
-    addButton( o_fullscreen, @"fs_exit_fullscreen", @"fs_exit_fullscreen_hightlight", 507, 13, toggleFullscreen );
+    addButton(o_prev, @"fs_skip_previous_highlight" , @"fs_skip_previous", 174, 15, prev, _NS("Click to go to the previous playlist item."), _NS("Previous"));
+    addButton(o_bwd, @"fs_rewind_highlight"        , @"fs_rewind"       , 211, 14, backward, _NS("Click and hold to skip backward through the current media."), _NS("Backward"));
+    addButton(o_play, @"fs_play_highlight"          , @"fs_play"         , 265, 10, play, _NS("Click to play or pause the current media."), _NS("Play/Pause"));
+    addButton(o_fwd, @"fs_forward_highlight"       , @"fs_forward"      , 313, 14, forward, _NS("Click and hold to skip forward through the current media."), _NS("Forward"));
+    addButton(o_next, @"fs_skip_next_highlight"     , @"fs_skip_next"    , 365, 15, next, _NS("Click to go to the next playlist item."), _NS("Next"));
+    addButton(o_fullscreen, @"fs_exit_fullscreen_highlight", @"fs_exit_fullscreen", 507, 13, toggleFullscreen, _NS("Click to exit fullscreen playback."), _NS("Toggle Fullscreen mode"));
 /*
-    addButton( o_button, @"image (off state)", @"image (on state)", 38, 51, something );
+    addButton(o_button, @"image (off state)", @"image (on state)", 38, 51, something, accessibility help string, usual tool tip);
  */
     [o_fwd setContinuous:YES];
     [o_bwd setContinuous:YES];
 
     /* time slider */
-    s_rc = [self frame];
+    // (surrounding progress view for swipe behaviour)
     s_rc.origin.x = 15;
     s_rc.origin.y = 45;
     s_rc.size.width = 518;
     s_rc.size.height = 13;
+    o_progress_view = [[VLCProgressView alloc] initWithFrame: s_rc];
+    s_rc.origin.x = 0;
+    s_rc.origin.y = 0;
     o_fs_timeSlider = [[VLCFSTimeSlider alloc] initWithFrame: s_rc];
     [o_fs_timeSlider setMinValue:0];
     [o_fs_timeSlider setMaxValue:10000];
@@ -445,7 +441,10 @@
     [o_fs_timeSlider setContinuous: YES];
     [o_fs_timeSlider setTarget: self];
     [o_fs_timeSlider setAction: @selector(fsTimeSliderUpdate:)];
-    [self addSubview: o_fs_timeSlider];
+    [[o_fs_volumeSlider cell] accessibilitySetOverrideValue:_NS("Position") forAttribute:NSAccessibilityTitleAttribute];
+    [[o_fs_timeSlider cell] accessibilitySetOverrideValue:_NS("Click and move the mouse while keeping the button pressed to use this slider to change current playback position.") forAttribute:NSAccessibilityDescriptionAttribute];
+    [self addSubview: o_progress_view];
+    [o_progress_view addSubview: o_fs_timeSlider];
 
     /* volume slider */
     s_rc = [self frame];
@@ -455,29 +454,38 @@
     s_rc.size.height = 12;
     o_fs_volumeSlider = [[VLCFSVolumeSlider alloc] initWithFrame: s_rc];
     [o_fs_volumeSlider setMinValue:0];
-    [o_fs_volumeSlider setMaxValue:AOUT_VOLUME_MAX];
+    [o_fs_volumeSlider setMaxValue: [[VLCCoreInteraction sharedInstance] maxVolume]];
     [o_fs_volumeSlider setIntValue:AOUT_VOLUME_DEFAULT];
     [o_fs_volumeSlider setContinuous: YES];
     [o_fs_volumeSlider setTarget: self];
     [o_fs_volumeSlider setAction: @selector(fsVolumeSliderUpdate:)];
+    [o_fs_volumeSlider setUsesBrightArtwork:NO];
+    [[o_fs_volumeSlider cell] accessibilitySetOverrideValue:_NS("Volume") forAttribute:NSAccessibilityTitleAttribute];
+    [[o_fs_volumeSlider cell] accessibilitySetOverrideValue:_NS("Click and move the mouse while keeping the button pressed to use this slider to change the volume.") forAttribute:NSAccessibilityDescriptionAttribute];
     [self addSubview: o_fs_volumeSlider];
 
     /* time counter and stream title output fields */
     s_rc = [self frame];
-    s_rc.origin.x = 98;
+    // 10 px gap between time fields
+    s_rc.origin.x = 90;
     s_rc.origin.y = 64;
-    s_rc.size.width = 352;
+    s_rc.size.width = 361;
     s_rc.size.height = 14;
-    addTextfield( NSTextField, o_streamTitle_txt, NSCenterTextAlignment, systemFontOfSize, whiteColor );
+    addTextfield(NSTextField, o_streamTitle_txt, NSCenterTextAlignment, systemFontOfSize, whiteColor);
+    s_rc.origin.x = 15;
+    s_rc.origin.y = 64;
+    s_rc.size.width = 65;
+    addTextfield(VLCTimeField, o_streamPosition_txt, NSLeftTextAlignment, systemFontOfSize, whiteColor);
     s_rc.origin.x = 471;
     s_rc.origin.y = 64;
     s_rc.size.width = 65;
-    addTextfield( VLCTimeField, o_streamPosition_txt, NSRightTextAlignment, systemFontOfSize, whiteColor );
+    addTextfield(VLCTimeField, o_streamLength_txt, NSRightTextAlignment, systemFontOfSize, whiteColor);
+    [o_streamLength_txt setRemainingIdentifier: @"DisplayFullscreenTimeAsTimeRemaining"];
 
     o_background_img = [[NSImage imageNamed:@"fs_background"] retain];
     o_vol_sld_img = [[NSImage imageNamed:@"fs_volume_slider_bar"] retain];
-    o_vol_mute_img = [[NSImage imageNamed:@"fs_volume_mute"] retain];
-    o_vol_max_img = [[NSImage imageNamed:@"fs_volume_max"] retain];
+    o_vol_mute_img = [[NSImage imageNamed:@"fs_volume_mute_highlight"] retain];
+    o_vol_max_img = [[NSImage imageNamed:@"fs_volume_max_highlight"] retain];
     o_time_sld_img = [[NSImage imageNamed:@"fs_time_slider"] retain];
 
     return view;
@@ -505,14 +513,14 @@
 
 - (void)setPlay
 {
-    [o_play setImage:[NSImage imageNamed:@"fs_play"]];
-    [o_play setAlternateImage: [NSImage imageNamed:@"fs_play_highlight"]];
+    [o_play setImage:[NSImage imageNamed:@"fs_play_highlight"]];
+    [o_play setAlternateImage: [NSImage imageNamed:@"fs_play"]];
 }
 
 - (void)setPause
 {
-    [o_play setImage: [NSImage imageNamed:@"fs_pause"]];
-    [o_play setAlternateImage: [NSImage imageNamed:@"fs_pause_highlight"]];
+    [o_play setImage: [NSImage imageNamed:@"fs_pause_highlight"]];
+    [o_play setAlternateImage: [NSImage imageNamed:@"fs_pause"]];
 }
 
 - (void)setStreamTitle:(NSString *)o_title
@@ -520,10 +528,54 @@
     [o_streamTitle_txt setStringValue: o_title];
 }
 
-- (void)setStreamPos:(float) f_pos andTime:(NSString *)o_time
+- (void)updatePositionAndTime
 {
-    [o_streamPosition_txt setStringValue: o_time];
-    [o_fs_timeSlider setFloatValue: f_pos];
+    input_thread_t * p_input;
+    p_input = pl_CurrentInput(VLCIntf);
+    if (p_input) {
+        
+        vlc_value_t pos;
+        float f_updated;
+
+        var_Get(p_input, "position", &pos);
+        f_updated = 10000. * pos.f_float;
+        [o_fs_timeSlider setFloatValue: f_updated];
+
+        vlc_value_t time;
+        char psz_time[MSTRTIME_MAX_SIZE];
+
+        var_Get(p_input, "time", &time);
+        mtime_t dur = input_item_GetDuration(input_GetItem(p_input));
+
+        // update total duration (right field)
+        if(dur <= 0) {
+            [o_streamLength_txt setHidden: YES];
+        } else {
+            [o_streamLength_txt setHidden: NO];
+
+            NSString *o_total_time;
+            if ([o_streamLength_txt timeRemaining]) {
+                mtime_t remaining = 0;
+                if (dur > time.i_time)
+                    remaining = dur - time.i_time;
+                o_total_time = [NSString stringWithFormat: @"-%s", secstotimestr(psz_time, (remaining / 1000000))];
+            } else
+                o_total_time = [NSString stringWithUTF8String:secstotimestr(psz_time, (dur / 1000000))];
+
+            [o_streamLength_txt setStringValue: o_total_time];
+        }
+
+        // update current position (left field)
+        NSString *o_playback_pos = [NSString stringWithUTF8String:secstotimestr(psz_time, (time.i_time / 1000000))];
+               
+        [o_streamPosition_txt setStringValue: o_playback_pos];
+        vlc_object_release(p_input);
+    } else {
+        [o_fs_timeSlider setFloatValue: 0.0];
+        [o_streamPosition_txt setStringValue: @"00:00"];
+        [o_streamLength_txt setHidden: YES];
+    }
+
 }
 
 - (void)setSeekable:(BOOL)b_seekable
@@ -540,13 +592,12 @@
 
 - (IBAction)play:(id)sender
 {
-    [[VLCCoreInteraction sharedInstance] play];
+    [[VLCCoreInteraction sharedInstance] playOrPause];
 }
 
 - (IBAction)forward:(id)sender
 {
-    if (([NSDate timeIntervalSinceReferenceDate] - last_fwd_event) > 0.16 )
-    {
+    if (([NSDate timeIntervalSinceReferenceDate] - last_fwd_event) > 0.16) {
         // we just skipped 4 "continous" events, otherwise we are too fast
         [[VLCCoreInteraction sharedInstance] forwardExtraShort];
         last_fwd_event = [NSDate timeIntervalSinceReferenceDate];
@@ -555,8 +606,7 @@
 
 - (IBAction)backward:(id)sender
 {
-    if (([NSDate timeIntervalSinceReferenceDate] - last_bwd_event) > 0.16 )
-    {
+    if (([NSDate timeIntervalSinceReferenceDate] - last_bwd_event) > 0.16) {
         // we just skipped 4 "continous" events, otherwise we are too fast
         [[VLCCoreInteraction sharedInstance] backwardExtraShort];
         last_bwd_event = [NSDate timeIntervalSinceReferenceDate];
@@ -581,14 +631,13 @@
 - (IBAction)fsTimeSliderUpdate:(id)sender
 {
     input_thread_t * p_input;
-    p_input = pl_CurrentInput( VLCIntf );
-    if( p_input != NULL )
-    {
+    p_input = pl_CurrentInput(VLCIntf);
+    if (p_input != NULL) {
         vlc_value_t pos;
 
         pos.f_float = [o_fs_timeSlider floatValue] / 10000.;
-        var_Set( p_input, "position", pos );
-        vlc_object_release( p_input );
+        var_Set(p_input, "position", pos);
+        vlc_object_release(p_input);
     }
     [[VLCMain sharedInstance] updatePlaybackPosition];
 }
@@ -614,11 +663,11 @@
     NSRect image_rect;
     NSSize image_size;
     NSImage *img;
-    addImage( o_background_img, 0, 0, NSCompositeCopy );
-    addImage( o_vol_sld_img, 26, 23, NSCompositeSourceOver );
-    addImage( o_vol_mute_img, 16, 18, NSCompositeSourceOver );
-    addImage( o_vol_max_img, 124, 18, NSCompositeSourceOver );
-    addImage( o_time_sld_img, 15, 45, NSCompositeSourceOver );
+    addImage(o_background_img, 0, 0, NSCompositeCopy);
+    addImage(o_vol_sld_img, 26, 23, NSCompositeSourceOver);
+    addImage(o_vol_mute_img, 16, 18, NSCompositeSourceOver);
+    addImage(o_vol_max_img, 124, 18, NSCompositeSourceOver);
+    addImage(o_time_sld_img, 15, 45, NSCompositeSourceOver);
 }
 
 @end
@@ -660,10 +709,21 @@
 * VLCFSVolumeSlider
 *****************************************************************************/
 @implementation VLCFSVolumeSlider
+
+- (id)initWithFrame:(NSRect)frame
+{
+    self = [super initWithFrame:frame];
+    if(self) {
+        [self setCell:[[[VolumeSliderCell alloc] init] autorelease]];
+    }
+
+    return self;
+}
+
 - (void)drawKnobInRect:(NSRect) knobRect
 {
     NSRect image_rect;
-    NSImage *img = [NSImage imageNamed:@"fs_volume_slider_knob"];
+    NSImage *img = [NSImage imageNamed:@"fs_volume_slider_knob_highlight"];
     image_rect.size = [img size];
     image_rect.origin.x = 0;
     image_rect.origin.y = 0;
@@ -681,11 +741,20 @@
     [super drawRect:rect];
     [[NSGraphicsContext currentContext] restoreGraphicsState];
 
+    [self drawFullVolumeMarker];
+
     NSRect knobRect = [[self cell] knobRectFlipped:NO];
     knobRect.origin.y+=7.5;
     [[[NSColor blackColor] colorWithAlphaComponent:0.6] set];
     [self drawKnobInRect: knobRect];
 }
 
+- (void)drawFullVolBezierPath:(NSBezierPath*)bezierPath
+{
+    CGFloat fullVolPos = [self fullVolumePos];
+    [bezierPath moveToPoint:NSMakePoint(fullVolPos, [self frame].size.height)];
+    [bezierPath lineToPoint:NSMakePoint(fullVolPos, 1.)];
+}
+
 @end
 
