@@ -1,11 +1,12 @@
 /*****************************************************************************
  * prefs.m: MacOS X module for vlc
  *****************************************************************************
- * Copyright (C) 2002-2006 VLC authors and VideoLAN
- * $Id: 96ce5e3431726dfeb7af5a3b80af9ecf00bc26bf $
+ * Copyright (C) 2002-2012 VLC authors and VideoLAN
+ * $Id: 0b87421d65f4ce6ff76a468d9d24942dfd08816d $
  *
  * Authors: Jon Lech Johansen <jon-vl@nanocrew.net>
  *          Derk-Jan Hartman <hartman at videolan dot org>
+ *          Felix Paul Kühne <fkuehne at videolan dot org>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -60,6 +61,7 @@
 #import "CoreInteraction.h"
 #import <vlc_keys.h>
 #import <vlc_modules.h>
+#import <vlc_plugin.h>
 
 /* /!\ Warning: Unreadable code :/ */
 
@@ -152,11 +154,9 @@ static VLCPrefs *_o_sharedMainInstance = nil;
 
 - (id)init
 {
-    if( _o_sharedMainInstance ) {
+    if (_o_sharedMainInstance)
         [self dealloc];
-    }
-    else
-    {
+    else {
         _o_sharedMainInstance = [super init];
         p_intf = VLCIntf;
         o_empty_view = [[NSView alloc] init];
@@ -177,7 +177,7 @@ static VLCPrefs *_o_sharedMainInstance = nil;
 {
     p_intf = VLCIntf;
 
-    if (OSX_LION)
+    if (!OSX_SNOW_LEOPARD)
         [o_prefs_window setCollectionBehavior: NSWindowCollectionBehaviorFullScreenAuxiliary];
 
     [self initStrings];
@@ -215,7 +215,7 @@ static VLCPrefs *_o_sharedMainInstance = nil;
     /* TODO: call savePrefs on Root item */
     [_rootTreeItem applyChanges];
     [[VLCCoreInteraction sharedInstance] fixPreferences];
-    config_SaveConfigFile( p_intf );
+    config_SaveConfigFile(p_intf);
     [o_prefs_window orderOut:self];
 }
 
@@ -224,31 +224,6 @@ static VLCPrefs *_o_sharedMainInstance = nil;
     [o_prefs_window orderOut:self];
 }
 
-- (IBAction)resetAll: (id)sender
-{
-    NSBeginInformationalAlertSheet(_NS("Reset Preferences"), _NS("Cancel"),
-        _NS("Continue"), nil, o_prefs_window, self,
-        @selector(sheetDidEnd: returnCode: contextInfo:), NULL, nil,
-        _NS("Beware this will reset the VLC media player preferences.\n"
-            "Are you sure you want to continue?") );
-}
-
-- (void)sheetDidEnd:(NSWindow *)o_sheet returnCode:(int)i_return
-    contextInfo:(void *)o_context
-{
-    if( i_return == NSAlertAlternateReturn )
-    {
-        /* reset VLC's config */
-        config_ResetAll( p_intf );
-        [_rootTreeItem resetView];
-        config_SaveConfigFile( p_intf );
-
-        /* reset OS X defaults */
-        [NSUserDefaults resetStandardUserDefaults];
-        [[NSUserDefaults standardUserDefaults] synchronize];
-    }
-}
-
 - (IBAction)buttonAction: (id)sender
 {
     [o_prefs_window orderOut: self];
@@ -303,10 +278,9 @@ static VLCPrefs *_o_sharedMainInstance = nil;
 - (VLCTreeCategoryItem *)itemRepresentingCategory:(int)category
 {
     NSUInteger childrenCount = [[self children] count];
-    for( int i = 0; i < childrenCount; i++ )
-    {
+    for (int i = 0; i < childrenCount; i++) {
         VLCTreeCategoryItem * categoryItem = [[self children] objectAtIndex:i];
-        if( [categoryItem category] == category )
+        if ([categoryItem category] == category)
             return categoryItem;
     }
     return nil;
@@ -314,14 +288,13 @@ static VLCPrefs *_o_sharedMainInstance = nil;
 
 - (bool)isSubCategoryGeneral:(int)category
 {
-    if(category == SUBCAT_VIDEO_GENERAL ||
+    if (category == SUBCAT_VIDEO_GENERAL ||
           category == SUBCAT_ADVANCED_MISC ||
           category == SUBCAT_INPUT_GENERAL ||
           category == SUBCAT_INTERFACE_GENERAL ||
           category == SUBCAT_SOUT_GENERAL||
           category == SUBCAT_PLAYLIST_GENERAL||
-          category == SUBCAT_AUDIO_GENERAL )
-    {
+          category == SUBCAT_AUDIO_GENERAL) {
         return true;
     }
     return false;
@@ -331,20 +304,19 @@ static VLCPrefs *_o_sharedMainInstance = nil;
  * Loads children incrementally */
 - (NSMutableArray *)children
 {
-    if( _children ) return _children;
+    if (_children) return _children;
     _children = [[NSMutableArray alloc] init];
 
     intf_thread_t   *p_intf = VLCIntf;
 
     /* List the modules */
     size_t count, i;
-    module_t ** modules = module_list_get( &count );
-    if( !modules ) return nil;
+    module_t ** modules = module_list_get(&count);
+    if (!modules) return nil;
 
     /* Build a tree of the plugins */
     /* Add the capabilities */
-    for( i = 0; i < count; i++ )
-    {
+    for (i = 0; i < count; i++) {
         VLCTreeCategoryItem * categoryItem = nil;
         VLCTreeSubCategoryItem * subCategoryItem = nil;
         VLCTreePluginItem * pluginItem = nil;
@@ -356,9 +328,9 @@ static VLCPrefs *_o_sharedMainInstance = nil;
 
         /* Exclude empty plugins (submodules don't have config */
         /* options, they are stored in the parent module) */
-        if( module_is_main( p_module ) ) {
+        if (module_is_main(p_module)) {
             pluginItem = self;
-            _configItems = module_config_get( p_module, &confsize );
+            _configItems = module_config_get(p_module, &confsize);
             _configSize = confsize;
         } else {
             pluginItem = [VLCTreePluginItem pluginTreeItemWithPlugin: p_module];
@@ -367,55 +339,44 @@ static VLCPrefs *_o_sharedMainInstance = nil;
         p_configs = [pluginItem configItems];
 
         unsigned int j;
-        for( j = 0; j < confsize; j++ )
-        {
+        for (j = 0; j < confsize; j++) {
             int configType = p_configs[j].i_type;
-            if( configType == CONFIG_CATEGORY )
-            {
+            if (configType == CONFIG_CATEGORY) {
                 categoryItem = [self itemRepresentingCategory:p_configs[j].value.i];
-                if(!categoryItem)
-                {
+                if (!categoryItem) {
                     categoryItem = [VLCTreeCategoryItem categoryTreeItemWithCategory:p_configs[j].value.i];
-                    if(categoryItem) [[self children] addObject:categoryItem];
+                    if (categoryItem)
+                        [[self children] addObject:categoryItem];
                 }
             }
-            else if( configType == CONFIG_SUBCATEGORY )
-            {
+            else if (configType == CONFIG_SUBCATEGORY) {
                 lastsubcat = p_configs[j].value.i;
-                if( categoryItem && ![self isSubCategoryGeneral:lastsubcat] )
-                {
+                if (categoryItem && ![self isSubCategoryGeneral:lastsubcat]) {
                     subCategoryItem = [categoryItem itemRepresentingSubCategory:lastsubcat];
-                    if(!subCategoryItem)
-                    {
+                    if (!subCategoryItem) {
                         subCategoryItem = [VLCTreeSubCategoryItem subCategoryTreeItemWithSubCategory:lastsubcat];
-                        if(subCategoryItem) [[categoryItem children] addObject:subCategoryItem];
+                        if (subCategoryItem)
+                            [[categoryItem children] addObject:subCategoryItem];
                     }
                 }
             }
 
-            if( module_is_main( p_module) && CONFIG_ITEM(configType) )
-            {
-                if( categoryItem && [self isSubCategoryGeneral:lastsubcat] )
-                {
+            if (module_is_main(p_module) && (CONFIG_ITEM(configType) || configType == CONFIG_SECTION)) {
+                if (categoryItem && [self isSubCategoryGeneral:lastsubcat])
                     [[categoryItem options] addObject:[[VLCTreeLeafItem alloc] initWithConfigItem:&p_configs[j]]];
-                }
-                else if( subCategoryItem )
-                {
+                else if (subCategoryItem)
                     [[subCategoryItem options] addObject:[[VLCTreeLeafItem alloc] initWithConfigItem:&p_configs[j]]];
-                }
             }
-            else if( !module_is_main( p_module) && CONFIG_ITEM(configType))
-            {
-                if( ![[subCategoryItem children] containsObject: pluginItem] )
-                {
+            else if (!module_is_main(p_module) && (CONFIG_ITEM(configType) || configType == CONFIG_SECTION)) {
+                if (![[subCategoryItem children] containsObject: pluginItem])
                     [[subCategoryItem children] addObject:pluginItem];
-                }
-                if( pluginItem )
+
+                if (pluginItem)
                     [[pluginItem options] addObject:[[VLCTreeLeafItem alloc] initWithConfigItem:&p_configs[j]]];
             }
         }
     }
-    module_list_free( modules );
+    module_list_free(modules);
     return _children;
 }
 @end
@@ -424,28 +385,27 @@ static VLCPrefs *_o_sharedMainInstance = nil;
 @implementation VLCTreeCategoryItem
 + (VLCTreeCategoryItem *)categoryTreeItemWithCategory:(int)category
 {
-    if(!config_CategoryNameGet( category )) return nil;
+    if (!config_CategoryNameGet(category)) return nil;
     return [[[[self class] alloc] initWithCategory:category] autorelease];
 }
+
 - (id)initWithCategory:(int)category
 {
-    NSString * name = _NS(config_CategoryNameGet( category ));
-    if(self = [super initWithName:name])
-    {
+    NSString * name = _NS(config_CategoryNameGet(category));
+    if (self = [super initWithName:name]) {
         _category = category;
-        //_help = [_NS(config_CategoryHelpGet( category )) retain];
+        //_help = [_NS(config_CategoryHelpGet(category)) retain];
     }
     return self;
 }
 
 - (VLCTreeSubCategoryItem *)itemRepresentingSubCategory:(int)subCategory
 {
-    assert( [self isKindOfClass:[VLCTreeCategoryItem class]] );
+    assert([self isKindOfClass:[VLCTreeCategoryItem class]]);
     NSUInteger childrenCount = [[self children] count];
-    for( NSUInteger i = 0; i < childrenCount; i++ )
-    {
+    for (NSUInteger i = 0; i < childrenCount; i++) {
         VLCTreeSubCategoryItem * subCategoryItem = [[self children] objectAtIndex:i];
-        if( [subCategoryItem subCategory] == subCategory )
+        if ([subCategoryItem subCategory] == subCategory)
             return subCategoryItem;
     }
     return nil;
@@ -461,18 +421,18 @@ static VLCPrefs *_o_sharedMainInstance = nil;
 @implementation VLCTreeSubCategoryItem
 - (id)initWithSubCategory:(int)subCategory
 {
-    NSString * name = _NS(config_CategoryNameGet( subCategory ));
-    if(self = [super initWithName:name])
-    {
+    NSString * name = _NS(config_CategoryNameGet(subCategory));
+    if (self = [super initWithName:name]) {
         _subCategory = subCategory;
-        //_help = [_NS(config_CategoryHelpGet( subCategory )) retain];
+        //_help = [_NS(config_CategoryHelpGet(subCategory)) retain];
     }
     return self;
 }
 
 + (VLCTreeSubCategoryItem *)subCategoryTreeItemWithSubCategory:(int)subCategory
 {
-    if(!config_CategoryNameGet( subCategory )) return nil;
+    if (!config_CategoryNameGet(subCategory))
+        return nil;
     return [[[[self class] alloc] initWithSubCategory:subCategory] autorelease];
 }
 
@@ -487,12 +447,11 @@ static VLCPrefs *_o_sharedMainInstance = nil;
 @implementation VLCTreePluginItem
 - (id)initWithPlugin:(module_t *)plugin
 {
-    NSString * name = _NS( module_get_name( plugin, false ) );
-    if(self = [super initWithName:name])
-    {
-        _configItems = module_config_get( plugin, &_configSize );
+    NSString * name = _NS(module_get_name(plugin, false));
+    if (self = [super initWithName:name]) {
+        _configItems = module_config_get(plugin, &_configSize);
         //_plugin = plugin;
-        //_help = [_NS(config_CategoryHelpGet( subCategory )) retain];
+        //_help = [_NS(config_CategoryHelpGet(subCategory)) retain];
     }
     return self;
 }
@@ -504,7 +463,7 @@ static VLCPrefs *_o_sharedMainInstance = nil;
 
 - (void)dealloc
 {
-    module_config_free( _configItems );
+    module_config_free(_configItems);
     [super dealloc];
 }
 
@@ -528,10 +487,9 @@ static VLCPrefs *_o_sharedMainInstance = nil;
     NSString * name = _NS(configItem->psz_name);
     self = [super initWithName:name];
     [name release];
-    if( self != nil )
-    {
+    if (self != nil)
         _configItem = configItem;
-    }
+
     return self;
 }
 
@@ -548,10 +506,9 @@ static VLCPrefs *_o_sharedMainInstance = nil;
 - (id)initWithName:(NSString*)name
 {
     self = [super init];
-    if( self != nil )
-    {
+    if (self != nil)
         _name = [name retain];
-    }
+
     return self;
 }
 
@@ -589,19 +546,16 @@ static VLCPrefs *_o_sharedMainInstance = nil;
     view = [[VLCFlippedView alloc] initWithFrame: s_vrc];
     [view setAutoresizingMask: NSViewWidthSizable | NSViewMinYMargin | NSViewMaxYMargin];
 
-    if(!_subviews)
-    {
+    if (!_subviews) {
         _subviews = [[NSMutableArray alloc] initWithCapacity:10];
 
         NSUInteger count = [[self options] count];
-        for( NSUInteger i = 0; i < count; i++)
-        {
+        for (NSUInteger i = 0; i < count; i++) {
             VLCTreeLeafItem * item = [[self options] objectAtIndex:i];
 
             VLCConfigControl *control;
             control = [VLCConfigControl newControl:[item configItem] withView:view];
-            if( control )
-            {
+            if (control) {
                 [control setAutoresizingMask: NSViewMaxYMargin | NSViewWidthSizable];
                 [_subviews addObject: control];
             }
@@ -618,13 +572,13 @@ static VLCPrefs *_o_sharedMainInstance = nil;
     VLCConfigControl *widget;
     NSRect frame;
 
-    while( ( widget = [enumerator nextObject] ) )
-        if( i_max_label < [widget labelSize] )
+    while((widget = [enumerator nextObject])) {
+        if (i_max_label < [widget labelSize])
             i_max_label = [widget labelSize];
+    }
 
     enumerator = [_subviews objectEnumerator];
-    while( ( widget = [enumerator nextObject] ) )
-    {
+    while((widget = [enumerator nextObject])) {
         int i_widget;
 
         i_widget = [widget viewType];
@@ -649,40 +603,41 @@ static VLCPrefs *_o_sharedMainInstance = nil;
 {
     NSUInteger i;
     NSUInteger count = [_subviews count];
-    for( i = 0 ; i < count ; i++ )
+    for (i = 0 ; i < count ; i++)
         [[_subviews objectAtIndex:i] applyChanges];
 
     count = [_children count];
-    for( i = 0 ; i < count ; i++ )
+    for (i = 0 ; i < count ; i++)
         [[_children objectAtIndex:i] applyChanges];
 }
 
 - (void)resetView
 {
-    NSUInteger i;
     NSUInteger count = [_subviews count];
-    for( i = 0 ; i < count ; i++ )
+    for (NSUInteger i = 0 ; i < count ; i++)
         [[_subviews objectAtIndex:i] resetValues];
 
     count = [_options count];
-    for( i = 0 ; i < count ; i++ )
+    for (NSUInteger i = 0 ; i < count ; i++)
         [[_options objectAtIndex:i] resetView];
 
     count = [_children count];
-    for( i = 0 ; i < count ; i++ )
+    for (NSUInteger i = 0 ; i < count ; i++)
         [[_children objectAtIndex:i] resetView];
 
 }
 
 - (NSMutableArray *)children
 {
-    if(!_children) _children = [[NSMutableArray alloc] init];
+    if (!_children)
+        _children = [[NSMutableArray alloc] init];
     return _children;
 }
 
 - (NSMutableArray *)options
 {
-    if(!_options) _options = [[NSMutableArray alloc] init];
+    if (!_options)
+        _options = [[NSMutableArray alloc] init];
     return _options;
 }
 @end
@@ -692,7 +647,7 @@ static VLCPrefs *_o_sharedMainInstance = nil;
 
 - (BOOL)isFlipped
 {
-    return( YES );
+    return(YES);
 }
 
 @end
