@@ -1,8 +1,8 @@
 /*****************************************************************************
- * Controller.cpp : Controller for the main interface
+ * controller.cpp : Controller for the main interface
  ****************************************************************************
  * Copyright (C) 2006-2009 the VideoLAN team
- * $Id: dd75161ff3b58d155acdd5ae17257d0b9e092b6a $
+ * $Id: 2928bd52438a13f8b1e5b9aec9b27dce081912b2 $
  *
  * Authors: Jean-Baptiste Kempf <jb@videolan.org>
  *          Ilkka Ollakka <ileoo@videolan.org>
@@ -109,7 +109,7 @@ void AbstractController::setupButton( QAbstractButton *aButton )
 /* Open the generic config line for the toolbar, parse it
  * and create the widgets accordingly */
 void AbstractController::parseAndCreate( const QString& config,
-                                         QBoxLayout *controlLayout )
+                                         QBoxLayout *newControlLayout )
 {
     QStringList list = config.split( ";", QString::SkipEmptyParts ) ;
     for( int i = 0; i < list.count(); i++ )
@@ -126,7 +126,7 @@ void AbstractController::parseAndCreate( const QString& config,
         buttonType_e i_type = (buttonType_e)list2.at( 0 ).toInt( &ok );
         if( !ok )
         {
-            msg_Warn( p_intf, "Parsing error 2. Please report this." );
+            msg_Warn( p_intf, "Parsing error 2. Please, report this." );
             continue;
         }
 
@@ -140,17 +140,17 @@ void AbstractController::parseAndCreate( const QString& config,
             }
         }
 
-        createAndAddWidget( controlLayout, -1, i_type, i_option );
+        createAndAddWidget( newControlLayout, -1, i_type, i_option );
     }
 
     if( buttonGroupLayout )
     {
-        controlLayout->addLayout( buttonGroupLayout );
+        newControlLayout->addLayout( buttonGroupLayout );
         buttonGroupLayout = NULL;
     }
 }
 
-void AbstractController::createAndAddWidget( QBoxLayout *controlLayout,
+void AbstractController::createAndAddWidget( QBoxLayout *controlLayout_,
                                              int i_index,
                                              buttonType_e i_type,
                                              int i_option )
@@ -160,18 +160,18 @@ void AbstractController::createAndAddWidget( QBoxLayout *controlLayout,
     /* Close the current buttonGroup if we have a special widget or a spacer */
     if( buttonGroupLayout && i_type > BUTTON_MAX )
     {
-        controlLayout->addLayout( buttonGroupLayout );
+        controlLayout_->addLayout( buttonGroupLayout );
         buttonGroupLayout = NULL;
     }
 
     /* Special case for SPACERS, who aren't QWidgets */
     if( i_type == WIDGET_SPACER )
     {
-        controlLayout->addSpacing( 12 );
+        controlLayout_->addSpacing( 12 );
     }
     else if(  i_type == WIDGET_SPACER_EXTEND )
     {
-        controlLayout->addStretch( 12 );
+        controlLayout_->addStretch( 12 );
     }
     else
     {
@@ -191,7 +191,7 @@ void AbstractController::createAndAddWidget( QBoxLayout *controlLayout,
         }
         else /* Special widgets */
         {
-            controlLayout->addWidget( widg );
+            controlLayout_->addWidget( widg );
         }
     }
 }
@@ -359,6 +359,9 @@ QWidget *AbstractController::createWidget( buttonType_e button, int options )
                  THEMIM->getIM(), sliderUpdate( float ) );
         CONNECT( THEMIM->getIM(), cachingChanged( float ),
                  slider, updateBuffering( float ) );
+        /* Give hint to disable slider's interactivity when useless */
+        CONNECT( THEMIM->getIM(), inputCanSeek( bool ),
+                 slider, setSeekable( bool ) );
         widget = slider;
         }
         break;
@@ -648,7 +651,7 @@ ControlsWidget::ControlsWidget( intf_thread_t *_p_i,
     setStyleSheet( "background: red ");
 #endif
     setAttribute( Qt::WA_MacBrushedMetal);
-    QVBoxLayout *controlLayout = new QVBoxLayout( this );
+    controlLayout = new QVBoxLayout( this );
     controlLayout->setContentsMargins( 3, 1, 0, 1 );
     controlLayout->setSpacing( 0 );
     QHBoxLayout *controlLayout1 = new QHBoxLayout;
@@ -737,7 +740,6 @@ FullscreenControllerWidget::FullscreenControllerWidget( intf_thread_t *_p_i, QWi
 
     setWindowFlags( Qt::ToolTip );
     setMinimumWidth( FSC_WIDTH );
-    setMinimumHeight( FSC_HEIGHT );
     isWideFSC = false;
 
     setFrameShape( QFrame::StyledPanel );
@@ -905,7 +907,7 @@ void FullscreenControllerWidget::updateFullwidthGeometry( int number )
 {
     QRect screenGeometry = QApplication::desktop()->screenGeometry( number );
     setMinimumWidth( screenGeometry.width() );
-    setGeometry( screenGeometry.x(), screenGeometry.y() + screenGeometry.height() - FSC_HEIGHT, screenGeometry.width(), FSC_HEIGHT );
+    setGeometry( screenGeometry.x(), screenGeometry.y() + screenGeometry.height() - height(), screenGeometry.width(), height() );
     adjustSize();
 }
 
@@ -935,7 +937,7 @@ void FullscreenControllerWidget::customEvent( QEvent *event )
     switch( (int)event->type() )
     {
         /* This is used when the 'i' hotkey is used, to force quick toggle */
-        case FullscreenControlToggle_Type:
+        case IMEvent::FullscreenControlToggle:
             vlc_mutex_lock( &lock );
             b_fs = b_fullscreen;
             vlc_mutex_unlock( &lock );
@@ -952,7 +954,7 @@ void FullscreenControllerWidget::customEvent( QEvent *event )
             }
             break;
         /* Event called to Show the FSC on mouseChanged() */
-        case FullscreenControlShow_Type:
+        case IMEvent::FullscreenControlShow:
             vlc_mutex_lock( &lock );
             b_fs = b_fullscreen;
             vlc_mutex_unlock( &lock );
@@ -962,12 +964,12 @@ void FullscreenControllerWidget::customEvent( QEvent *event )
 
             break;
         /* Start the timer to hide later, called usually with above case */
-        case FullscreenControlPlanHide_Type:
+        case IMEvent::FullscreenControlPlanHide:
             if( !b_mouse_over ) // Only if the mouse is not over FSC
                 planHideFSC();
             break;
         /* Hide */
-        case FullscreenControlHide_Type:
+        case IMEvent::FullscreenControlHide:
             hideFSC();
             break;
         default:
@@ -1062,7 +1064,7 @@ static int FullscreenControllerWidgetFullscreenChanged( vlc_object_t *vlc_object
 
     vout_thread_t *p_vout = (vout_thread_t *) vlc_object;
 
-    msg_Dbg( p_vout, "Qt4: Fullscreen state changed" );
+    msg_Dbg( p_vout, "Qt: Fullscreen state changed" );
     FullscreenControllerWidget *p_fs = (FullscreenControllerWidget *)data;
 
     p_fs->fullscreenChanged( p_vout, new_val.b_bool, var_GetInteger( p_vout, "mouse-hide-timeout" ) );
@@ -1171,7 +1173,7 @@ void FullscreenControllerWidget::fullscreenChanged( vout_thread_t *p_vout,
                 FullscreenControllerWidgetMouseMoved, this );
 
         /* Force fs hiding */
-        IMEvent *eHide = new IMEvent( FullscreenControlHide_Type, 0 );
+        IMEvent *eHide = new IMEvent( IMEvent::FullscreenControlHide, 0 );
         QApplication::postEvent( this, eHide );
     }
     vlc_mutex_unlock( &lock );
@@ -1199,11 +1201,11 @@ void FullscreenControllerWidget::mouseChanged( vout_thread_t *, int i_mousex, in
     if( b_toShow )
     {
         /* Show event */
-        IMEvent *eShow = new IMEvent( FullscreenControlShow_Type, 0 );
+        IMEvent *eShow = new IMEvent( IMEvent::FullscreenControlShow, 0 );
         QApplication::postEvent( this, eShow );
 
         /* Plan hide event */
-        IMEvent *eHide = new IMEvent( FullscreenControlPlanHide_Type, 0 );
+        IMEvent *eHide = new IMEvent( IMEvent::FullscreenControlPlanHide, 0 );
         QApplication::postEvent( this, eHide );
     }
 }
