@@ -2,7 +2,7 @@
  * ml_model.hpp ML model
  *****************************************************************************
  * Copyright (C) 2008-2011 the VideoLAN Team and AUTHORS
- * $Id: 17bafc11b57acb5263da1d43ba616f8bbcc9fe01 $
+ * $Id: 3b4653273c8d6890bf50c8e944ef2adacd9bd02f $
  *
  * Authors: Antoine Lejeune <phytos@videolan.org>
  *          Jean-Philippe André <jpeg@videolan.org>
@@ -54,13 +54,9 @@ public:
     // Basic QAbstractItemModel implementation
     MLModel( intf_thread_t *_p_intf, QObject *parent = NULL );
     virtual ~MLModel();
-    inline MLItem *getItem( QModelIndex index ) const
-    {
-        if( index.isValid() )
-            return static_cast<MLItem*>( index.internalPointer() );
-        else return NULL;
-    }
+
     virtual int itemId( const QModelIndex & ) const;
+    virtual input_item_t *getInputItem( const QModelIndex &index ) const;
 
     QVariant data( const QModelIndex &idx, const int role = Qt::DisplayRole ) const;
     bool setData( const QModelIndex &idx, const QVariant &value,
@@ -71,7 +67,6 @@ public:
                        const QModelIndex & parent = QModelIndex() ) const;
     virtual QModelIndex currentIndex() const;
     int rowCount( const QModelIndex & parent = QModelIndex() ) const;
-    int columnCount( const QModelIndex & parent = QModelIndex() ) const;
 
     QModelIndex parent( const QModelIndex& ) const;
     QVariant headerData( int, Qt::Orientation, int ) const;
@@ -95,30 +90,34 @@ public:
                            bool bSignal = true );
 
     virtual void doDelete( QModelIndexList list );
-    void remove( MLItem *item );
     void remove( QModelIndex idx );
 
     void clear();
-    virtual bool popup( const QModelIndex & index, const QPoint &point, const QModelIndexList &list );
     void play( const QModelIndex &idx );
-    QStringList selectedURIs();
+    virtual QString getURI( const QModelIndex &index ) const;
+    virtual QModelIndex rootIndex() const;
+    virtual bool isTree() const;
+    virtual bool canEdit() const;
+    virtual bool isCurrentItem( const QModelIndex &index, playLocation where ) const;
+    QModelIndex getIndexByMLID( int id ) const;
 
 public slots:
     void activateItem( const QModelIndex &index );
+    virtual void actionSlot( QAction *action );
 
-protected slots:
-    void popupDel();
-    void popupPlay();
-    void popupInfo();
-    void popupStream();
-    void popupSave();
+protected:
+    void remove( MLItem *item );
+    inline MLItem *getItem( QModelIndex index ) const
+    {
+        if( index.isValid() )
+            return static_cast<MLItem*>( index.internalPointer() );
+        else return NULL;
+    }
 
 private:
     QList< MLItem* > items;
     media_library_t* p_ml;
 
-    QModelIndex current_index;
-    QModelIndexList current_selection;
 };
 
 #endif
