@@ -1,8 +1,8 @@
 /*****************************************************************************
- * Messages.cpp : Information about an item
+ * messages.cpp : Information about an item
  ****************************************************************************
  * Copyright (C) 2006-2011 the VideoLAN team
- * $Id: a9cef28cbeed29ed82393494189b08c763115c43 $
+ * $Id: 223c4798f2534ceda56741a5ec62e6a8f05228dd $
  *
  * Authors: Jean-Baptiste Kempf <jb (at) videolan.org>
  *
@@ -25,10 +25,10 @@
 #endif
 
 #include "dialogs/messages.hpp"
-#include <vlc_atomic.h>
 
-#include <QTextEdit>
+#include <QPlainTextEdit>
 #include <QTextCursor>
+#include <QTextBlock>
 #include <QFileDialog>
 #include <QTextStream>
 #include <QMessageBox>
@@ -38,17 +38,19 @@
 #include <QMutex>
 #include <QLineEdit>
 #include <QScrollBar>
+#include <QMutex>
+#include <QMutexLocker>
 
 #include <assert.h>
 
 enum {
-    MsgEvent_Type = QEvent::User + MsgEventType + 1,
+    MsgEvent_Type = QEvent::User + MsgEventTypeOffset + 1,
 };
 
 class MsgEvent : public QEvent
 {
 public:
-    MsgEvent( int, const msg_item_t *, const char * );
+    MsgEvent( int, const vlc_log_t *, const char * );
 
     int priority;
     uintptr_t object_id;
@@ -58,7 +60,7 @@ public:
     QString text;
 };
 
-MsgEvent::MsgEvent( int type, const msg_item_t *msg, const char *text )
+MsgEvent::MsgEvent( int type, const vlc_log_t *msg, const char *text )
     : QEvent( (QEvent::Type)MsgEvent_Type ),
       priority( type ),
       object_id( msg->i_object_id ),
@@ -78,7 +80,6 @@ MessagesDialog::MessagesDialog( intf_thread_t *_p_intf)
     ui.setupUi( this );
     ui.bottomButtonsBox->addButton( new QPushButton( qtr("&Close"), this ),
                                          QDialogButtonBox::RejectRole );
-    updateTree();
 
     /* Modules tree */
     ui.modulesTree->setHeaderHidden( true );
@@ -86,31 +87,39 @@ MessagesDialog::MessagesDialog( intf_thread_t *_p_intf)
     /* Buttons and general layout */
     ui.saveLogButton->setToolTip( qtr( "Saves all the displayed logs to a file" ) );
 
-    int verbosity = var_InheritInteger( p_intf, "verbose" );
-    vlc_atomic_set( &this->verbosity, verbosity );
-    ui.verbosityBox->setValue( qMin( verbosity, 2 ) );
+    int i_verbosity = var_InheritInteger( p_intf, "verbose" );
+    changeVerbosity( i_verbosity );
+    ui.verbosityBox->setValue( qMin( i_verbosity, 2 ) );
 
-    char *objs = var_InheritString( p_intf, "verbose-objects" );
-    if( objs != NULL )
-    {
-        ui.vbobjectsEdit->setText( qfu(objs) );
-        free( objs );
-    }
-    updateConfig();
-    ui.vbobjectsEdit->setToolTip( "verbose-objects usage: \n"
-                            "--verbose-objects=+printthatobject,-dontprintthatone\n"
-                            "(keyword 'all' to applies to all objects)");
+    getSettings()->beginGroup( "Messages" );
+    ui.filterEdit->setText( getSettings()->value( "messages-filter" ).toString() );
+    getSettings()->endGroup();
 
     updateButton = new QPushButton( QIcon(":/update"), "" );
-    updateButton->setToolTip( qtr("Update the tree") );
-    ui.mainTab->setCornerWidget( updateButton );
-    updateButton->setVisible( false );
     updateButton->setFlat( true );
+    ui.mainTab->setCornerWidget( updateButton );
 
-    BUTTONACT( ui.clearButton, clear() );
-    BUTTONACT( updateButton, updateTree() );
+#ifndef NDEBUG
+    QWidget *pldebugTab = new QWidget();
+    QVBoxLayout *pldebugTabLayout = new QVBoxLayout();
+    pldebugTab->setLayout( pldebugTabLayout );
+    ui.mainTab->addTab( pldebugTab, "Playlist Tree" );
+    pldebugTree = new QTreeWidget();
+    pldebugTree->headerItem()->setText( 0, "Name" );
+    pldebugTree->headerItem()->setText( 1, "PL id" );
+    pldebugTree->headerItem()->setText( 2, "Item id" );
+    pldebugTree->headerItem()->setText( 3, "PL flags" );
+    pldebugTree->headerItem()->setText( 4, "Item flags" );
+    pldebugTree->setColumnCount( 5 );
+    pldebugTabLayout->addWidget( pldebugTree );
+#endif
+
+    tabChanged(0);
+
+    BUTTONACT( updateButton, updateOrClear() );
     BUTTONACT( ui.saveLogButton, save() );
-    CONNECT( ui.vbobjectsEdit, editingFinished(), this, updateConfig());
+    CONNECT( ui.filterEdit, editingFinished(), this, updateConfig() );
+    CONNECT( ui.filterEdit, textChanged(QString), this, filterMessages() );
     CONNECT( ui.bottomButtonsBox, rejected(), this, hide() );
     CONNECT( ui.verbosityBox, valueChanged( int ),
              this, changeVerbosity( int ) );
@@ -118,64 +127,70 @@ MessagesDialog::MessagesDialog( intf_thread_t *_p_intf)
     CONNECT( ui.mainTab, currentChanged( int ), this, tabChanged( int ) );
 
     /* General action */
-    readSettings( "Messages", QSize( 600, 450 ) );
+    restoreWidgetPosition( "Messages", QSize( 600, 450 ) );
 
     /* Hook up to LibVLC messaging */
-    sub = vlc_Subscribe( MsgCallback, this );
+    vlc_LogSet( p_intf->p_libvlc, MsgCallback, this );
+
+    buildTree( NULL, VLC_OBJECT( p_intf->p_libvlc ) );
 }
 
 MessagesDialog::~MessagesDialog()
 {
-    writeSettings( "Messages" );
-    vlc_Unsubscribe( sub );
+    saveWidgetPosition( "Messages" );
+    vlc_LogSet( p_intf->p_libvlc, NULL, NULL );
 };
 
-void MessagesDialog::changeVerbosity( int verbosity )
+void MessagesDialog::changeVerbosity( int i_verbosity )
 {
-    vlc_atomic_set( &this->verbosity, verbosity );
+    vlc_atomic_set( &this->verbosity, i_verbosity );
 }
 
 void MessagesDialog::updateConfig()
 {
-    const QString& objects = ui.vbobjectsEdit->text();
-    /* FIXME: config item should be part of Qt4 module */
-    config_PutPsz(p_intf, "verbose-objects", qtu(objects));
+    getSettings()->beginGroup( "Messages" );
+    getSettings()->setValue( "messages-filter", ui.filterEdit->text() );
+    getSettings()->endGroup();
+}
 
-    QStringList filterOut, filterIn;
-    /* If a filter is set, disable by default */
-    /* If no filters are set, enable */
-    filterDefault = objects.isEmpty();
-    foreach( const QString& elem, objects.split(QChar(',')) )
+void MessagesDialog::filterMessages()
+{
+    QMutexLocker locker( &messageLocker );
+    QPlainTextEdit *messages = ui.messages;
+    QTextBlock block = messages->document()->firstBlock();
+
+    while( block.isValid() )
     {
-        QString object = elem;
-        bool add = true;
-
-        if( elem.startsWith(QChar('-')) )
-        {
-            add = false;
-            object.remove( 0, 1 );
-        }
-        else if( elem.startsWith(QChar('+')) )
-            object.remove( 0, 1 );
-
-        if( object.compare(qfu("all"), Qt::CaseInsensitive) == 0 )
-            filterDefault = add;
-        else
-            (add ? &filterIn : &filterOut)->append( object );
+        block.setVisible( matchFilter( block.text().toLower() ) );
+        block = block.next();
     }
-    filter = filterDefault ? filterOut : filterIn;
-    filter.removeDuplicates();
+
+    /* Consider the whole QTextDocument as dirty now */
+    messages->document()->markContentsDirty( 0, messages->document()->characterCount() );
+
+    /* FIXME This solves a bug (Qt?) with the viewport not resizing the
+       vertical scroll bar when one or more QTextBlock are hidden */
+    QSize vsize = messages->viewport()->size();
+    messages->viewport()->resize( vsize + QSize( 1, 1 ) );
+    messages->viewport()->resize( vsize );
+}
+
+bool MessagesDialog::matchFilter( const QString& text )
+{
+    const QString& filter = ui.filterEdit->text();
+
+    if( filter.isEmpty() || text.contains( filter.toLower() ) )
+        return true;
+    return false;
 }
 
 void MessagesDialog::sinkMessage( const MsgEvent *msg )
 {
-    if( (filter.contains(msg->module) || filter.contains(msg->object_type))
-                                                            == filterDefault )
-        return;
+    QMutexLocker locker( &messageLocker );
 
-    QTextEdit *messages = ui.messages;
+    QPlainTextEdit *messages = ui.messages;
     /* Only scroll if the viewport is at the end.
-       Don't bug user by auto-changing/loosing viewport on insert(). */
+       Don't bug user by auto-changing/losing viewport on insert(). */
     bool b_autoscroll = ( messages->verticalScrollBar()->value()
                           + messages->verticalScrollBar()->pageStep()
                           >= messages->verticalScrollBar()->maximum() );
@@ -189,36 +204,41 @@ void MessagesDialog::sinkMessage( const MsgEvent *msg )
          messages->textCursor().anchor() != messages->textCursor().position() )
          messages->moveCursor( QTextCursor::End );
 
-    messages->setFontItalic( true );
-    messages->setTextColor( "darkBlue" );
-    messages->insertPlainText( msg->module );
+    /* Start a new logic block so we can hide it on-demand */
+    messages->textCursor().insertBlock();
 
-    switch (msg->priority)
+    QString buf = QString( "<i><font color='darkblue'>%1</font>" ).arg( msg->module );
+
+    switch ( msg->priority )
     {
         case VLC_MSG_INFO:
-            messages->setTextColor( "blue" );
-            messages->insertPlainText( " info: " );
+            buf += "<font color='blue'> info: </font>";
             break;
         case VLC_MSG_ERR:
-            messages->setTextColor( "red" );
-            messages->insertPlainText( " error: " );
+            buf += "<font color='red'> error: </font>";
             break;
         case VLC_MSG_WARN:
-            messages->setTextColor( "green" );
-            messages->insertPlainText( " warning: " );
+            buf += "<font color='green'> warning: </font>";
             break;
         case VLC_MSG_DBG:
         default:
-            messages->setTextColor( "grey" );
-            messages->insertPlainText( " debug: " );
+            buf += "<font color='grey'> debug: </font>";
             break;
     }
 
-    /* Add message Regular black Font */
-    messages->setFontItalic( false );
-    messages->setTextColor( "black" );
-    messages->insertPlainText( msg->text );
-    messages->insertPlainText( "\n" );
+    /* Insert the prefix */
+    messages->textCursor().insertHtml( buf /* + "</i>" */ );
+
+    /* Insert the message */
+    messages->textCursor().insertHtml( msg->text );
+
+    /* Pass the new message thru the filter */
+    QTextBlock b = messages->document()->lastBlock();
+    b.setVisible( matchFilter( b.text() ) );
+
+    /* Tell the QTextDocument to recompute the size of the given area */
+    messages->document()->markContentsDirty( b.position(), b.length() );
+
     if ( b_autoscroll ) messages->ensureCursorVisible();
 }
 
@@ -230,11 +250,6 @@ void MessagesDialog::customEvent( QEvent *event )
     sinkMessage( msge );
 }
 
-void MessagesDialog::clear()
-{
-    ui.messages->clear();
-}
-
 bool MessagesDialog::save()
 {
     QString saveLogFileName = QFileDialog::getSaveFileName(
@@ -254,8 +269,15 @@ bool MessagesDialog::save()
         }
 
         QTextStream out( &file );
-        out << ui.messages->toPlainText() << "\n";
 
+        QTextBlock block = ui.messages->document()->firstBlock();
+        while( block.isValid() )
+        {
+            if( block.isVisible() )
+                out << block.text() << "\n";
+
+            block = block.next();
+        }
         return true;
     }
     return false;
@@ -288,18 +310,29 @@ void MessagesDialog::buildTree( QTreeWidgetItem *parentItem,
     vlc_list_release( l );
 }
 
-void MessagesDialog::updateTree()
+void MessagesDialog::updateOrClear()
 {
-    ui.modulesTree->clear();
-    buildTree( NULL, VLC_OBJECT( p_intf->p_libvlc ) );
+    if( ui.mainTab->currentIndex() == 1)
+    {
+        ui.modulesTree->clear();
+        buildTree( NULL, VLC_OBJECT( p_intf->p_libvlc ) );
+    }
+    else if( ui.mainTab->currentIndex() == 0 )
+        ui.messages->clear();
+#ifndef NDEBUG
+    else
+        updatePLTree();
+#endif
 }
 
 void MessagesDialog::tabChanged( int i )
 {
-    updateButton->setVisible( i == 1 );
+    updateButton->setIcon( i != 0 ? QIcon(":/update") : QIcon(":/toolbar/clear") );
+    updateButton->setToolTip( i != 0 ? qtr("Update the tree")
+                                     : qtr("Clear the messages") );
 }
 
-void MessagesDialog::MsgCallback( void *self, int type, const msg_item_t *item,
+void MessagesDialog::MsgCallback( void *self, int type, const vlc_log_t *item,
                                   const char *format, va_list ap )
 {
     MessagesDialog *dialog = (MessagesDialog *)self;
@@ -315,3 +348,31 @@ void MessagesDialog::MsgCallback( void *self, int type, const msg_item_t *item,
     vlc_restorecancel( canc );
     free( str );
 }
+
+#ifndef NDEBUG
+static QTreeWidgetItem * PLWalk( playlist_item_t *p_node )
+{
+    QTreeWidgetItem *current = new QTreeWidgetItem();
+    current->setText( 0, qfu( p_node->p_input->psz_name ) );
+    current->setToolTip( 0, qfu( p_node->p_input->psz_uri ) );
+    current->setText( 1, QString("%1").arg( p_node->i_id ) );
+    current->setText( 2, QString("%1").arg( p_node->p_input->i_id ) );
+    current->setText( 3, QString("0x%1").arg( p_node->i_flags, 0, 16 ) );
+    current->setText( 4, QString("0x%1").arg(  p_node->p_input->i_type, 0, 16 ) );
+    for ( int i = 0; p_node->i_children > 0 && i < p_node->i_children; i++ )
+        current->addChild( PLWalk( p_node->pp_children[ i ] ) );
+    return current;
+}
+
+void MessagesDialog::updatePLTree()
+{
+    playlist_t *p_playlist = THEPL;
+    pldebugTree->clear();
+    PL_LOCK;
+    pldebugTree->addTopLevelItem( PLWalk( p_playlist->p_root_category ) );
+    PL_UNLOCK;
+    pldebugTree->expandAll();
+    for ( int i=0; i< 5; i++ )
+        pldebugTree->resizeColumnToContents( i );
+}
+#endif
