@@ -2,7 +2,7 @@
  * intf.c: Generic lua interface functions
  *****************************************************************************
  * Copyright (C) 2007-2008 the VideoLAN team
- * $Id: 0e1bd391c6975a630d72c3ee7df966da40503539 $
+ * $Id: d8eab715c15b485e2fd8ca30c6bf3ff3199dd1c9 $
  *
  * Authors: Antoine Cellerier <dionoea at videolan tod org>
  *
@@ -24,22 +24,17 @@
 /*****************************************************************************
  * Preamble
  *****************************************************************************/
-#ifndef  _GNU_SOURCE
-#   define  _GNU_SOURCE
-#endif
-
 #ifdef HAVE_CONFIG_H
 # include "config.h"
 #endif
 
-#include <vlc_common.h>
-#include <vlc_interface.h>
 #include <sys/types.h>
 #include <sys/stat.h>
+#include <unistd.h>
 
-#include <lua.h>        /* Low level lua C API */
-#include <lauxlib.h>    /* Higher level C API */
-#include <lualib.h>     /* Lua libs */
+#include <vlc_common.h>
+#include <vlc_interface.h>
+#include <vlc_fs.h>
 
 #include "vlc.h"
 #include "libs.h"
@@ -62,6 +57,91 @@ static inline void luaL_register_submodule( lua_State *L, const char *psz_name,
     lua_setfield( L, -2, psz_name );
 }
 
+static char *MakeConfig( intf_thread_t *p_intf, const char *name )
+{
+    char *psz_config = NULL;
+
+    if( !strcmp( name, "http" ) )
+    {
+        char *psz_http_src = var_InheritString( p_intf, "http-src" );
+        bool b_http_index = var_InheritBool( p_intf, "http-index" );
+        if( psz_http_src )
+        {
+            char *psz_esc = config_StringEscape( psz_http_src );
+
+            if( asprintf( &psz_config, "http={dir='%s',no_index=%s}", psz_esc,
+                          b_http_index ? "true" : "false" ) == -1 )
+                psz_config = NULL;
+            free( psz_esc );
+            free( psz_http_src );
+        }
+        else
+        {
+            if( asprintf( &psz_config, "http={no_index=%s}",
+                          b_http_index ? "true" : "false" ) == -1 )
+                psz_config = NULL;
+        }
+    }
+    else if( !strcmp( name, "telnet" ) )
+    {
+        char *psz_host = var_InheritString( p_intf, "telnet-host" );
+        if( !strcmp( psz_host, "*console" ) )
+            ;
+        else
+        {
+            vlc_url_t url;
+            vlc_UrlParse( &url, psz_host, 0 );
+            unsigned i_port = var_InheritInteger( p_intf, "telnet-port" );
+            if ( url.i_port != 0 )
+            {
+                if ( i_port == TELNETPORT_DEFAULT )
+                    i_port = url.i_port;
+                else if ( url.i_port != i_port )
+                    msg_Warn( p_intf, "ignoring port %d (using %d)",
+                              url.i_port, i_port );
+            }
+
+            char *psz_esc_host = config_StringEscape( url.psz_host );
+            free( psz_host );
+            vlc_UrlClean( &url );
+
+            if( asprintf( &psz_host, "telnet://%s:%d",
+                          psz_esc_host ? psz_esc_host : "", i_port ) == -1 )
+                psz_host = NULL;
+            free( psz_esc_host );
+        }
+
+        char *psz_passwd = var_InheritString( p_intf, "telnet-password" );
+
+        char *psz_esc_passwd = config_StringEscape( psz_passwd );
+
+        if( asprintf( &psz_config, "telnet={host='%s',password='%s'}",
+                      psz_host, psz_esc_passwd ) == -1 )
+            psz_config = NULL;
+
+        free( psz_esc_passwd );
+        free( psz_passwd );
+        free( psz_host );
+    }
+    else if( !strcmp( name, "cli" ) )
+    {
+        char *psz_rc_host = var_InheritString( p_intf, "rc-host" );
+        if( !psz_rc_host )
+            psz_rc_host = var_InheritString( p_intf, "cli-host" );
+        if( psz_rc_host )
+        {
+            char *psz_esc_host = config_StringEscape( psz_rc_host );
+
+            if( asprintf( &psz_config, "cli={host='%s'}", psz_esc_host ) == -1 )
+                psz_config = NULL;
+            free( psz_esc_host );
+            free( psz_rc_host );
+        }
+    }
+
+    return psz_config;
+}
+
 static char *StripPasswords( const char *psz_config )
 {
     unsigned n = 0;
@@ -124,9 +204,6 @@ static int Start_LuaIntf( vlc_object_t *p_this, const char *name )
 
     config_ChainParse( p_intf, "lua-", ppsz_intf_options, p_intf->p_cfg );
 
-    char *psz_config;
-    bool b_config_set = false;
-
     if( name == NULL )
     {
         char *n = var_InheritString( p_this, "lua-intf" );
@@ -146,7 +223,7 @@ static int Start_LuaIntf( vlc_object_t *p_this, const char *name )
         return VLC_ENOMEM;
     }
     p_sys = p_intf->p_sys;
-    p_sys->psz_filename = vlclua_find_file( p_this, "intf", name );
+    p_sys->psz_filename = vlclua_find_file( "intf", name );
     if( !p_sys->psz_filename )
     {
         msg_Err( p_intf, "Couldn't find lua interface script \"%s\".",
@@ -163,7 +240,6 @@ static int Start_LuaIntf( vlc_object_t *p_this, const char *name )
     }
 
     vlclua_set_this( L, p_intf );
-    vlclua_set_intf( L, p_sys );
 
     luaL_openlibs( L );
 
@@ -171,7 +247,6 @@ static int Start_LuaIntf( vlc_object_t *p_this, const char *name )
     luaL_register( L, "vlc", p_reg );
 
     /* register submodules */
-    luaopen_acl( L );
     luaopen_config( L );
     luaopen_volume( L );
     luaopen_httpd( L );
@@ -192,12 +267,15 @@ static int Start_LuaIntf( vlc_object_t *p_this, const char *name )
     luaopen_gettext( L );
     luaopen_xml( L );
     luaopen_equalizer( L );
+#if defined(_WIN32) && !VLC_WINSTORE_APP
+    luaopen_win( L );
+#endif
 
     /* clean up */
     lua_pop( L, 1 );
 
     /* Setup the module search path */
-    if( vlclua_add_modules_path( p_intf, L, p_sys->psz_filename ) )
+    if( vlclua_add_modules_path( L, p_sys->psz_filename ) )
     {
         msg_Warn( p_intf, "Error while setting the module search path for %s",
                   p_sys->psz_filename );
@@ -210,89 +288,10 @@ static int Start_LuaIntf( vlc_object_t *p_this, const char *name )
      * If the string is empty, try with the old http-* or telnet-* options
      * and build the right configuration line
      */
-    psz_config = var_CreateGetNonEmptyString( p_intf, "lua-config" );
+    bool b_config_set = false;
+    char *psz_config = var_InheritString( p_intf, "lua-config" );
     if( !psz_config )
-    {
-        if( !strcmp( name, "http" ) )
-        {
-            char *psz_http_src = var_CreateGetNonEmptyString( p_intf, "http-src" );
-            bool b_http_index = var_CreateGetBool( p_intf, "http-index" );
-            if( psz_http_src )
-            {
-                char *psz_esc = config_StringEscape( psz_http_src );
-                if( psz_config )
-                {
-                    char *psz_tmp;
-                    asprintf( &psz_tmp, "%s,dir='%s'", psz_config, psz_esc );
-                    free( psz_config );
-                    psz_config = psz_tmp;
-                }
-                else
-                    asprintf( &psz_config, "http={dir='%s'", psz_esc );
-                free( psz_esc );
-                free( psz_http_src );
-            }
-            if( psz_config )
-            {
-                char *psz_tmp;
-                asprintf( &psz_tmp, "%s,no_index=%s}", psz_config, b_http_index ? "true" : "false" );
-                free( psz_config );
-                psz_config = psz_tmp;
-            }
-            else
-                asprintf( &psz_config, "http={no_index=%s}", b_http_index ? "true" : "false" );
-        }
-        else if( !strcmp( name, "telnet" ) )
-        {
-            char *psz_telnet_host = var_CreateGetString( p_intf, "telnet-host" );
-            if( !strcmp( psz_telnet_host, "*console" ) )
-                ;
-            else
-            {
-                vlc_url_t url;
-                vlc_UrlParse( &url, psz_telnet_host, 0 );
-                int i_telnet_port = var_CreateGetInteger( p_intf, "telnet-port" );
-                if ( url.i_port != 0 )
-                {
-                    if ( i_telnet_port == TELNETPORT_DEFAULT )
-                        i_telnet_port = url.i_port;
-                    else if ( url.i_port != i_telnet_port )
-                        msg_Warn( p_intf, "ignoring port %d (using %d)", url.i_port, i_telnet_port );
-                }
-
-                char *psz_esc_host = config_StringEscape( url.psz_host );
-                free( psz_telnet_host );
-                vlc_UrlClean( &url );
-
-                asprintf( &psz_telnet_host, "telnet://%s:%d", psz_esc_host ? psz_esc_host : "", i_telnet_port );
-                free( psz_esc_host );
-            }
-
-            char *psz_telnet_passwd = var_CreateGetString( p_intf, "telnet-password" );
-
-            char *psz_esc_passwd = config_StringEscape( psz_telnet_passwd );
-
-            asprintf( &psz_config, "telnet={host='%s',password='%s'}", psz_telnet_host, psz_esc_passwd );
-
-            free( psz_esc_passwd );
-            free( psz_telnet_passwd );
-            free( psz_telnet_host );
-        }
-        else if( !strcmp( name, "cli" ) )
-        {
-            char *psz_rc_host = var_CreateGetNonEmptyString( p_intf, "rc-host" );
-            if( !psz_rc_host )
-                psz_rc_host = var_CreateGetNonEmptyString( p_intf, "cli-host" );
-            if( psz_rc_host )
-            {
-                char *psz_esc_host = config_StringEscape( psz_rc_host );
-                asprintf( &psz_config, "cli={host='%s'}", psz_esc_host );
-
-                free( psz_esc_host );
-                free( psz_rc_host );
-            }
-        }
-    }
+        psz_config = MakeConfig( p_intf, name );
 
     if( psz_config )
     {
@@ -348,7 +347,7 @@ static int Start_LuaIntf( vlc_object_t *p_this, const char *name )
         /* msg_Warn( p_intf, "The `telnet' lua interface script was replaced "
                           "by `cli', please update your configuration!" ); */
 
-        char *wrapped_file = vlclua_find_file( p_this, "intf", "cli" );
+        char *wrapped_file = vlclua_find_file( "intf", "cli" );
         if( !wrapped_file )
         {
             msg_Err( p_intf, "Couldn't find lua interface script \"cli\", "
@@ -363,14 +362,20 @@ static int Start_LuaIntf( vlc_object_t *p_this, const char *name )
 
     p_sys->L = L;
 
-    vlc_mutex_init( &p_sys->lock );
-    vlc_cond_init( &p_sys->wait );
-    p_sys->exiting = false;
+#ifndef _WIN32
+    if( vlc_pipe( p_sys->fd ) )
+    {
+        lua_close( p_sys->L );
+        goto error;
+    }
+#else
+# define close(fd) (void)0
+#endif
 
     if( vlc_clone( &p_sys->thread, Run, p_intf, VLC_THREAD_PRIORITY_LOW ) )
     {
-        vlc_cond_destroy( &p_sys->wait );
-        vlc_mutex_destroy( &p_sys->lock );
+        close( p_sys->fd[1] );
+        close( p_sys->fd[0] );
         lua_close( p_sys->L );
         goto error;
     }
@@ -389,18 +394,11 @@ void Close_LuaIntf( vlc_object_t *p_this )
     intf_thread_t *p_intf = (intf_thread_t*)p_this;
     intf_sys_t *p_sys = p_intf->p_sys;
 
-    vlc_cancel( p_sys->thread );
-
-    vlc_mutex_lock( &p_sys->lock );
-    p_sys->exiting = true;
-    vlc_cond_signal( &p_sys->wait );
-    vlc_mutex_unlock( &p_sys->lock );
+    close( p_sys->fd[1] );
     vlc_join( p_sys->thread, NULL );
-    vlc_cond_destroy( &p_sys->wait );
-    vlc_mutex_destroy( &p_sys->lock );
 
     lua_close( p_sys->L );
-
+    close( p_sys->fd[0] );
     free( p_sys->psz_filename );
     free( p_sys );
 }
@@ -432,16 +430,18 @@ int Open_LuaHTTP( vlc_object_t *p_this )
 
 int Open_LuaCLI( vlc_object_t *p_this )
 {
-    /* Hack to work around submodule brokenness */
-    intf_thread_t *p_intf = (intf_thread_t*)p_this;
-    if( !strcmp( p_intf->psz_intf, "lua" )
-        || !strncmp( p_intf->psz_intf, "lua,", 4 ) )
-        return VLC_EGENERIC;
-
     return Start_LuaIntf( p_this, "cli" );
 }
 
 int Open_LuaTelnet( vlc_object_t *p_this )
 {
+    char *pw = var_CreateGetNonEmptyString( p_this, "telnet-password" );
+    if( pw == NULL )
+    {
+        msg_Err( p_this, "password not configured" );
+        msg_Info( p_this, "Please specify the password in the preferences." );
+        return VLC_EGENERIC;
+    }
+    free( pw );
     return Start_LuaIntf( p_this, "telnet" );
 }
