@@ -2,7 +2,7 @@
  * playlist.c
  *****************************************************************************
  * Copyright (C) 2007-2011 the VideoLAN team
- * $Id: 5498520736eb7ca029829517566f8ea64123e3ec $
+ * $Id: 29f5e4bcc1ac8841a368954c7196726a21218aac $
  *
  * Authors: Antoine Cellerier <dionoea at videolan tod org>
  *
@@ -37,9 +37,6 @@
 #include <vlc_interface.h>
 #include <vlc_playlist.h>
 
-#include <lua.h>        /* Low level lua C API */
-#include <lauxlib.h>    /* Higher level C API */
-
 #include "../vlc.h"
 #include "../libs.h"
 #include "input.h"
@@ -144,7 +141,35 @@ static int vlclua_playlist_delete( lua_State * L )
     int i_id = luaL_checkint( L, 1 );
     playlist_t *p_playlist = vlclua_get_playlist_internal( L );
     PL_LOCK;
-    int i_ret = playlist_DeleteFromInput(p_playlist, playlist_ItemGetById( p_playlist, i_id ) -> p_input, true );
+    playlist_item_t *p_item = playlist_ItemGetById( p_playlist, i_id );
+    if( !p_item )
+    {
+       PL_UNLOCK;
+       return vlclua_push_ret( L, -1 );
+    }
+    int i_ret = playlist_DeleteFromInput( p_playlist, p_item -> p_input, true );
+    PL_UNLOCK;
+    return vlclua_push_ret( L, i_ret );
+}
+
+static int vlclua_playlist_move( lua_State * L )
+{
+    int i_item = luaL_checkint( L, 1 );
+    int i_target = luaL_checkint( L, 2 );
+    playlist_t *p_playlist = vlclua_get_playlist_internal( L );
+    PL_LOCK;
+    playlist_item_t *p_item = playlist_ItemGetById( p_playlist, i_item );
+    playlist_item_t *p_target = playlist_ItemGetById( p_playlist, i_target );
+    if( !p_item || !p_target )
+    {
+       PL_UNLOCK;
+       return vlclua_push_ret( L, -1 );
+    }
+    int i_ret;
+    if( p_target->i_children != -1 )
+        i_ret = playlist_TreeMove( p_playlist, p_item, p_target, 0 );
+    else
+    	i_ret = playlist_TreeMove( p_playlist, p_item, p_target->p_parent, p_target->i_id - p_target->p_parent->pp_children[0]->i_id + 1 );
     PL_UNLOCK;
     return vlclua_push_ret( L, i_ret );
 }
@@ -392,6 +417,7 @@ static const luaL_Reg vlclua_playlist_reg[] = {
     { "sort", vlclua_playlist_sort },
     { "status", vlclua_playlist_status },
     { "delete", vlclua_playlist_delete },
+    { "move", vlclua_playlist_move },
     { NULL, NULL }
 };
 
