@@ -2,7 +2,7 @@
  * rtsp.c: rtsp VoD server module
  *****************************************************************************
  * Copyright (C) 2003-2006 the VideoLAN team
- * $Id: 37d8758adeeb1c3a95023de5df21e94d8017b499 $
+ * $Id: 45cfc7a7202ab23015e68c12088ce2263e40a016 $
  *
  * Authors: Laurent Aimar <fenrir@via.ecp.fr>
  *          Gildas Bazin <gbazin@videolan.org>
@@ -44,7 +44,7 @@
 #include <vlc_strings.h>
 #include <vlc_rand.h>
 
-#ifndef WIN32
+#ifndef _WIN32
 # include <locale.h>
 #endif
 
@@ -73,7 +73,7 @@ static void Close( vlc_object_t * );
 
 vlc_module_begin ()
     set_shortname( N_("RTSP VoD" ) )
-    set_description( N_("RTSP VoD server") )
+    set_description( "Legacy RTSP VoD server" )
     set_category( CAT_SOUT )
     set_subcategory( SUBCAT_SOUT_VOD )
     set_capability( "vod server", 1 )
@@ -279,8 +279,6 @@ static int Open( vlc_object_t *p_this )
 
     p_sys->psz_path = strdup( url.psz_path ? url.psz_path : "/" );
 
-    vlc_UrlClean( &url );
-
     TAB_INIT( p_sys->i_media, p_sys->media );
     p_sys->i_media_id = 0;
 
@@ -296,6 +294,7 @@ static int Open( vlc_object_t *p_this )
         goto error;
     }
 
+    vlc_UrlClean( &url );
     return VLC_SUCCESS;
 
 error:
@@ -372,8 +371,7 @@ static vod_media_t *MediaNew( vod_t *p_vod, const char *psz_name,
                   p_sys->psz_path, psz_name ) <0 )
         return NULL;
     p_media->p_rtsp_url =
-        httpd_UrlNewUnique( p_sys->p_rtsp_host, p_media->psz_rtsp_path, NULL,
-                            NULL, NULL );
+        httpd_UrlNew( p_sys->p_rtsp_host, p_media->psz_rtsp_path, NULL, NULL );
 
     if( !p_media->p_rtsp_url )
     {
@@ -678,8 +676,7 @@ static int MediaAddES( vod_t *p_vod, vod_media_t *p_media, es_format_t *p_fmt )
     }
 
     p_es->p_rtsp_url =
-        httpd_UrlNewUnique( p_vod->p_sys->p_rtsp_host, psz_urlc, NULL, NULL,
-                            NULL );
+        httpd_UrlNew( p_vod->p_sys->p_rtsp_host, psz_urlc, NULL, NULL );
 
     if( !p_es->p_rtsp_url )
     {
@@ -757,7 +754,7 @@ static void CommandPush( vod_t *p_vod, rtsp_cmd_type_t i_type, vod_media_t *p_me
     if( psz_arg )
         cmd.psz_arg = strdup(psz_arg);
 
-    p_cmd = block_New( p_vod, sizeof(rtsp_cmd_t) );
+    p_cmd = block_Alloc( sizeof(rtsp_cmd_t) );
     memcpy( p_cmd->p_buffer, &cmd, sizeof(cmd) );
 
     block_FifoPut( p_vod->p_sys->p_fifo_cmd, p_cmd );
