@@ -2,7 +2,7 @@
  * flac.c: flac packetizer module.
  *****************************************************************************
  * Copyright (C) 1999-2001 VLC authors and VideoLAN
- * $Id: 16d3c46b1fb9824bdbacb8a0a428c9c3654119e1 $
+ * $Id: 090b60160eae10a9b4e68618f5e2d7aafb459b26 $
  *
  * Authors: Gildas Bazin <gbazin@videolan.org>
  *          Sigmund Augdal Helberg <dnumgis@videolan.org>
@@ -248,7 +248,6 @@ static block_t *Packetize( decoder_t *p_dec, block_t **pp_block )
 
     while( 1 )
     {
-        int previous_size;
         switch( p_sys->i_state )
         {
         case STATE_NOSYNC:
@@ -316,8 +315,6 @@ static block_t *Packetize( decoder_t *p_dec, block_t **pp_block )
             /* TODO: If pp_block == NULL, flush the buffer without checking the
              * next sync word */
 
-            previous_size = 0; /* Try to return the biggest frame */
-
             /* Check if next expected frame contains the sync word */
             while( block_PeekOffsetBytes( &p_sys->bytestream,
                                           p_sys->i_frame_size, p_header,
@@ -333,27 +330,13 @@ static block_t *Packetize( decoder_t *p_dec, block_t **pp_block )
                                   &p_sys->i_rate,
                                   &p_sys->i_bits_per_sample );
 
-                    if( i_frame_length ) {
-                        if( !p_sys->b_stream_info || p_sys->stream_info.max_framesize <= 0 ) {
-                            /* Stop immediately if we don't know the maximum framesize */
-                            p_sys->i_state = STATE_SEND_DATA;
-                            break;
-                        }
-                        previous_size = p_sys->i_frame_size;
+                    if( i_frame_length )
+                    {
+                        p_sys->i_state = STATE_SEND_DATA;
+                        break;
                     }
                 }
                 p_sys->i_frame_size++;
-
-                if( p_sys->b_stream_info && p_sys->stream_info.max_framesize > 0 &&
-                    p_sys->i_frame_size > p_sys->stream_info.max_framesize )
-                    break;
-            }
-
-            if (previous_size)
-            {
-                /* Use the largest frame size */
-                p_sys->i_frame_size = previous_size;
-                p_sys->i_state = STATE_SEND_DATA;
             }
 
             if( p_sys->i_state != STATE_SEND_DATA )
