@@ -7,19 +7,19 @@
  * Authors: Christopher Mueller <christopher.mueller@itec.uni-klu.ac.at>
  *          Christian Timmerer  <christian.timmerer@itec.uni-klu.ac.at>
  *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU Lesser General Public License as published
+ * This program is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU Lesser General Public License as published
  * by the Free Software Foundation; either version 2.1 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  * GNU Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston MA 02110-1301, USA.
+ * along with this program; if not, write to the Free Software Foundation,
+ * Inc., 51 Franklin Street, Fifth Floor, Boston MA 02110-1301, USA.
  *****************************************************************************/
 
 /*****************************************************************************
@@ -38,7 +38,7 @@
 #include "xml/DOMParser.h"
 #include "http/HTTPConnectionManager.h"
 #include "adaptationlogic/IAdaptationLogic.h"
-#include "mpd/BasicCMParser.h"
+#include "mpd/MPDFactory.h"
 
 #define SEEK 0
 
@@ -48,12 +48,24 @@
 static int  Open    (vlc_object_t *);
 static void Close   (vlc_object_t *);
 
+#define DASH_WIDTH_TEXT N_("Preferred Width")
+#define DASH_WIDTH_LONGTEXT N_("Preferred Width")
+
+#define DASH_HEIGHT_TEXT N_("Preferred Height")
+#define DASH_HEIGHT_LONGTEXT N_("Preferred Height")
+
+#define DASH_BUFFER_TEXT N_("Buffer Size (Seconds)")
+#define DASH_BUFFER_LONGTEXT N_("Buffer size in seconds")
+
 vlc_module_begin ()
         set_shortname( N_("DASH"))
         set_description( N_("Dynamic Adaptive Streaming over HTTP") )
         set_capability( "stream_filter", 19 )
         set_category( CAT_INPUT )
         set_subcategory( SUBCAT_INPUT_STREAM_FILTER )
+        add_integer( "dash-prefwidth",  480, DASH_WIDTH_TEXT,  DASH_WIDTH_LONGTEXT,  true )
+        add_integer( "dash-prefheight", 360, DASH_HEIGHT_TEXT, DASH_HEIGHT_LONGTEXT, true )
+        add_integer( "dash-buffersize", 30, DASH_BUFFER_TEXT, DASH_BUFFER_LONGTEXT, true )
         set_callbacks( Open, Close )
 vlc_module_end ()
 
@@ -62,14 +74,13 @@ vlc_module_end ()
  *****************************************************************************/
 struct stream_sys_t
 {
-        dash::DASHManager                   *p_dashManager;
-        dash::http::HTTPConnectionManager   *p_conManager;
-        dash::mpd::MPD                      *p_mpd;
-        int                                 position;
+        dash::DASHManager   *p_dashManager;
+        dash::mpd::MPD      *p_mpd;
+        uint64_t                            position;
         bool                                isLive;
 };
 
-static int  Read            (stream_t *p_stream, void *p_buffer, unsigned int i_len);
+static int  Read            (stream_t *p_stream, void *p_ptr, unsigned int i_len);
 static int  Peek            (stream_t *p_stream, const uint8_t **pp_peek, unsigned int i_peek);
 static int  Control         (stream_t *p_stream, int i_query, va_list args);
 
@@ -90,36 +101,29 @@ static int Open(vlc_object_t *p_obj)
         msg_Dbg( p_stream, "Could not parse mpd file." );
         return VLC_EGENERIC;
     }
+
     //Begin the actual MPD parsing:
-    dash::mpd::BasicCMParser    mpdParser( parser.getRootNode(), p_stream->p_source );
-    if ( mpdParser.parse() == false || mpdParser.getMPD() == NULL )
-    {
-        msg_Err( p_obj, "MPD file parsing failed." );
+    dash::mpd::MPD *mpd = dash::mpd::MPDFactory::create(parser.getRootNode(), p_stream->p_source, parser.getProfile());
+
+    if(mpd == NULL)
         return VLC_EGENERIC;
-    }
 
     stream_sys_t        *p_sys = (stream_sys_t *) malloc(sizeof(stream_sys_t));
     if (unlikely(p_sys == NULL))
         return VLC_ENOMEM;
 
-    p_sys->p_mpd = mpdParser.getMPD();
-    dash::http::HTTPConnectionManager *p_conManager =
-                              new dash::http::HTTPConnectionManager( p_stream );
-    dash::DASHManager*p_dashManager =
-            new dash::DASHManager( p_conManager, p_sys->p_mpd,
-                                   dash::logic::IAdaptationLogic::RateBased );
+    p_sys->p_mpd = mpd;
+    dash::DASHManager*p_dashManager = new dash::DASHManager(p_sys->p_mpd,
+                                          dash::logic::IAdaptationLogic::RateBased,
+                                          p_stream);
 
-    if ( p_dashManager->getMpdManager() == NULL ||
-         p_dashManager->getMpdManager()->getMPD() == NULL ||
-         p_dashManager->getAdaptionLogic() == NULL )
+    if(!p_dashManager->start())
     {
-        delete p_conManager;
         delete p_dashManager;
         free( p_sys );
         return VLC_EGENERIC;
     }
     p_sys->p_dashManager    = p_dashManager;
-    p_sys->p_conManager     = p_conManager;
     p_sys->position         = 0;
     p_sys->isLive           = p_dashManager->getMpdManager()->getMPD()->isLive();
     p_stream->p_sys         = p_sys;
@@ -139,24 +143,73 @@ static void Close(vlc_object_t *p_obj)
     stream_t                            *p_stream       = (stream_t*) p_obj;
     stream_sys_t                        *p_sys          = (stream_sys_t *) p_stream->p_sys;
     dash::DASHManager                   *p_dashManager  = p_sys->p_dashManager;
-    dash::http::HTTPConnectionManager   *p_conManager   = p_sys->p_conManager;
 
-    delete(p_conManager);
     delete(p_dashManager);
     free(p_sys);
 }
 /*****************************************************************************
  * Callbacks:
  *****************************************************************************/
-static int  Read            (stream_t *p_stream, void *p_buffer, unsigned int i_len)
+static int  Seek            ( stream_t *p_stream, uint64_t pos )
 {
     stream_sys_t        *p_sys          = (stream_sys_t *) p_stream->p_sys;
     dash::DASHManager   *p_dashManager  = p_sys->p_dashManager;
     int                 i_ret           = 0;
+    unsigned            i_len           = 0;
+    long                i_read          = 0;
 
-    i_ret = p_dashManager->read(p_buffer, i_len );
+    if( pos < p_sys->position )
+    {
+        if( p_sys->position - pos > UINT_MAX )
+        {
+            msg_Err( p_stream, "Cannot seek backward that far!" );
+            return VLC_EGENERIC;
+        }
+        i_len = p_sys->position - pos;
+        i_ret = p_dashManager->seekBackwards( i_len );
+        if( i_ret == VLC_EGENERIC )
+        {
+            msg_Err( p_stream, "Cannot seek backward outside the current block :-/" );
+            return VLC_EGENERIC;
+        }
+        else
+            return VLC_SUCCESS;
+    }
 
-    if (i_ret < 0)
+    /* Seek forward */
+    if( pos - p_sys->position > UINT_MAX )
+    {
+        msg_Err( p_stream, "Cannot seek forward that far!" );
+        return VLC_EGENERIC;
+    }
+    i_len = pos - p_sys->position;
+    i_read = Read( p_stream, (void *)NULL, i_len );
+    if( (unsigned)i_read == i_len )
+        return VLC_SUCCESS;
+    else
+        return VLC_EGENERIC;
+}
+
+static int  Read            (stream_t *p_stream, void *p_ptr, unsigned int i_len)
+{
+    stream_sys_t        *p_sys          = (stream_sys_t *) p_stream->p_sys;
+    dash::DASHManager   *p_dashManager  = p_sys->p_dashManager;
+    uint8_t             *p_buffer       = (uint8_t*)p_ptr;
+    int                 i_ret           = 0;
+    int                 i_read          = 0;
+
+    while( i_len > 0 )
+    {
+        i_read = p_dashManager->read( p_buffer, i_len );
+        if( i_read < 0 )
+            break;
+        p_buffer += i_read;
+        i_ret += i_read;
+        i_len -= i_read;
+    }
+    p_buffer -= i_ret;
+
+    if (i_read < 0)
     {
         switch (errno)
         {
@@ -194,15 +247,40 @@ static int  Control         (stream_t *p_stream, int i_query, va_list args)
             /*TODO Support Seek */
             *(va_arg (args, bool *)) = SEEK;
             break;
+        case STREAM_CAN_PAUSE:
+        case STREAM_CAN_CONTROL_PACE:
+            *(va_arg (args, bool *)) = false; /* TODO */
+            break;
+
         case STREAM_GET_POSITION:
             *(va_arg (args, uint64_t *)) = p_sys->position;
             break;
         case STREAM_SET_POSITION:
-            return VLC_EGENERIC;
+        {
+            uint64_t pos = (uint64_t)va_arg(args, uint64_t);
+            if(Seek(p_stream, pos) == VLC_SUCCESS)
+            {
+                p_sys->position = pos;
+                break;
+            }
+            else
+                return VLC_EGENERIC;
+        }
         case STREAM_GET_SIZE:
+        {
+            uint64_t*   res = (va_arg (args, uint64_t *));
             if(p_sys->isLive)
-                *(va_arg (args, uint64_t *)) = 0;
+                *res = 0;
+            else
+            {
+                const dash::mpd::Representation *rep = p_sys->p_dashManager->getAdaptionLogic()->getCurrentRepresentation();
+                if ( rep == NULL )
+                    *res = 0;
+                else
+                    *res = p_sys->p_mpd->getDuration() * rep->getBandwidth() / 8;
+            }
             break;
+        }
         default:
             return VLC_EGENERIC;
     }
