@@ -4,7 +4,7 @@
  * Copyright (C) 2003-2004, 2010 the VideoLAN team
  * Copyright © 2007 Rémi Denis-Courmont
  *
- * $Id: 2f9569ae5c65a047b862e44bde525685862fe03e $
+ * $Id: da781594c532401379fcae1398d97d8cce8715ff $
  *
  * Authors: Laurent Aimar <fenrir@via.ecp.fr>
  *          Pierre Ynard
@@ -45,7 +45,7 @@
 #include <stdlib.h>
 #include <time.h>
 
-#ifndef WIN32
+#ifndef _WIN32
 # include <locale.h>
 #endif
 #ifdef HAVE_XLOCALE_H
@@ -125,8 +125,7 @@ rtsp_stream_t *RtspSetup( vlc_object_t *owner, vod_media_t *media,
     char *user = var_InheritString(owner, "sout-rtsp-user");
     char *pwd = var_InheritString(owner, "sout-rtsp-pwd");
 
-    rtsp->url = httpd_UrlNewUnique( rtsp->host, rtsp->psz_path,
-                                    user, pwd, NULL );
+    rtsp->url = httpd_UrlNew( rtsp->host, rtsp->psz_path, user, pwd );
     free(user);
     free(pwd);
     if( rtsp->url == NULL )
@@ -258,7 +257,7 @@ rtsp_stream_id_t *RtspAddId( rtsp_stream_t *rtsp, sout_stream_id_t *sid,
     char *user = var_InheritString(rtsp->owner, "sout-rtsp-user");
     char *pwd = var_InheritString(rtsp->owner, "sout-rtsp-pwd");
 
-    url = id->url = httpd_UrlNewUnique( rtsp->host, urlbuf, user, pwd, NULL );
+    url = id->url = httpd_UrlNew( rtsp->host, urlbuf, user, pwd );
     free( user );
     free( pwd );
     free( urlbuf );
@@ -423,7 +422,7 @@ static void RtspClientAlive( rtsp_session_t *session )
 static int dup_socket(int oldfd)
 {
     int newfd;
-#if !defined(WIN32) || defined(UNDER_CE)
+#ifndef _WIN32
     newfd = vlc_dup(oldfd);
 #else
     WSAPROTOCOL_INFO info;
