@@ -1,25 +1,25 @@
 /*****************************************************************************
  * rss.c : rss/atom feed display video plugin for vlc
  *****************************************************************************
- * Copyright (C) 2003-2006 the VideoLAN team
- * $Id: af00625e264abcac77605282028d1b010156cb79 $
+ * Copyright (C) 2003-2006 VLC authors and VideoLAN
+ * $Id: 63ea3b29659d31444329d189290b6081c4130a18 $
  *
  * Authors: Antoine Cellerier <dionoea -at- videolan -dot- org>
  *          Rémi Duraffort <ivoire -at- videolan -dot- org>
  *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * This program is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU Lesser General Public License as published by
+ * the Free Software Foundation; either version 2.1 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+ * GNU Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston MA 02110-1301, USA.
+ * You should have received a copy of the GNU Lesser General Public License
+ * along with this program; if not, write to the Free Software Foundation,
+ * Inc., 51 Franklin Street, Fifth Floor, Boston MA 02110-1301, USA.
  *****************************************************************************/
 
 /*****************************************************************************
@@ -211,6 +211,7 @@ vlc_module_begin ()
                   false )
         change_integer_list( pi_color_values, ppsz_color_descriptions )
     add_integer( CFG_PREFIX "size", -1, SIZE_TEXT, SIZE_LONGTEXT, false )
+        change_integer_range( -1, 4096)
 
     set_section( N_("Misc"), NULL )
     add_integer( CFG_PREFIX "speed", 100000, SPEED_TEXT, SPEED_LONGTEXT,
@@ -227,7 +228,7 @@ vlc_module_begin ()
 vlc_module_end ()
 
 static const char *const ppsz_filter_options[] = {
-    "urls", "x", "y", "position", "color", "size", "speed", "length",
+    "urls", "x", "y", "position", "opacity", "color", "size", "speed", "length",
     "ttl", "images", "title", NULL
 };
 
