@@ -2,7 +2,7 @@
  * update.c: VLC update checking and downloading
  *****************************************************************************
  * Copyright © 2005-2008 VLC authors and VideoLAN
- * $Id: cf80471476264a128eb03233d3a19592ecbee3e8 $
+ * $Id: 600e900262af2d6f6897f62ad5abe93ffc4e7ba3 $
  *
  * Authors: Antoine Cellerier <dionoea -at- videolan -dot- org>
  *          Rémi Duraffort <ivoire at via.ecp.fr>
@@ -52,7 +52,7 @@
 
 #include <gcrypt.h>
 #include <vlc_gcrypt.h>
-#ifdef WIN32
+#ifdef _WIN32
 #include <shellapi.h>
 #endif
 #include "update.h"
@@ -74,11 +74,9 @@
  * Remaining text is a required description of the update
  */
 
-#if defined( UNDER_CE )
-# define UPDATE_OS_SUFFIX "-ce"
-#elif defined( WIN64 )
+#if defined( _WIN64 )
 # define UPDATE_OS_SUFFIX "-win-x64"
-#elif defined( WIN32 )
+#elif defined( _WIN32 )
 # define UPDATE_OS_SUFFIX "-win-x86"
 #else
 # define UPDATE_OS_SUFFIX ""
@@ -143,7 +141,7 @@ void update_Delete( update_t *p_update )
 
     if( p_update->p_download )
     {
-        vlc_object_kill( p_update->p_download );
+        vlc_atomic_set( &p_update->p_download->aborted, 1 );
         vlc_join( p_update->p_download->thread, NULL );
         vlc_object_release( p_update->p_download );
     }
@@ -494,7 +492,7 @@ void update_Download( update_t *p_update, const char *psz_destdir )
     // If the object already exist, destroy it
     if( p_update->p_download )
     {
-        vlc_object_kill( p_update->p_download );
+        vlc_atomic_set( &p_update->p_download->aborted, 1 );
         vlc_join( p_update->p_download->thread, NULL );
         vlc_object_release( p_update->p_download );
     }
@@ -509,6 +507,7 @@ void update_Download( update_t *p_update, const char *psz_destdir )
     p_update->p_download = p_udt;
     p_udt->psz_destdir = psz_destdir ? strdup( psz_destdir ) : NULL;
 
+    vlc_atomic_set(&p_udt->aborted, 0);
     vlc_clone( &p_udt->thread, update_DownloadReal, p_udt, VLC_THREAD_PRIORITY_LOW );
 }
 
@@ -572,24 +571,24 @@ static void* update_DownloadReal( void *obj )
 
     /* Create a buffer and fill it with the downloaded file */
     p_buffer = (void *)malloc( 1 << 10 );
-    if( !p_buffer )
-    {
-        msg_Err( p_udt, "Can't malloc (1 << 10) bytes! download cancelled." );
+    if( unlikely(p_buffer == NULL) )
         goto end;
-    }
 
     msg_Dbg( p_udt, "Downloading Stream '%s'", p_update->release.psz_url );
 
     psz_size = size_str( l_size );
     if( asprintf( &psz_status, _("%s\nDownloading... %s/%s %.1f%% done"),
-        p_update->release.psz_url, "0.0", psz_size, 0.0 ) != -1 )
-    {
-        p_progress = dialog_ProgressCreate( p_udt, _( "Downloading ..."),
-                                            psz_status, _("Cancel") );
-        free( psz_status );
-    }
+        p_update->release.psz_url, "0.0", psz_size, 0.0 ) == -1 )
+        goto end;
 
-    while( vlc_object_alive( p_udt ) &&
+    p_progress = dialog_ProgressCreate( p_udt, _( "Downloading ..."),
+                                        psz_status, _("Cancel") );
+
+    free( psz_status );
+    if( p_progress == NULL )
+        goto end;
+
+    while( !vlc_atomic_get( &p_udt->aborted ) &&
            ( i_read = stream_Read( p_stream, p_buffer, 1 << 10 ) ) &&
            !dialog_ProgressCancelled( p_progress ) )
     {
@@ -617,7 +616,7 @@ static void* update_DownloadReal( void *obj )
     fclose( p_file );
     p_file = NULL;
 
-    if( vlc_object_alive( p_udt ) &&
+    if( !vlc_atomic_get( &p_udt->aborted ) &&
         !dialog_ProgressCancelled( p_progress ) )
     {
         dialog_ProgressDestroy( p_progress );
@@ -707,7 +706,7 @@ static void* update_DownloadReal( void *obj )
     msg_Info( p_udt, "%s authenticated", psz_destfile );
     free( p_hash );
 
-#ifdef WIN32
+#ifdef _WIN32
     int answer = dialog_Question( p_udt, _("Update VLC media player"),
     _("The new version was successfully downloaded. Do you want to close VLC and install it now?"),
     _("Install"), _("Cancel"), NULL);
