@@ -33,82 +33,35 @@
 
 #include "../config/vlc_getopt.h"
 
-#if !defined( UNDER_CE )
-#   include  <mmsystem.h>
-#endif
-
+#include <mmsystem.h>
 #include <winsock.h>
 
-/*****************************************************************************
- * system_Init: initialize winsock and misc other things.
- *****************************************************************************/
-void system_Init( void )
+
+static int system_InitWSA(int hi, int lo)
 {
-    WSADATA Data;
-    MEMORY_BASIC_INFORMATION mbi;
+    WSADATA data;
 
-    /* Get our full path */
-    char psz_path[MAX_PATH];
-    char *psz_vlc;
-
-    wchar_t psz_wpath[MAX_PATH];
-    if( VirtualQuery(system_Init, &mbi, sizeof(mbi) ) )
+    if (WSAStartup(MAKEWORD(hi, lo), &data) == 0)
     {
-        HMODULE hMod = (HMODULE) mbi.AllocationBase;
-        if( GetModuleFileName( hMod, psz_wpath, MAX_PATH ) )
-        {
-            WideCharToMultiByte( CP_UTF8, 0, psz_wpath, -1,
-                                psz_path, MAX_PATH, NULL, NULL );
-        }
-        else psz_path[0] = '\0';
+        if (LOBYTE(data.wVersion) == 2 && HIBYTE(data.wVersion) == 2)
+            return 0;
+        /* Winsock DLL is not usable */
+        WSACleanup( );
     }
-    else psz_path[0] = '\0';
+    return -1;
+}
 
-    psz_vlc = strrchr( psz_path, '\\' );
-    if( psz_vlc )
-        *psz_vlc = '\0';
-
-    {
-        /* remove trailing \.libs from executable dir path if seen,
-           we assume we are running vlc through libtool wrapper in build dir */
-        size_t len = strlen(psz_path);
-        if( len >= 5 && !stricmp(psz_path + len - 5, "\\.libs" ) )
-            psz_path[len - 5] = '\0';
-    }
-
-    psz_vlcpath = strdup( psz_path );
-
-    /* Set the default file-translation mode */
-#if !defined( UNDER_CE )
+/**
+ * Initializes MME timer, Winsock.
+ */
+void system_Init(void)
+{
+#if !VLC_WINSTORE_APP
     timeBeginPeriod(5);
 #endif
 
-    /* WinSock Library Init. */
-    if( !WSAStartup( MAKEWORD( 2, 2 ), &Data ) )
-    {
-        /* Aah, pretty useless check, we should always have Winsock 2.2
-         * since it appeared in Win98. */
-        if( LOBYTE( Data.wVersion ) != 2 || HIBYTE( Data.wVersion ) != 2 )
-            /* We could not find a suitable WinSock DLL. */
-            WSACleanup( );
-        else
-            /* Everything went ok. */
-            return;
-    }
-
-    /* Let's try with WinSock 1.1 */
-    if( !WSAStartup( MAKEWORD( 1, 1 ), &Data ) )
-    {
-        /* Confirm that the WinSock DLL supports 1.1.*/
-        if( LOBYTE( Data.wVersion ) != 1 || HIBYTE( Data.wVersion ) != 1 )
-            /* We could not find a suitable WinSock DLL. */
-            WSACleanup( );
-        else
-            /* Everything went ok. */
-            return;
-    }
-
-    fprintf( stderr, "error: can't initialize WinSocks\n" );
+    if (system_InitWSA(2, 2) && system_InitWSA(1, 1))
+        fputs("Error: cannot initialize Winsocks\n", stderr);
 }
 
 /*****************************************************************************
@@ -129,7 +82,7 @@ typedef struct
 
 void system_Configure( libvlc_int_t *p_this, int i_argc, const char *const ppsz_argv[] )
 {
-#if !defined( UNDER_CE )
+#if !VLC_WINSTORE_APP
     /* Raise default priority of the current process */
 #ifndef ABOVE_NORMAL_PRIORITY_CLASS
 #   define ABOVE_NORMAL_PRIORITY_CLASS 0x00008000
@@ -256,10 +209,10 @@ void system_Configure( libvlc_int_t *p_this, int i_argc, const char *const ppsz_
             exit( 0 );
         }
     }
-
 #endif
 }
 
+#if !VLC_WINSTORE_APP
 static unsigned __stdcall IPCHelperThread( void *data )
 {
     vlc_object_t *p_this = data;
@@ -339,8 +292,14 @@ LRESULT CALLBACK WMCOPYWNDPROC( HWND hwnd, UINT uMsg, WPARAM wParam,
                     i_options++;
                 }
 
-                char *psz_URI = make_URI( ppsz_argv[i_opt], NULL );
-                playlist_AddExt( p_playlist, psz_URI,
+#warning URI conversion must be done in calling process instead!
+                /* FIXME: This breaks relative paths if calling vlc.exe is
+                 * started from a different working directory. */
+                char *psz_URI = NULL;
+                if( strstr( ppsz_argv[i_opt], "://" ) == NULL )
+                    psz_URI = vlc_path2uri( ppsz_argv[i_opt], NULL );
+                playlist_AddExt( p_playlist,
+                        (psz_URI != NULL) ? psz_URI : ppsz_argv[i_opt],
                         NULL, PLAYLIST_APPEND |
                         ( ( i_opt || p_data->enqueue ) ? 0 : PLAYLIST_GO ),
                         PLAYLIST_END, -1,
@@ -359,17 +318,16 @@ LRESULT CALLBACK WMCOPYWNDPROC( HWND hwnd, UINT uMsg, WPARAM wParam,
 
     return DefWindowProc( hwnd, uMsg, wParam, lParam );
 }
+#endif
 
-/*****************************************************************************
- * system_End: terminate winsock.
- *****************************************************************************/
-void system_End( void )
+/**
+ * Cleans up after system_Init() and system_Configure().
+ */
+void system_End(void)
 {
+#if !VLC_WINSTORE_APP
     HWND ipcwindow;
 
-    free( psz_vlcpath );
-    psz_vlcpath = NULL;
-
     /* FIXME: thread-safety... */
     if (p_helper)
     {
@@ -381,9 +339,9 @@ void system_End( void )
         p_helper = NULL;
     }
 
-#if !defined( UNDER_CE )
     timeEndPeriod(5);
 #endif
 
+    /* XXX: In theory, we should not call this if WSAStartup() failed. */
     WSACleanup();
 }
